////////////////////////////////////////////////////////////////////////////////
//
//  The tribes2 default rotation only allows maps to be excluded from the
//  rotation.  Classic offers some more advantages over base, but not much.
//  EzRotation allows users to only add maps that they want.  When a map is
//  added it can be specified whether it is in the cycle or is just available
//  to be voted on.
//
//
//  EzRotation Customization Variables and Functions:
//  --------------------------------------------------------------------------
//
//  Set these variables somwhere in your main preferences file.
//
//  $Host::MapRotationFile = "prefs/EZ_Rotation.cs";
//
//    This defines a custom rotation file to use.  If no value is set
//    all missions are loaded.  If a value is set, but no file exists
//    then a default rotation file of the name is created with all
//    missions added to the cycle.  You can add and remove these at your
//    convenience.
//
//  $Host::RotationStyle = "Linear";
//
//    This defines the style of the custom rotation.  If no value is set
//    "Linear" is assumed.  Available options are:  Linear, Random.
//
//  Maps can be add to the server by using the function:
//
//    MapList.addMap (%file, %type, %cycle);
//
//
//  Credits:
//     - the writers of bwadmin
//     - some ideas from Evolution admin mod
//
/////////////////////////////////////////////////////////////////////////////////
//
//  Credits:
//
//  - Some code borrowed from bwadmin (Crunchy)
//  - Some ideas borrowed from Evolution (Kater)
//
/////////////////////////////////////////////////////////////////////////////////
//
//  Todo:
//
//  1. Do we want to add a mixed option to the custom rotation to allow maps
//     from different types to be in the same cycle?
//
//  2. Do we want to add min and max players to maps added to the rotation?
//
//  3. Add random rotation type.
//
//
/////////////////////////////////////////////////////////////////////////////////
//
//  History:
//
//  Revision 1.1 03/14/2004 sonic9k
//    Initial revision
//
//  
/////////////////////////////////////////////////////////////////////////////////
//
//

// List of potential functions that may need to be overwritten:
// 
// GameGui.cs
//   - GM_HostPane::onActivate (%this);
//   - buildMissionList();
//   - validateMissionAndType (%misName, %misType);
//   - getNextMission (%misName, %misType);
//   - GMH_MissionType::onSelect (%this, %id, %text);
//   - StartHostedGame();
//
// defaultGame.cs
//   - DefaultGame::voteChangeMission (%game, %admin, %missionDisplayName, %typeDisplayName, %missionId, %missionTypeId);
//   - DefaultGame::voteTournamentMode (%game, %admin, %missionDisplayName, %typeDisplayName, %missionId, %missionTypeId);
//
// server.cs
//   - CreateServer (%mission, %missionType);
//   - initGameBots (%mission, %mType)
//   - findNextCycleMission();
//   - serverCmdGetMissionList (%client, %key, %type);
//   - cycleMissions();
//   - loadMission (%missionName, %missionType, %firstMission);
//   



////////////////////////////////////////////////////////////////////////////////
// Create the MapList object
//
if (!isObject (MapList))
{
	new ScriptObject (MapList)
	{
		class = MapList;

		cycle = "";

		currentMapName = "";
		currentMapType = "";
	};

}

////////////////////////////////////////////////////////////////////////////////
// Returns the index of the type from $HostTypeName
//
function MapList::getTypeIndex (%this, %type)
{
	for (%i = 0; %i < $HostTypeCount; %i++)
		if ($HostTypeName[%i] $= %type)
			return %i;
	return -1;
}

////////////////////////////////////////////////////////////////////////////////
// Returns the index of the map from $HostMissionFile.  $HostMissionFile
// contains all of the maps in no particular order.  $HostMission orders
// the maps based on game type.
//
function MapList::getFileIndex (%this, %file, %type)
{
	for (%i = 0; %i < $HostMissionCount; %i++)
		if ($HostMissionFile[%i] $= %file)
			return %i;
	return -1;
}

////////////////////////////////////////////////////////////////////////////////
// Read the mission file to grab the display name.  If no display name
// found the file name is used.
//
function MapList::getDisplayName (%this, %file)
{
	%fd = new FileObject();
	if (!%fd.openForRead ("missions/"@%file@".mis"))
	{
		error ("Unable to find map missions/"@%file);
		return %file;
	}

	// Read entire file and look for display name
	while (!%fd.isEOF())
	{
		%line = %fd.readLine();
		if (getSubStr (%line, 0, 17) $= "// DisplayName = ")
		{
			%name = getSubStr (%line, 17, 1000);
			break;
		}
	}

	%fd.close();
	%fd.delete();
	return (%name $= "") ? %file : %name;
}

////////////////////////////////////////////////////////////////////////////////
// Debugging function for dumping a map list
//
function MapList::dumpList (%this, %list)
{
	for (%i = 0; %i < getWordCount (%list); %i++)
	{
		%fileIndex = getWord (%list, %i);
		%file = $HostMissionFile[%fileIndex];
		%name = $HostMissionName[%fileIndex];


		// Print the map file name
		echo (%i@":  file = "@%file@", name = "@%name);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Finc the index of the file and type in the cycle.  If not found then
// -1 is returned.
//
function MapList::getIndexInCycle (%this, %file, %type)
{
	// Find the index of the type in the master type list
	if ((%typeIndex = %this.getTypeIndex (%type)) == -1)
	{
		return -1;
	}

	// Find the index of the map in the master map list
	if ((%fileIndex = %this.getFileIndex (%file, %type)) == -1)
	{
		return -1;
	}

	// Locate the fileIndex in the cycle list
	%count = getWordCount (%this.cycle[%type]);

	for (%i = 0; %i < %count; %i++)
	{
		if ((%word = getWord (%this.cycle[%type], %i)) == %fileIndex)
			return %i;
	}

	return -1;
}



////////////////////////////////////////////////////////////////////////////////
// Adds a file and game type to the map list.  If %cycle is set then the map
// is added to the cycle, otherwise it is available to be voted on but not in
// the cycle.
//
function MapList::addMap (%this, %file, %type, %cycle)
{
	// See if the game type exists, if not add it
	if ((%typeIndex = %this.getTypeIndex (%type)) == -1)
	{
		$HostTypeName[$HostTypeCount] = %type;
		$HostMissionCount[$HostTypeCount] = 0;
		%typeIndex = $HostTypeCount;
		$HostTypeCount++;
	}

	// See if the file already exists, if not add it
	if ((%fileIndex = %this.getFileIndex (%file, %type)) == -1)
	{
		%name = %this.getDisplayName(%file);

		// Add the new map to the array
		$HostMissionFile[$HostMissionCount] = %file;
		$HostMissionName[$HostMissionCount] = %name;
		%fileIndex = $HostMissionCount;
		$HostMissionCount++;

		// See if bots are enabled for the map
		$BotEnabled[%fileIndex] = isFile ("terrains/"@%file@".nav");
	}

	// Add an entry for the mission in the main list
	%misCount = $HostMissionCount[%typeIndex];
	$HostMission[%typeIndex, %misCount] = %fileIndex;
	$HostMissionCount[%typeIndex]++;

	// Add map to the cycle
	if (%cycle)
	{
		%this.cycle[%type] = (%this.cycle[%type] $= "") ? %fileIndex :
			%this.cycle[%type] SPC %fileIndex;
	}

	// Should we also allow min and max players per map?
	//$Host::MapPlayerLimits[%file, %type] = %min SPC %max;
}

////////////////////////////////////////////////////////////////////////////////
// Returns the next map in the cycle.  If the end of the cycle has been the
// cycle is started over again.  If a map cannot be found in the cycle the
// the first map in the cycle is used. 
//
function MapList::nextInCycle (%this, %file, %type)
{
	//echo ("Here 11 - file = "@%file@", type = "@%type@", this.cycle["@%type@"] = "@%this.cycle[%type]);


	// If we couldn't find map in cycle then use the first map in cycle
	if ((%cycleIndex = %this.getIndexInCycle (%file, %type)) == -1)
		%cycleIndex = 0;
	else
	{
		// Need count to see if we have reached end or not
		%cycleCount = getWordCount (%this.cycle[%type]);

		//echo ("Here 11.1 - cycleIndex = "@%cycleIndex@", cycleCount = "@%cycleCount);

		// If the current cycle map is at the end then start over from
		// the beginning of the rotation.
		if (%cycleIndex + 1 >= %cycleCount)
			%nextCycleIndex = 0;
		else
			%nextCycleIndex = %cycleIndex + 1;
	}

	%nextFileIndex = getWord (%this.cycle[%type], %nextCycleIndex);
	//echo ("Here 11.2 - nextFileIndex = "@%nextFileIndex);
	return %nextFileIndex;
}

function MapList::setCycleType (%this, %type)
{
	// Random, Mixed, All, Alpha
	%this.cycleType = %type;
}

////////////////////////////////////////////////////////////////////////////////
// Resets the global vars that hold the missions and types.
//
function MapList::reset (%this)
{
	deleteVariables ("$HostMission*");
	deleteVariables ("$HostType*");
	$HostMissionCount = 0;
	$HostTypeCount = 0;
}

function printHostMissionFile()
{
	for (%i = 0; %i < $HostMissionCount; %i++)
	{
		echo (%i@":  file = "@$HostMissionFile[%i]@", name = "@$HostmissionName[%i]);
	}
}

function printHostMission()
{
	//echo ("Here 1 - $HostTypeCount = "@$HostTypeCount);

	for (%i = 0; %i < $HostTypeCount; %i++)
	{
		%type = $HostTypeName[%i];
		%count = $HostMissionCount[%i];

		// count is zero here for some reason

		//echo ("Here 2 - type = "@%type@", count = "@%count);

		for (%j = 0; %j < %count; %j++)
		{
			%fileIndex = $HostMission[%i, %j];
			//echo ("Here 3 - fileIndex = "@%fileIndex);

			echo (%i@","@%j@","@%fileIndex@":  type = "@%type@", file = "@$HostMissionFile[%fileIndex]@", name = "@$HostmissionName[%fileIndex]);
		}
	}
}

function printHostTypes()
{
	for (%i = 0; %i < $HostTypeCount; %i++)
	{
		echo (%i@":  "@$HostTypeName[%i]);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Creates a custom rotation file with all maps on the server.  All maps are
// added to the cycle.  Modified from bwadmin.
//
function EzCreateRotationFile (%rotFname)
{
	%search = "missions/*.mis";
	%fd = new FileObject();
	%typeCount = 0;

	// Loop through all of the missions
	for (%file = findFirstFile (%search);
	     %file !$= "";
	     %file = findNextFile (%search))
	{
		%name = fileBase (%file);

		if (!%fd.openForRead (%file)) continue;
		%typeList = "None";

		// Grab list of valid types for this map
		while (!%fd.isEOF ())
		{
			%line = %fd.readLine();
			if (getSubStr (%line, 0, 18) $= "// MissionTypes = ")
			{
				%typeList = getSubStr (%line, 18, 1000);
				break;
			}
		}

		%fd.close();

		// Do not include singleplayer missions
		if (strstr (%typeList, "SinglePlayer") != -1) continue;

		for (%word = 0;
		     (%type = getWord(%typeList, %word)) !$= "";
		     %word++)
		{
			// If type == CTF then add PracticeCTF and SCtF as well


			for (%i = 0; %i < %typeCount; %i++)
				if (%typeName[%i] $= %type)
					break;
			if (%i == %typeCount)
			{
				%typeCount++;
				%typeName[%i] = %type;
				%missionCount[%i] = 0;
			}

			%m = %missionCount[%i];
			%mission[%i, %m] = %name;
			%missionCount[%i]++;
		}
	}

	// Write out the rotation file
	%fd.openForWrite (%rotFname);

	for (%i = 0; %i < %typeCount; %i++)
	{
		%type = %typeName[%i];
		%fd.writeLine("");
		%fd.writeLine("//");
		%fd.writeLine("// --- "@%type@" ---");
		%fd.writeLine("//");

		for (%j = 0; %j < %missionCount[%i]; %j++)
		{
			%file = %mission[%i, %j];
			%text = "MapList.add ("@%file@", "@%type@", 1);";
			%fd.writeLine (%text);
		}
	}

	// Close and delete file object
	%fd.close();
	%fd.delete();
}

//////////////////////////////////////////////////////////////////////////////


package EzRotation {

	// 
	// Overrides default function to provide custom behavior for
	// choosing the next map
	// 
	function getNextMission (%file, %type)
	{
		if ($Host::MapRotationFile !$= "")
		{
			%nextFileIndex = MapList.nextInCycle (%file, %type);
			return %nextFileIndex;
		}
		else
			parent::getNextMission (%file, %type);
	}

	//
	// Overrides default function to build custom rotation
	//
	function buildMIssionList ()
	{
		// Setup some default values for globals.  If we have set the
		// rotation file attempt to try and create it if it doesn't
		// exist.  If the value is not set then assume the normal behavior
		// of loading all maps in all map packs.
		if ($Host::MapRotationFile !$= "")
		{
			if (!isFile($Host::MapRotationFile))
			{
				EzCreateRotationFile($Host::MapRotationFile);
			}

			if ($Host::RotationStyle $= "")
			{
				$Host::RotationStyle = "Linear";
			}
		}

		if (isFile ($Host::MapRotationFile))
		{
			MapList.reset();
			exec ($Host::MapRotationFile);
			getMissionTypeDisplayNames();
		}
		else
			parent::buildMissionList();
	}

	// Check if valid FFA map
	//function loadMission ()
	//{

	//}

	//function CycleMissions ()
	//{

	//}

	//function findNextCycleMission()
	//{

	//}
};


activatePackage (EzRotation);

