//------------------------------
//TASKS AND OBJECTIVES

//AIO Attack Location
//AIO Attack Object - teleport true
//AIO Attack Player
//AIO Defend Location - teleport true
//AIO Defense Patrol - teleport true
//AIO Deploy Equipment - teleport true
//AIO Deploy Vehicle
//AIO Destroy Object - teleport true
//AIO Escort Player
//AIO Laze Object
//AIO MAttack Location
//AIO MDestroy Object - teleport true
//AIO Missile Object - teleport true
//AIO Mortar Object - teleport true, requests escort
//AIO Place Camera
//AIO Repair Object - teleport true
//AIO Touch Object - teleport true, CTF requests escort, sniper, artillery (mortar)

//--------------------------------------------------------------------------------------------------- AIO Attack Location ---

function AIOAttackLocation::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }
      
   //now, if this bot is linked to a human who has issued this command, up the weight
   if (%this.issuedByClientId == %client.controlByHuman)
   {
      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
      {
         if ($AIWeightHumanIssuedCommand < %minWeight)
	    return 0;
	 else
	    %weight = $AIWeightHumanIssuedCommand;
      }
      else
      {
         // calculate the default...
	 %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	 if (%weight < $AIWeightHumanIssuedCommand)
	    %weight = $AIWeightHumanIssuedCommand;
      }
   }
   else
   {
      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
         return 0;

      // calculate the default...
      %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
   }

   //add or decrease weight if the objective has inBase or inRoom fields - roberto
   //------------------------------------------------------------------------------
   if (%this.inBase)
   {
      if (%client.inBase)
      {
         if (%weight <= 4000) { %weight += 100; }
      }
   }

   if (%this.inRoom)
   {
      if (%client.inRoom)
      {
         if (%weight <= 4000) { %weight += 200; }
      }
   }
   //------------------------------------------------------------------------------

   return %weight;
}

function AIOAttackLocation::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIAttackLocation);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIOAttackLocation::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//----------------------------------------------------------------------------------------------------- AI Attack Location ---
//ai.getLOSLocation(targetPoint [, minDist, maxDist, nearPoint])

function AIAttackLocation::initFromObjective(%task, %objective, %client)
{
   //initialize the task vars from the objective
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.location = %objective.location;
   %task.equipment = %objective.equipment;
   %task.buyEquipmentSet = %objective.buyEquipmentSet;
   %task.desiredEquipment = %objective.desiredEquipment;
   %task.issuedByClient = %objective.issuedByClientId;
   %task.chat = %objective.chat;

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //initialize other task vars
   %task.sendMsg = true;
   %task.sendMsgTime = 0;
   %task.engageTarget = -1;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;
}

function AIAttackLocation::assume(%task, %client)
{
   %task.setWeightFreq(30);
   %task.setMonitorFreq(30);

   if (%task.equipment !$= "")
      %equipmentList = %task.equipment;
   else
      %equipmentList = %task.desiredEquipment;
   %client.needEquipment = AINeedEquipment(%equipmentList, %client);

   //even if we don't *need* equipemnt, see if we should buy some... 
   if (! %client.needEquipment && %task.buyEquipmentSet !$= "")
   {
      //see if we could benefit from inventory
      %needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
      %result = AIFindClosestInventory(%client, %needArmor);
      %closestInv = getWord(%result, 0);
      %closestDist = getWord(%result, 1);
      if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
      {
         %result = AIFindClosestEnemyToLoc(%client, %task.location, 50, $AIClientLOSTimeout);
	 %closestEnemy = getWord(%result, 0);

         if (%closestEnemy <= 0 && %closestDist < 100)
            %client.needEquipment = true;
      }
   }

   //-------------- added extra - Lagg...
   if (%client.needEquipment && %task.equipment $= "")
   {
      if (%client.player.getInventory(SniperRifle) > 0 && %client.player.getInventory(EnergyPack) > 0)
         %client.needEquipment = false;

      %hasMortar = (%client.player.getInventory("Mortar") > 0) && (%client.player.getInventory("MortarAmmo") > 0);
      %hasDisc = (%client.player.getInventory("Disc") > 0) && (%client.player.getInventory("DiscAmmo") > 0) && (%client.player.getInventory("Blaster") <= 0);
      if (%hasMortar || %hasDisc)
         %client.needEquipment = false;
   }      

   //mark the current time for the buy inventory state machine
   %task.buyInvTime = getSimTime();

   //initialize the sniper varibles
   %task.snipeLocation = "";
   %task.hideLocation = "";
   %task.moveToPosition = true;
   %task.moveToSnipe = false;
   %task.nextSnipeTime = 0;
}

function AIAttackLocation::retire(%task, %client)
{
   //do nothing here
}

function AIAttackLocation::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;

   //if we're a sniper, we're going to cheat, and see if there are clients near the attack location
   %losTimeout = $AIClientMinLOSTime + ($AIClientLOSTimeout * %client.getSkillLevel());
   %distToLoc = VectorDist(%client.player.getWorldBoxCenter(), %task.location);

   //if we have sniper gear (cheat ignores LOS)...
   if (((%client.player.getInventory(SniperRifle) > 0 && %client.player.getInventory(EnergyPack) > 0) ||
     (%client.player.getInventory(Mortar) > 0 && %client.player.getInventory(MortarAmmo) > 0)) && %distToLoc > 60)
      %result = AIFindClosestEnemyToLoc(%client, %task.location, 100, $AIClientLOSTimeout, true);

   //otherwise, do the search normally.
   else
      %result = AIFindClosestEnemyToLoc(%client, %task.location, 75, %losTimeout, false);

   %closestEnemy = getWord(%result, 0);
   %closestdist = getWord(%result, 1);
   %task.setWeight(%task.baseWeight);
   
   //see if we found someone
   if (%closestEnemy > 0)
      %task.engageTarget = %closestEnemy;
   else
      %task.engageTarget = -1;
}

function AIAttackLocation::monitor(%task, %client)
{
   //first, buy the equipment - if we really really need some :) - Lagg... - 4-6-2003
   %hasMortar = (%client.player.getInventory("Mortar") > 0 && %client.player.getInventory("MortarAmmo") > 0);
   %hasSniper = (%client.player.getInventory("SniperRifle") > 0 && %client.player.getInventory("EnergyPack") > 0);

   if (%hasMortar || %hasSniper)
   {
      %client.needEquipment = false;
   }


   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
    %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(30);
    %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
    //if this task is the objective task, choose a new objective
    if (%task == %client.objectiveTask)
    {
       AIUnassignClient(%client);
       Game.AIChooseGameObjective(%client);
    }
    return;
      }
   }

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();

   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
    if (%client.isAIControlled())
    {
       if (%task.chat !$= "")
       {
          %chatMsg = getWord(%task.chat, 0);
          %chatTemplate = getWord(%task.chat, 1);
          if (%chatTemplate !$= "")
             AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
          else
        AIMessageThread(%task.chat, %client, -1);
       }
       else
          AIMessageThreadTemplate("AttackBase", "ChatSelfAttack", %client, -1);
    }
      }
   }

   //how far are we from the location we're attacking
   %myPos = %client.player.getWorldBoxCenter();
   %distance = %client.getPathDistance(%task.location);
   if (%distance < 0)
   {
      %distance = 32767;
   }

   if (%hasSniper)
   {
      //first, find an LOS location
      if (%task.snipeLocation $= "")
      {
         %task.snipeLocation = %client.getLOSLocation(%task.location, 150, 450);
         %task.hideLocation = %client.getHideLocation(%task.location, VectorDist(%task.location, %task.snipeLocation), %task.snipeLocation, 1); 

         //We are not near distance to hide, see if we must take longer routes to it or not - roberto
         //Divert attacker path - Roberto
         //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
         //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.
         %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));

         if (%DivertPathExists && %distToHide > 325) //must be further than 325m from hide location to go to a divert location
         {
            //Pick a location to move to if we don't have one
            if (%client.divertPosition $= "")
            {
               %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
               %locCount = %DivertLocations.getCount();

               //get the marker that is closest to the hide position
               for (%i = 0; %i < %locCount; %i++)
               {
                  %currMarker = %DivertLocations.getObject(%i);
                  %markerToHideDist = VectorDist(%currMarker.position, %task.hidelocation);

                  //if that marker from DivertLocations is closer to the hide location than we are this is our marker
                  if (%markerToHideDist < %distToHide)
                  {
                     %client.divertPosition = %currMarker.position;
                     break;
                  }
               }

               //If we still haven't found a close marker just go to the hide location
               if (%client.divertPosition $= "")
               {
                  %client.divertPosition = %task.hidelocation;
               }
            }
            else //we have a divert position, is it still closer to our desired location than we are?
            {
               %divertCanContinue = ( VectorDist(%client.divertPosition, %task.hidelocation) < %distToHide );

               //We are way off track, abort divert and move right to desired location
               if (!%divertCanContinue) 
               {
                  %client.divertPosition = %task.hidelocation;
               }
            }

            //By now we should have a marker as destination or the hide location, Mooooooo!
            %client.setTargetObject(-1);

            //don't move like a maniac when near lava - roberto
            if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
            else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
         }
         //NO DIVERTLOCATIONS SIMGROUP or we are close to hide location, just move near hide location
         else
         {
            //don't move like a maniac when near lava - roberto
            if (%client.inDanger || %distToHide < 10) { %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk); }
            else { %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); }

            %client.divertposition = ""; //stop diverting our path
         }

         %task.moveToPosition = true;
      }
      else
      {
         %energy = %client.player.getEnergyPercent();
         %distToSnipe = VectorDist(%task.snipelocation, %client.player.getWorldBoxCenter());
         %distToHide = VectorDist(%task.hidelocation, %client.player.getWorldBoxCenter());

         //until we're in position, we can move using the AIModeExpress, after that, we only want to walk...
         if (%task.moveToPosition)
         {
            if (%distToHide < 4.0)
            {
               //dissolve the human control link
               if (%task == %client.objectiveTask)
               {
                  if (aiHumanHasControl(%task.issuedByClient, %client))
                  {
                     aiReleaseHumanControl(%client.controlByHuman, %client);
                     //should re-evaluate the current objective weight
                     %inventoryStr = AIFindClosestInventories(%client);
                     %client.objectiveWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
                  }
               }

               %task.moveToPosition = false;
            }
            else
            {
               //We are not near distance to hide, see if we must take longer routes to it or not - roberto
               //Divert attacker path - Roberto
               //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
               //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.
               %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));

               if (%DivertPathExists && %distToHide > 325) //must be further than 325m from hide location to go to a divert location
               {
                  //Pick a location to move to if we don't have one
                  if (%client.divertPosition $= "")
                  {
                     %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
                     %locCount = %DivertLocations.getCount();

                     //get the marker that is closest to the hide position
                     for (%i = 0; %i < %locCount; %i++)
                     {
                        %currMarker = %DivertLocations.getObject(%i);
                        %markerToHideDist = VectorDist(%currMarker.position, %task.hidelocation);

                        //if that marker from DivertLocations is closer to the hide location than we are this is our marker
                        if (%markerToHideDist < %distToHide)
                        {
                           %client.divertPosition = %currMarker.position;
                           break;
                        }
                     }

                     //If we still haven't found a close marker just go to the hide location
                     if (%client.divertPosition $= "")
                     {
                        %client.divertPosition = %task.hidelocation;
                     }
                  }
                  else //we have a divert position, is it still closer to our desired location than we are?
                  {
                     %divertCanContinue = ( VectorDist(%client.divertPosition, %task.hidelocation) < %distToHide );

                     //We are way off track, abort divert and move right to desired location
                     if (!%divertCanContinue) 
                     {
                        %client.divertPosition = %task.hidelocation;
                     }
                  }

                  //By now we should have a marker as destination or the hide location, Mooooooo!
                  %client.setTargetObject(-1);

                  //don't move like a maniac when near lava - roberto
                  if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
                  else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
               }
               //NO DIVERTLOCATIONS SIMGROUP or we are close to hide location, just move near hide location
               else
               {
                  //don't move like a maniac when near lava - roberto
                  if (%client.inDanger || %distToHide < 10) { %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk); }
                  else { %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); }

                  %client.divertposition = ""; //stop diverting our path
               }
            }
         }
         else if (%task.moveToSnipe)
         {
            if (%energy > 0.75 && %client.getStepStatus() $= "Finished")
            {
               %client.stepMove(%task.snipeLocation, 4.0, $AIModeWalk);

               if (%task.engageTarget > 0)
               {
                  %client.setEngageTarget(%task.engageTarget);
               }
               else
               {
                  %task.moveToSnipe = false;
               }
            }
            else if (%energy < 0.4)
            {
                %client.setEngageTarget(-1);
                %client.stepMove(%task.hideLocation, 4.0);
                %task.nextSnipeTime = getSimTime() + 1000 + (getRandom() * 4000);
                %task.moveToSnipe = false;
            }
         }
         else if (%energy > 0.5 && getSimTime() > %task.nextSnipeTime)
         { 
            if (%task.engageTarget <= 0)
            {
               //%client.aimAt(%task.location, 8000);
               %targetNearLocation = AIFindClosestEnemyToLoc(%client, %task.snipeLocation, 450, $AIClientLOSTimeout, false); //last argument specifies that it doesn't ignore LOS
               %closestEnemyToLoc = getWord(%targetNearLocation, 0);

               if (AIClientIsAlive(%closestEnemyToLoc))
               {
                  %task.engageTarget = %closestEnemyToLoc;
                  %client.setEngageTarget(%task.engageTarget);
                  %client.stepRangeObject(%task.engageTarget.player.getWorldBoxCenter(), "BasicSniperShot", 150, 250, %task.snipelocation);
                  %client.aimAt(%task.engageTarget.player.getWorldBoxCenter(), 8000);
                  %task.moveToSnipe = true;
               }
            }
            else if (AIClientIsAlive(%task.engageTarget))
            {
               %client.setEngageTarget(%task.engageTarget);
               %client.stepRangeObject(%task.engageTarget.player.getWorldBoxCenter(), "BasicSniperShot", 150, 250, %task.snipelocation);
               %client.aimAt(%task.engageTarget.player.getWorldBoxCenter(), 8000);
               %task.moveToSnipe = true;
            }
         }
         else
         {
            if (%client.inDanger || %distToHide < 15.0) 
            { 
               %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk);
            }
            else 
            { 
               %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); 
            }
         }
      }
   }
   //---------------------------------------------------------------------------------------------------------------------
   //see if we can mortar spam somebody
   else if (%hasMortar)
   {
      //first, find an LOS location
      if (%task.snipeLocation $= "")
      {

         %task.snipeLocation = %client.getLOSLocation(%task.location, 150, 350);
         %task.hideLocation = %client.getHideLocation(%task.location, VectorDist(%task.location, %task.snipeLocation), %task.snipeLocation, 1); 

         //We are not near distance to hide, see if we must take longer routes to it or not - roberto
         //Divert attacker path - Roberto
         //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
         //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.
         %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));

         if (%DivertPathExists && %distToHide > 325) //must be further than 325m from hide location to go to a divert location
         {
            //Pick a location to move to if we don't have one
            if (%client.divertPosition $= "")
            {
               %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
               %locCount = %DivertLocations.getCount();

               //get the marker that is closest to the hide position
               for (%i = 0; %i < %locCount; %i++)
               {
                  %currMarker = %DivertLocations.getObject(%i);
                  %markerToHideDist = VectorDist(%currMarker.position, %task.hidelocation);

                  //if that marker from DivertLocations is closer to the hide location than we are this is our marker
                  if (%markerToHideDist < %distToHide)
                  {
                     %client.divertPosition = %currMarker.position;
                     break;
                  }
               }

               //If we still haven't found a close marker just go to the hide location
               if (%client.divertPosition $= "")
               {
                  %client.divertPosition = %task.hidelocation;
               }
            }
            else //we have a divert position, is it still closer to our desired location than we are?
            {
               %divertCanContinue = ( VectorDist(%client.divertPosition, %task.hidelocation) < %distToHide );

               //We are way off track, abort divert and move right to desired location
               if (!%divertCanContinue) 
               {
                  %client.divertPosition = %task.hidelocation;
               }
            }

            //By now we should have a marker as destination or the hide location, Mooooooo!
            %client.setTargetObject(-1);

            //don't move like a maniac when near lava - roberto
            if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
            else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
         }
         //NO DIVERTLOCATIONS SIMGROUP or we are close to hide location, just move near hide location
         else
         {
            //don't move like a maniac when near lava - roberto
            if (%client.inDanger || %distToHide < 10) { %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk); }
            else { %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); }

            %client.divertposition = ""; //stop diverting our path
         }

         %task.moveToPosition = true;
      }
      else
      {
         %energy = %client.player.getEnergyPercent();
         %distToSnipe = VectorDist(%task.snipelocation, %client.player.getWorldBoxCenter());
         %distToHide = VectorDist(%task.hidelocation, %client.player.getWorldBoxCenter());

         //until we're in position, we can move using the AIModeExpress, after that, we only want to walk...
         if (%task.moveToPosition)
         {
            if (%distToHide < 4.0)
            {
               //dissolve the human control link
               if (%task == %client.objectiveTask)
               {
                  if (aiHumanHasControl(%task.issuedByClient, %client))
                  {
                     aiReleaseHumanControl(%client.controlByHuman, %client);
                     //should re-evaluate the current objective weight
                     %inventoryStr = AIFindClosestInventories(%client);
                     %client.objectiveWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
                  }
               }

               %task.moveToPosition = false;
            }
            else
            {
               //We are not near distance to hide, see if we must take longer routes to it or not - roberto
               //Divert attacker path - Roberto
               //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
               //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.
               %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));

               if (%DivertPathExists && %distToHide > 325) //must be further than 325m from hide location to go to a divert location
               {
                  //Pick a location to move to if we don't have one
                  if (%client.divertPosition $= "")
                  {
                     %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
                     %locCount = %DivertLocations.getCount();

                     //get the marker that is closest to the hide position
                     for (%i = 0; %i < %locCount; %i++)
                     {
                        %currMarker = %DivertLocations.getObject(%i);
                        %markerToHideDist = VectorDist(%currMarker.position, %task.hidelocation);

                        //if that marker from DivertLocations is closer to the hide location than we are this is our marker
                        if (%markerToHideDist < %distToHide)
                        {
                           %client.divertPosition = %currMarker.position;
                           break;
                        }
                     }

                     //If we still haven't found a close marker just go to the hide location
                     if (%client.divertPosition $= "")
                     {
                        %client.divertPosition = %task.hidelocation;
                     }
                  }
                  else //we have a divert position, is it still closer to our desired location than we are?
                  {
                     %divertCanContinue = ( VectorDist(%client.divertPosition, %task.hidelocation) < %distToHide );

                     //We are way off track, abort divert and move right to desired location
                     if (!%divertCanContinue) 
                     {
                        %client.divertPosition = %task.hidelocation;
                     }
                  }

                  //By now we should have a marker as destination or the hide location, Mooooooo!
                  %client.setTargetObject(-1);

                  //don't move like a maniac when near lava - roberto
                  if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
                  else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
               }
               //NO DIVERTLOCATIONS SIMGROUP or we are close to hide location, just move near hide location
               else
               {
                  //don't move like a maniac when near lava - roberto
                  if (%client.inDanger || %distToHide < 10) { %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk); }
                  else { %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); }

                  %client.divertposition = ""; //stop diverting our path
               }
            }
         }
         else if (%task.moveToSnipe)
         {
            if (%energy > 0.75 && %client.getStepStatus() $= "Finished")
            {
               %client.stepMove(%task.snipeLocation, 4.0, $AIModeWalk);

               if (%task.engageTarget > 0)
               {
                  %client.setEngageTarget(%task.engageTarget);
               }
               else
               {
                  %task.moveToSnipe = false;
               }
            }
            else if (%energy < 0.4)
            {
                %client.setEngageTarget(-1);
                %client.stepMove(%task.hideLocation, 4.0);
                %task.nextSnipeTime = getSimTime() + 1000 + (getRandom() * 4000);
                %task.moveToSnipe = false;
            }
         }
         //if we have a repair pack and we're fully healed and the shooting delay passed, or if the shooting delay passed and we don't have a repair pack
         else if ( ((%client.player.getDamagePercent() <= 0 && %client.player.getInventory(RepairPack) > 0) || (%client.player.getInventory(RepairPack) <= 0)) && getSimTime() > %task.nextSnipeTime)
         { 
            if (%task.engageTarget <= 0)
            {
               //%client.aimAt(%task.location, 8000);
               %targetNearLocation = AIFindClosestEnemyToLoc(%client, %task.snipeLocation, 450, $AIClientLOSTimeout, false); //last argument specifies that it doesn't ignore LOS
               %closestEnemyToLoc = getWord(%targetNearLocation, 0);

               if (AIClientIsAlive(%closestEnemyToLoc))
               {
                  %task.engageTarget = %closestEnemyToLoc;
                  %client.setEngageTarget(%task.engageTarget);
                  %client.stepRangeObject(%task.engageTarget.player.getWorldBoxCenter(), "MortarShot", 40, 400, %task.snipelocation);
                  //%client.aimAt(%task.engageTarget.player.getWorldBoxCenter(), 1000);
                  %task.moveToSnipe = true;
               }
            }
            else if (AIClientIsAlive(%task.engageTarget))
            {
               %client.setEngageTarget(%task.engageTarget);
               %client.stepRangeObject(%task.engageTarget.player.getWorldBoxCenter(), "MortarShot", 40, 400, %task.snipelocation);
               //%client.aimAt(%task.engageTarget.player.getWorldBoxCenter(), 1000);
               %task.moveToSnipe = true;
            }
         }
         else
         {
            if (%client.inDanger || %distToHide < 10.0) 
            { 
               %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk);
            }
            else 
            { 
               %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); 
            }

            //if we're not fighting a player aim at the floor to repair ourselves if we have a repair pack 
            if ( (%client.getEngageTarget() <= 0) && ((%client.player.getDamagePercent() > 0) && (%client.player.getInventory(RepairPack) > 0)) )
            {
               %pos = %client.player.getWorldBoxCenter();
               %posX = getWord(%pos, 0);
               %posY = getWord(%pos, 1);
               %posZ = getWord(%pos, 2) - 1;
               %pos = %posX SPC %posY SPC %posZ;
               %client.aimAt(%pos, 500);
            }
         }
      }
   }
   //----------------------------------------------------------------------------------------------------------------

   //see if we're supposed to be engaging anyone...
   if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
   {
      %client.setEngageTarget(%client.shouldEngage);
   }
}

//returns the number of powered assets of a type from a team - roberto
//if %percentage is true then the number of powered assets from a total count will be returned
// returns -1 if that team doesn't have that asset if percentage is enabled or not

//if you use this in a mission (ctf), then the generators, turrets, sentry turrets, sensors must be named like this:
//TxBaseGeneratorx, TxBaseTurretx, TxSentryTurretx, TxSensorx. Where the first X is the team number (1 or 2) and the second X is the count of that object starting with 1.
function poweredAssetCount(%type, %team, %percentage) 
{
	%poweredAssets = 0;
	%totalAssets = 0;

	for (%i = 0; %i < 5; %i++) //checking 5 times is enough
	{
		%currentObjName = ("T" @ %team @ %type @ (%i + 1)); // T NUMBER OBJECTTYPE OBJECTCOUNT. Start with the count of 1.
		%foundObj = nameToId(%currentObjName); 

		if (isObject(%foundObj))
		{
			%totalAssets++;
			if (!%foundObj.isDisabled())
			{
				%poweredAssets++;
			}
		}
		else
		{
			if (%totalAssets > 0) //if there are assets and we didn't find the next asset break the loop and proceed with returning results
			{
				//echo("AIOAttackObject: We stopped finding the asset " @ %type @ " at the count of " @ %i);
				break;
			}
			else
			{
				//echo("AIOAttackObject: There are no  " @ %type @ " in team " @ %team);
				return -1; //there are no assets of this type in that team
			}
		}
	}

	if (!%percentage) //just return the number of powered assets
	{
		//echo("AIOAttackObject: Number of powered " @ %type @ " for team " @ %team @ " is " @ %poweredAssets);
		return %poweredAssets;
	}
	else
	{
		//echo("AIOAttackObject: The percentage of powered " @ %type @ " from team " @ %team @ " is " @ (%poweredAssets / %totalAssets));
		return (%poweredAssets / %totalAssets); 
	}
	
}

//---------------------------------------------------------------------------------------------------------------------- AIO Attack Object ---
//totally rewritten so but will RAPE the object. used only for generators in siege - Lagg... 11-1-2003
function AIOAttackObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
   {
      return 0;          
   }
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }

   //-note, commented out because this is probably stopping bots without blasters from buying any set 
   //bots to use blasters on close gens must have blaster and a pack - roberto
   // %needBuyPack = (%client.player.getMountedImage($BackpackSlot) == 0);

   // if ( (getWord(%this.equipment, 0) $= "Blaster") && (%client.player.getInventory("Blaster") < 1 || %needBuyPack) )
   // {
   // 		return 0;
   // }

   //---MISSION SPECIFIC 1---//
   //1.Copy this entire function to your mission extras script.
   //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
   //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
   //---MISSION SPECIFIC 1---//

   //lets see if we have a force field in our path
   InitContainerRadiusSearch(%client.player.getWorldBoxCenter(), 8, $TypeMasks::ForceFieldObjectType);
   %objSearch = containerSearchNext();
   if (%objSearch > 0)
   {
      %ffDis = %client.getPathDistance(%objSearch.getWorldBoxCenter());
      %myDis = %client.getPathDistance(%this.position);
      if (%ffDis < %myDis )
      {  
         if (%objSearch.isPowered() && %objSearch.team != %client.team)
            return 0;
      }
   }
      
   //no need to attack if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
   else
   {
      //if this bot is linked to a human who has issued this command, up the weight
      if (%this.issuedByClientId == %client.controlByHuman)
      {
         //make sure we have the potential to reach the minWeight
	 if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	 {
	    if ($AIWeightHumanIssuedCommand < %minWeight)
	       return 0;
	    else
	       %weight = $AIWeightHumanIssuedCommand;
         }
	 else
	 {
	    // calculate the default...
	    %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	    if (%weight < $AIWeightHumanIssuedCommand)
	       %weight = $AIWeightHumanIssuedCommand;
	 }
      }
      else
      {
         //make sure we have the potential to reach the minWeight
	 if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	    return 0;

	 // calculate the default...

         %this.desiredEquipment = "Plasma PlasmaAmmo";
	 %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

      //---MISSION SPECIFIC 2---//
      //1.Copy this entire function to your mission extras script.
      //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
      //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
      //---MISSION SPECIFIC 2---//
      }

      //if we are playing DnD, check if objective was completed already, if not give some extra :) - Lagg... 3-29-2004
      if (Game.class $= "DnDGame")
      {
         if (%this.targetObjectId.objectiveCompleted)
            %weight -= 400;
         else
            %weight += 200;
      }

      %targetObjDist = VectorDist(%client.player.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter());
      //if he can see it set accordingly - Lagg...
      if (%targetObjDist < 125)
      {
         //check for LOS
         %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType |
           $TypeMasks::ForceFieldObjectType | $TypeMasks::VehicleObjectType;
         %hasLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %this.targetObjectId.getWorldBoxCenter(), %mask, 0);

	      //if near genereator in ctf, cloakers and heavies have very high destroy gen weights- roberto
	      if (Game.class $= "CTFGame")
	      {
	      	if (%weight < 4000)
	      	{
	      		%weight += 150;
	      	}

	      	if (%targetObjDist < 75)
	      	{
	      		%weight += 100;
	      	}

	         if (%client.isCloakerOrHeavy && %weight <= 3890)
	         {
	         	//cloakers and heavies get very high weight
	         	%weight += 350;
	         }
	         else if (%client.player.getMountedImage($BackpackSlot) == 0 && %targetObjDist < 50 && %weight <= 3890)
	         {
	         	//if we're very close to the object and we have no pack, no use in grabbing the flag, just go destroy the object
	         	%weight += 350;
	         }

	      }

         if (%hasLOS && %weight <= 4000)
         {
            %weight += 250;
         }
      }

      //super high weights
	if (Game.class $= "CTFGame")
	{
      	%farIntoPath = false;

      	if (%this.followPath)
      	{
      		if (%this.locationIndex >= 2) //if we reached at least 3 markers then have high weight
      		{
      			%farIntoPath = true;
      		}
      		else if (%this.locationIndex >= 1 && %weight <= 4150) //if we reached at least 2 markers have increased weight
      		{
      			%weight += 150;
      		}
      	}

      	if ( %farIntoPath || (%client.isCloakerOrHeavy && %targetObjDist < 150) ) 
      	{
			%weight = $AIWeightHeavyGenAtt;
      	}
    }

      //add or decrease weight if the objective has inBase or inRoom fields - roberto
      //------------------------------------------------------------------------------
      if (%weight < 4150 )
      {
	      if (%this.inBase)
	      {
	         if (%client.inBase)
	         {
	            %weight += 100;
	         }
	      }

	      if (%this.inRoom)
	      {
	         if (%client.inRoom)
	         {
	            %weight += 200;
	         }
	      }
	  }
      //------------------------------------------------------------------------------

      return %weight;
   }
}

function AIOAttackObject::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIAttackObject);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIOAttackObject::unassignClient(%this, %client)
{
   //error("AIOAttackObject::unassignClient - " @ gettaggedString(%client.name));

   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//------------------------------------------------------------------------------------------------------------------- AI Attack Object ---
function AIAttackObject::initFromObjective(%task, %objective, %client)
{
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
   %task.equipment = %objective.equipment;
   %task.buyEquipmentSet = %objective.buyEquipmentSet;
   %task.desiredEquipment = %objective.desiredEquipment;
   %task.issuedByClient = %objective.issuedByClientId;
   %task.location = %objective.location;
   %task.mode = %objective.mode;

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //follow path - Lagg...
   %task.paths = %objective.paths;
   %task.followPath = false;

   //initialize other task vars
   %task.sendMsg = true;
   %task.sendMsgTime = 0;

   //MPB Telepoter
   %task.useTeleport = false;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;
}

function AIAttackObject::assume(%task, %client)
{
	%task.setWeightFreq(50);//was 15
	%task.setMonitorFreq(50);//was 15

	//find where we are
	%clientPos = %client.player.getWorldBoxCenter();
	%objPos = %task.targetObject.getWorldBoxCenter();
	%distToObject = %client.getPathDistance(%objPos);

   //added here so Bot buys a random set if specified in objective dataBlock - Lagg... 2-14-2003
      if (%task.buyEquipmentSet $= "randomSet")
      {
         //pick a random equipment set... 
      	//cloakers and heavies get more priority - roberto
         %randNum = getRandom();

         //if this is an attack objective that really needs the blaster use different sets
         if (getWord(%task.equipment, 0) $= "Blaster")
         {
            if (%randNum < 0.35)
            {
               %task.desiredEquipment = "Blaster Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "HeavySensorJamSetBlaster"; 
            }
            else if (%randNum < 0.7)
            {
               %task.desiredEquipment = "Blaster Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "HeavyShieldOffBlaster"; 
            } 
            else
            {
               %task.desiredEquipment = "Blaster Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "MediumSensorJamSetBlaster"; 
            }
         }
         else
         {
            if (%randNum < 0.15)
            {
               %task.desiredEquipment = "Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "HeavyEnergySet"; //reminder that energy set now has SensorJammer pack instead of energy pack
            }
            else if (%randNum < 0.20)
            {
               %task.desiredEquipment = "Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "MediumSensorJamSet"; //new sensorjammer set for mediums
            } 
            else if (%randNum < 0.60) 
            {
               %task.desiredEquipment = "Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "LightCloakSet";
            }
            else if (%randNum < 0.85)
            {
               %task.desiredEquipment = "Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "HeavyShieldOff";
            }
            else
            {
               %task.desiredEquipment = "Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "MediumMissileSet";
            }
         }
      }

   if (%task.equipment !$= "")
      %equipmentList = %task.equipment;
   else
      %equipmentList = %task.desiredEquipment;
   %client.needEquipment = AINeedEquipment(%equipmentList, %client);

   //even if we don't *need* equipemnt, see if we should buy some... 
   if (!%client.needEquipment && %task.buyEquipmentSet !$= "")
   {
      //see if we could benefit from inventory
      %needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
      %result = AIFindClosestInventory(%client, %needArmor);
      %closestInv = getWord(%result, 0);
      %closestDist = getWord(%result, 1);
      if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
      {
		 if (%distToObject < 0 || %closestDist < 25)//-------- 25m or less to inventoryStation in this case
	            %client.needEquipment = true;
      }
   }

   //mark the current time for the buy inventory state machine
   %task.buyInvTime = getSimTime();

   //--------------------------------------------------- follow paths ---
   //set the destination paths for each team and game type

   //first see how many paths we have
   //check if attack path exists
   if (Game.class $= "SiegeGame")
      %team = Game.offenseTeam == %client.team ? 1 : 2;
   else
      %team = %client.team;

   //first see how many paths we have
   %mx = getWordCount(%task.paths);

   //then see if we are far enough  from the target object to start following a path
   %farEnough = (%client.getPathDistance(%objPos) > 200);

  //modified so we remember last path position, or we start at the last marker if needed - roberto
   //I know all of this could had easily be done with a for loop looking through all the markers and just getting the closest one to continue a path
   //but maybe when you have many paths it's not so fast to loop through 10 markers and it's faster to just instantly set the destination to a remembered marker
   if (%mx > 0 && %farEnough) 
   { 
      if (%client.lastIndexAttackPath > -1 && %client.lastAttackPath > -1)
      {
      	%task.locationIndex = %client.lastIndexAttackPath; //last marker in path
      	%tg = %client.lastAttackPath; 	//which path to use
      	%task.group = nameToId("T" @ %team @ "AttackPath" @ %tg); 
         %task.count = %task.group.getCount(); 
	    //echo("ATTACK OBJECT " @ %client.nameBase @ " following from where we left of, marker" @ %task.locationIndex @ " from AttackPath " @ %tg);
      }
      else
      {
		%random = mFloor(getRandom(1, %mx));
		%random--;
		%tg = getWord(%task.paths, %random); //which path to use
		%client.lastAttackPath = %tg;

		%task.group = nameToId("T" @ %team @ "AttackPath" @ %tg);
         %task.count = %task.group.getCount();  
		
		//get the last marker in the path, if we are close we shouldn't go to the beginning of the path, start from the last marker
		%lastMarker = %task.group.getObject(%task.count -1); 
         %distToLastMarker = VectorDist(%client.player.position, %lastMarker.position);

         if (%distToLastMarker < 300) //if we are close, start from the last marker
         {
			%task.locationIndex = %task.count -1; 
			%client.lastIndexAttackPath = %task.count -1;
			//echo("ATTACK OBJECT " @ %client.nameBase @ " on first try, close to target object, start following last marker " @ %task.locationIndex @ " from AttackPath " @ %tg);
         }
         else //we are not even close to the last marker in the path, try two times to see if we're close to last marker, this gonna be long but better than looping through 6 markers per path and checking distance.
         {
         	//try with another path again, will only work for two paths, no point trying with tons of paths
         	if (%tg < 2)
         	{
         		%tg++;
         	}
         	else
         	{
         		%tg--;
         	}

         	%nextPathExists = isObject(nameToId("T" @ %team @ "AttackPath" @ %tg));
         	%nextPathFoundMarker = false;

         	//if there's one more path to try, see if the distance from the last marker is good or not
         	if (%nextPathExists)
         	{
    			    %client.lastAttackPath = %tg;
				%task.group = nameToId("T" @ %team @ "AttackPath" @ %tg);
				%task.count = %task.group.getCount();  

				//get the last marker in the path, if we are close we shouldn't go to the beginning of the path, start from the last marker
				%lastMarker = %task.group.getObject(%task.count -1); 
				%distToLastMarker = VectorDist(%client.player.position, %lastMarker.position);

	            if (%distToLastMarker < 300) //if we are close, start from the last marker
	            {
	            	%nextPathFoundMarker = true;
					%task.locationIndex = %task.count -1; 
					%client.lastIndexAttackPath = %task.count -1;
					//echo("ATTACK OBJECT " @ %client.nameBase @ " on second try, close to target object, start following last marker " @ %task.locationIndex @ " from AttackPath " @ %tg);
	            }
         	}

         	if (!%nextPathFoundMarker) //only a single path exists, and we aren't close to the last marker in it, or we have no memory of following it, start from the beginning
         	{
				%random = mFloor(getRandom(1, %mx));
				%random--;
				%tg = getWord(%task.paths, %random); //which path to use
				%task.group = nameToId("T" @ %team @ "AttackPath" @ %tg);
				%task.count = %task.group.getCount();  

				%task.locationIndex = 0; 
				%client.lastAttackPath = %tg;
				%client.lastIndexAttackPath = 0;

				//echo("ATTACK OBJECT " @ %client.nameBase @ " following path from the beginning.");
         	}
         }
      }

      //make sure there are markers in the AttackPath group
      if (%task.count > 0)
      {
         %task.followPath = true;
      }
      else
      {
      	%task.followPath = false;
      }
   }
   else
   {
   	%task.followPath = false;
   }

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0 && isObject(%task.targetObject) )
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
		}
	}
    //--------- End teleporters check ---------
}

function AIAttackObject::retire(%task, %client)
{
   %client.setTargetObject(-1);
}

function AIAttackObject::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;

   //see if we can find someone to shoot at...
   if (%client.getEngageTarget() <= 0)
   {
      %losTimeout = $AIClientMinLOSTime + ($AIClientLOSTimeout * %client.getSkillLevel());
      %myLocation = %client.player.getWorldBoxCenter();
      %result = AIFindClosestEnemy(%client, 40, %losTimeout);
      %task.engageTarget = getWord(%result, 0);
   }
   
   %task.setWeight(%task.baseWeight);
}

function AIAttackObject::monitor(%task, %client)
{
   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk);
            return;
         }
      }
      else
         %task.useTeleport = false;
   }
   //--------------------------------------------------- monitor telporter - end -

   //first, buy the equipment - if we really really need some :) - Lagg... - 4-6-2003
   %hasMortar = (%client.player.getInventory("Mortar") > 0) && (%client.player.getInventory("MortarAmmo") > 0);
   %hasPlasma = (%client.player.getInventory("Plasma") > 0) && (%client.player.getInventory("PlasmaAmmo") > 0);
   %hasDisc = (%client.player.getInventory("Disc") > 0) && (%client.player.getInventory("DiscAmmo") > 0) && (!%client.player.getInventory("Blaster") > 0);

   if (%hasMortar || %hasPlasma || %hasDisc)
   {
      %client.needEquipment = false;
   }

   //bots that need to use blasters in very tight rooms must have a pack, blaster, and a plasma weapon - roberto
   %needToBuy = (%client.player.getMountedImage($BackpackSlot) == 0) || (%client.player.getInventory("Plasma") < 1);

   if ( (getWord(%task.equipment, 0) $= "Blaster") && (%client.player.getInventory("Blaster") < 1 || %needToBuy) )
   {
   		%client.needEquipment = true;
   }

   //first, buy the equipment - if we really really need some :) - Lagg... - 4-6-2003
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
         %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(50);//was 15
	 %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
	         //if this task is the objective task, choose a new objective
		 if (%task == %client.objectiveTask)
		 {
		    AIUnassignClient(%client);
		    Game.AIChooseGameObjective(%client);
		 }
	     return;
      }
   }  

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();
   
   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
         if (%client.isAIControlled())
         {
	    if (%task.chat !$= "")
	    {
	       %chatMsg = getWord(%task.chat, 0);
	       %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
	          AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, -1);
            }
	    else if (%task.targetObject > 0)
	    {
	       %type = %task.targetObject.getDataBlock().getName();
	       if (%type $= "GeneratorLarge")
	          AIMessageThreadTemplate("AttackBase", "ChatSelfAttackGenerator", %client, -1);
	       else if (%type $= "SensorLargePulse")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
	       else if (%type $= "SensorMediumPulse")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
	       else if (%type $= "TurretBaseLarge")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackTurrets", %client, -1);
	       else if (%type $= "StationVehicle")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackVehicle", %client, -1);
            }
         }
      }
   }


   //set the target object if targetObject not destroyed
   %outOfStuff = (AIAttackOutofAmmo(%client));
   if (isObject(%task.targetObject) && %task.targetObject.getDamageState() !$= "Destroyed" && !%outOfStuff)
   {
      %targetLOS = "false";
      %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType;
      %targetLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %task.targetObject.getWorldBoxCenter(), %mask, 0);
      %close = false;
       %distToTargObj = VectorDist(%task.targetObject.position, %client.player.position);
      %targetTooFar =  (%distToTargObj > 50);

      if (%distToTargObj < 7)
      {
         %close = true;

         //bots that must destroy generators very close to them and must use a blaster get a pass
         if (getWord(%task.equipment, 0) $= "Blaster" && %client.player.getInventory("Blaster") >= 1)
         {
         	%close = false;
         }
      }

	if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
      else if (%task.followPath)
      {
         //error("AIAttackObject::monitor - Following ATTACK Path - " @ getTaggedString(%client.name));
          //if we're close to the object, don't follow path
          if (%client.getPathDistance(%task.targetObject.position) < 50)
          {
              %task.followPath = false;
              %client.lastIndexAttackPath = -1;
              %client.lastAttackPath = -1;
              return;
          }

         //set the path and follow it to the end
         %task.spot = %task.group.getObject(%task.locationIndex).position;

         %pos = %client.player.position;
         %dest = %task.spot;

		 //heavies have a higher precision for markers since they're slow
         if (%client.player.getArmorSize() $= "Heavy")
         {
         	%nearMarker = 1.5; 
         }
         else
         {
         	%nearMarker = 8;
         }

         //are we close to location index marker?
         if (VectorDist(%dest, %pos) < %nearMarker)
         {
            //if we have another location index
            if ((%task.count - 1) > %task.locationIndex)//has to be -1 on the groups count
            {
               %task.locationIndex++;
               %client.lastIndexAttackPath = %task.locationIndex;
		       //error("AITouchObject::monitor - lastIndexAttackPath is " @ %client.lastIndexAttackPath);
            }
            //else we are at end of trail
            else
            {
				//if this task is the objective task, choose a new objective
					if (%task == %client.objectiveTask)
					{
					%task.followPath = false;
					%client.lastIndexAttackPath = -1;
					%client.lastAttackPath = -1;
					return;
				}
            }
         }
         else
         {
         	//don't move like a maniac when near lava - roberto
         	if (%client.inDanger) { %client.stepMove(%task.spot, 0.25, $AIModeWalk); }
         	else { %client.stepMove(%task.spot, 0.25, $AIModeExpress); }
         }
      }
      else
      {
      	//only move to the object if we don't have LOS or we are very far - roberto
      	if (!%targetLOS || %targetTooFar)
      	{
         	//don't move like a maniac when near lava - roberto
         	if (%client.inDanger) { %client.stepMove(%task.location, 0.25, $AIModeWalk); }
         	else { %client.stepMove(%task.location, 0.25, $AIModeExpress); }
      	}
      }
      
      //fire at target object if in sight
      if (%targetLOS && !%close)
      {
         %task.setWeight($AIWeightReturnFire);

         //if we are following a path, stop.
         %task.followPath = false;

         %client.setEngageTarget(-1); 

		//%client.stop(); // this will clear the players move state
		//%client.clearStep();
         //weapons depend on armor, close targets that require blaster get priority - roberto
         if (getWord(%task.equipment, 0) $= "Blaster" && %client.player.getInventory("Blaster") >= 1)
         {
         	%client.setTargetObject(%task.targetObject, 100, "Blaster");
         	//echo("WE ARE USING A BLASTER TO DESTROY THE GENS");
         }
         else if (%client.player.getInventory("Plasma") > 0 && %client.player.getInventory("PlasmaAmmo") >= 1)
         {
         	%client.setTargetObject(%task.targetObject, 100, "Plasma");
         }
         else if (%client.player.getInventory("Disc") > 0 && %client.player.getInventory("DiscAmmo") >= 1)
         {
         	%client.setTargetObject(%task.targetObject, 100, "Disc");
         }
         else if (%client.player.getInventory("Chaingun") > 0 && %client.player.getInventory("ChaingunAmmo") >= 1)
         {
         	%client.setTargetObject(%task.targetObject, 100, "Chaingun");
         }
         else
         {
         	%client.setTargetObject(%task.targetObject, 100, "Blaster");
         }
         
      }
      else
      {
         %client.setTargetObject(-1);
         %task.setWeight(%task.baseWeight);

         //see if we need to engage a new target
         %engageTarget = %client.getEngageTarget();
         if (!AIClientIsAlive(%engageTarget) && %task.engageTarget > 0)
            %client.setEngageTarget(%task.engageTarget);

         //else see if we should abandon the engagement
         else if (AIClientIsAlive(%engageTarget))
         {
            %myPos = %client.player.getWorldBoxCenter();
            %testPos = %engageTarget.player.getWorldBoxCenter();
            %distance = %client.getPathDistance(%testPos);
            if (%distance < 0 || %distance > 20)
               %client.setEngageTarget(-1);

            //see if we're supposed to be engaging anyone...
            if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
               %client.setEngageTarget(%client.shouldEngage);
         }
      } 
   }
   else
   {
      %client.setTargetObject(-1);      
            
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         //if we need to repair self, aim at the floor
         if (%client.player.getDamagePercent() > 0 && %client.player.getInventory(RepairPack) > 0)
         {
            %pos = %client.player.getWorldBoxCenter();
            %posX = getWord(%pos, 0);
            %posY = getWord(%pos, 1);
            %posZ = getWord(%pos, 2) - 1;
            %pos = %posX SPC %posY SPC %posZ;
            %client.aimAt(%pos, 500);
         }
         AIUnassignClient(%client);
         Game.AIChooseGameObjective(%client);
      }
   }
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO Attack Player +++

function AIOAttackPlayer::weight(%this, %client, %level, %minWeight, %inventoryStr)
{

   %mustReturnFlag = false;

   //make sure the player is still alive!!!!!
   if ( !AIClientIsAlive(%client) )
   {
      return 0;
   }

   //if our flag isn't returned and is dropped check our distance and return it if close - roberto
   if (Game.class $= "CTFGame")
   {
      if (!$AITeamFlag[%client.team].isHome && $AITeamFlag[%client.team].carrier $= "")
      {
         if (VectorDist(%client.player.position, $AITeamFlag[%client.team].position) < 230)
         {
            %mustReturnFlag = true;
         }
      }
   }

	//if we're attacking the flag carrier, make sure a flag carrier exists
	if (%this.mode $= "FlagCarrier")
	{
		if (%this.targetObjectId.carrier $= "" && !%mustReturnFlag) //we stop this objective only if we mustn't return the flag
      {
         return 0;
      }
		else
      {
			%this.targetClientId = %this.targetObjectId.carrier.client;

         //stop the LightEnergySniper chaser bots (there are about 2 in each map) if we have humans in our team and the enemy team is full of bots 
         if ($Host::AdaptiveCTFBots)
         {
         	if ( !$AITeamLosing[%client.team] || $Host::BotDifficulty !$= "Easy" ) //on any difficulty, or on easy if we're winning only
         	{
	            %enemyTeam = 1;
	            if (%client.team == 1) { %enemyTeam = 2; }

	            if ( $AIHumansInTeam[%client.team] >= 1 && $AIHumansInTeam[%enemyTeam] <= 0 )
	            {
	               if (%this.buyEquipmentSet $= "LightEnergySniper")
	               {
	                  return 0;
	               }
	            }
         	}
         }
      }
	}
      
	//now, if this bot is linked to a human who has issued this command, up the weight
	if (%this.issuedByClientId == %client.controlByHuman)
	{
		//make sure we have the potential to reach the minWeight
		if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
		{
			if ($AIWeightHumanIssuedCommand < %minWeight)
				return 0;
			else
				%weight = $AIWeightHumanIssuedCommand;
		}
		else
		{
			// calculate the default...
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
			if (%weight < $AIWeightHumanIssuedCommand)
				%weight = $AIWeightHumanIssuedCommand;
		}
	}
	else
	{
		//make sure we have the potential to reach the minWeight
		if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			return 0;

		// calculate the default...
		%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	}

	return %weight;
}

function AIOAttackPlayer::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIAttackPlayer);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIOAttackPlayer::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//------------------------------------------------------------------------------------------------- AI Attack Player ---

function AIAttackPlayer::initFromObjective(%task, %objective, %client)
{
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.targetClient = %objective.targetClientId;
   %task.equipment = %objective.equipment;
	%task.buyEquipmentSet = %objective.buyEquipmentSet;
	%task.desiredEquipment = %objective.desiredEquipment;
	%task.issuedByClient = %objective.issuedByClientId;
        %task.chat = %objective.chat;

	//initialize other task vars
	%task.sendMsg = true;
	%task.sendMsgTime = 0;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;
}

function AIAttackPlayer::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(15);
   %client.needEquipment = AINeedEquipment(%task.equipment, %client);
	if (! %client.needEquipment)
	   %client.stepEngage(%task.targetClient);

	//even if we don't *need* equipemnt, see if we should buy some... 
	if (! %client.needEquipment && %task.buyEquipmentSet !$= "")
	{
		//see if we could benefit from inventory
		%needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
		%result = AIFindClosestInventory(%client, %needArmor);
		%closestInv = getWord(%result, 0);
		%closestDist = getWord(%result, 1);
		if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0 && AIClientIsAlive(%task.targetClient.player)) //only need equipment if target is alive -roberto
		{
			%distToTarg = %client.getPathDistance(%task.targetClient.player.getWorldBoxCenter());
			if (%distToTarg < 0 || %distToTarg > 100)
				%client.needEquipment = true;
		}
	}

	//sparky's teleporters
	if ( isObject(%task.targetClient.player) )
	{
		%distToPlayer = VectorDist(%client.player.position, %task.targetClient.player.position);

		if ( %distToPlayer > 800 )
		{
		  %result = aiTeleporterCheck(%client, %task.targetClient.player);
		  %task.useTele = getWord(%result, 0);//true or false
		  %task.tele = getWord(%result, 1);//this is the linked tele to use

		  if (%task.useTele)
		  {
		  	%distToTele = vectorDist(%client.player.position, %task.tele.position);

		  	if (%distToTele > 50)
		  	{
				%task.useTele = 0;
				%task.tele = "-1 32000";
		  	}
		  }
		}
		else
		{
			%task.useTele = 0;
			%task.tele = "-1 32000";
		}
	}
	else
	{
		%task.useTele = 0;
		%task.tele = "-1 32000";
	}

	//mark the current time for the buy inventory state machine
	%task.buyInvTime = getSimTime();
}

function AIAttackPlayer::retire(%task, %client)
{
	//dissolve the human control link
	if (%task == %client.objectiveTask)
		aiReleaseHumanControl(%client.controlByHuman, %client);
}

function AIAttackPlayer::weight(%task, %client)
{
	//update the task weight
	if (%task == %client.objectiveTask)
		%task.baseWeight = %client.objectiveWeight;

   %task.setWeight(%task.baseWeight);
}

function AIAttackPlayer::monitor(%task, %client)
{
   //chat, first
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
         if (%client.isAIControlled())
	 {
	    if (%task.chat !$= "")
	    {
	       %chatMsg = getWord(%task.chat, 0);
	       %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
	          AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, true);
	    }
	    else
               AIMessageThread("ChatSelfAttack", %client, -1);
	 }
      }
   }

   //second, buy the equipment
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
         %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(15);
	 %client.needEquipment = false;
	 //%client.stepEngage(%task.targetClient);
      }
      else if (%result $= "Failed")
      {
         ////if this task is the objective task, choose a new objective
	 //if (%task == %client.objectiveTask)
	 //{
	    //AIUnassignClient(%client);
	    //Game.AIChooseGameObjective(%client);
	 //}
	 //return;

         %task.setMonitorFreq(15);
	 %client.needEquipment = false;
	 //%client.stepEngage(%task.targetClient);
      }
   }

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();

   	//-----monitor teleporter ------
	if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
	//------teleporter monitor end ------

   %needReturnFlag = false;

   if (Game.class $= "CTFGame")
   {
      %ourFlagIsDropped = (!$AITeamFlag[%client.team].isHome && $AITeamFlag[%client.team].carrier $= "");
      %wereNearTheFlag = (VectorDist(%client.player.position, $AITeamFlag[%client.team].position) < 250);
      %noEnemyNearFlag = true;

      %enemyResult = AIFindClosestEnemyToLoc(%client, $AITeamFlag[%client.team].position, 90, $AIClientLOSTimeout, true);
      %closestEnemy = getWord(%enemyResult, 0);

      if ( AIClientIsAlive(%closestEnemy) )
      {
         if ( !%closestEnemy.isAIControlled() ) //only humans drop flags as bait
         {
            %noEnemyNearFlag = false;
         }
      }

      if (%ourFlagIsDropped && %wereNearTheFlag && %noEnemyNearFlag)
      {
         %needReturnFlag = true;
      }
   }

   if (%needReturnFlag)
   {
      %task.targetClient = -1;
      %client.setEngageTarget(-1);
      %client.lastDamageClient = -1;
   }
   else
   {
      //cheap hack for now...   make the bot always know where you are...
      %client.clientDetected(%task.targetClient);

      //make sure we're still attacking...
      if (%client.getStepName() !$= "AIStepEngage")
      {
         %client.stepEngage(%task.targetClient);
      }

      //make sure we're still attacking the right target
      %client.setEngageTarget(%task.targetClient);
   }

   %superChaseActive = (Game.class $= "CTFGame" && $AISuperChaseSpeed && !%client.inBase);

   //if enabled, chase HUMAN enemy flag carrier with high speed - Roberto
   if (AIClientIsAlive(%task.targetClient) && %superChaseActive && !%task.targetClient.isAIControlled()) 
   {
       %targetHasFlag = %task.targetClient.player.holdingFlag;
       
      if (%targetHasFlag)
      {
          %ourpos = %client.player.position;
          %enemypos = %task.targetClient.player.position;
          %flagCarrierDistance = VectorDist(%ourpos, %enemypos);
            %targetIsVeryFar = %flagCarrierDistance >= 125;
            %targetIsMegaUltraFar = %flagCarrierDistance >= 175;
          %targetIsFar = %flagCarrierDistance > 10 && !%targetIsVeryFar;
            %lineofsight_obs = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType | $TypeMasks::ForceFieldObjectType; 
           %hasLineOfSight = !containerRayCast(%ourpos, %enemypos, %lineofsight_obs, 0); 
          %foundMarker = false;
          %tempEnemyTarget = %task.targetClient;

          if (%hasLineOfSight)
          {
          	if (%targetIsMegaUltraFar)
          	{
	          	%client.stepMove(%enemypos, 0.25, $AIModeExpress); 
	            %client.aimAt(%enemypos, 15000);
	            applyKick(%client.player, "foward", %client.chaseSpeedMegaUltraFar, false);
          	}
          	else if (%targetIsVeryFar)
          	{
	          	%client.stepMove(%enemypos, 0.25, $AIModeExpress);
	            %client.aimAt(%enemypos, 15000);
	            applyKick(%client.player, "foward", %client.chaseSpeedVeryFar, false);
          	}
          	else if (%targetIsFar)
          	{
	          	%client.stepMove(%enemypos, 0.25, $AIModeExpress);
	            %client.aimAt(%enemypos, 15000);
	            applyKick(%client.player, "foward", %client.chaseSpeedFar, false);
          	}
          	else
          	{
				//don't move like a maniac when near lava - roberto
				if (%client.inDanger) { %client.stepMove(%enemypos, 0.25, $AIModeWalk); }
				else { %client.stepMove(%enemypos, 0.25, $AIModeExpress); }
          	}
         }
         else //if we don't have line of sight then look for a boost marker in ChaserBoostPoints and boost towards it
         {
            %task.boostPointsGroup = nameToId("ChaserBoostPoints");
            %count = %task.boostPointsGroup.getCount();

            //get the marker that is closer to the enemy flag carrier than us 
            for (%i = 0; %i < %count; %i++)
            {
               %currentMarker = %task.boostPointsGroup.getObject(%i);
               %markerToCarrierDist = VectorDist(%currentMarker.position, %enemypos);

               if (%markerToCarrierDist < %flagCarrierDistance) //that marker is closer to the enemy flag carrier than we are
               {
                  %canSeeMarker = !containerRayCast(%ourpos, %currentMarker.position, %lineofsight_obs, 0); 

                  if (%canSeeMarker) 
                  {
                  	%client.setEngageTarget(-1); 
                  	%client.stepEngage(-1);
                  	%client.stepMove(%currentMarker.position, 0.25, $AIModeExpress);
                     %client.aimAt(%currentMarker.position, 2500);
                     applyKick(%client.player, "foward", %client.chaseSpeedMarker, false);
                     %foundMarker = true;
                     break;
                  }
               }
            }
			//if we did not find a visible marker just normally walk to the enemy
			if (!%foundMarker)
			{
				//don't move like a maniac when near lava - roberto
				if (%client.inDanger) { %client.stepMove(%enemypos, 0.25, $AIModeWalk); }
				else { %client.stepMove(%enemypos, 0.25, $AIModeExpress); }
			}
         }

         //after the boosts make the bot look back at it's target
         %client.setEngageTarget(%tempEnemyTarget); 
         %client.stepEngage(%client.getEngageTarget());
     }
   }

   if (%client.getStepStatus() !$= "InProgress" && %task == %client.objectiveTask)
   {
      if (Game.class $= "CTFGame")
      {
         //if our flag isn't returned and is dropped check our distance and return it if close - roberto
         if (%needReturnFlag)
         {
				//don't move like a maniac when near lava - roberto
				if (%client.inDanger) { %client.stepMove($AITeamFlag[%client.team].position, 0.25, $AIModeWalk); }
				else { %client.stepMove($AITeamFlag[%client.team].position, 0.25, $AIModeExpress); }
         }
         else
         {
            AIUnassignClient(%client);
            Game.AIChooseGameObjective(%client);
         }
      }
      else
      {
         AIUnassignClient(%client);
         Game.AIChooseGameObjective(%client);
      }
   }
}

//------------------------------------------------------------------------------------------------ AIO Defend Location ---

//modified so if set to offensive in .mis file the bot will defend a destroyed enemy object (can you say Camping in Base ?) - Lagg... 1/7/2004
//must be placed in the offense teams objective simgroup and DO NOT set defense = 1
//for defense: DO NOT set ofense = 1 in .mis file, for Camping offense must not have defense > 0 in the .mis file  - * IMPORTANT * -
//also added if we are a defending with sniper gear (SniperRifle and EnergyPack) than defend like a sniper :) - Lagg... 3-25-2004

function AIODefendLocation::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }

   %weightModifier = 0;

   //some ctf improvements - roberto
   if (Game.class $= "CTFGame")
   {
      //in ctf don't defend stuff if we're over 9000 miles away 
		if (VectorDist(%this.location, %client.player.position) > 400)
		{
			return 0;
		}

      //if we're in danger of losing and this is an emergency task only run if needed  - roberto
      if (%this.ctfEmergency && $emergencyFlagDefender[%client.team] == false)
      {
         return 0;
      }

      //if the enemy team is full of bots and we have humans in our team, and we're not losing, lower priority - roberto
      if ( isObject(%this.targetObjectId) && $Host::AdaptiveCTFBots )
      {
         %objType = %this.targetObjectId.getDataBlock().getName();

         if (%objType $= "Flag")
         {

            %enemyTeam = 1;
            if (%client.team == 1) { %enemyTeam = 2; }

            if ( $AIHumansInTeam[%client.team] >= 1 && $AIHumansInTeam[%enemyTeam] <= 0 )
            {
               %weightModifier = $AIDefenderWeightModifier[%client.team];

            }
         }
      }
   }

   //---MISSION SPECIFIC 1---//
   //1.Copy this entire function to your mission extras script.
   //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
   //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
   //---MISSION SPECIFIC 1---//

   //who owns the object we are to defend?
   if (%this.targetObjectId > 0  && %this.defense > 0)
   {
      if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team != %client.team)
         return 0;

      //don't defend unless the entire group is powered
      if (%this.targetObjectId.getGroup().powerCount <= 0)
         return 0;
   }
   else if (%this.targetObjectId > 0  && %this.offense > 0)
   {
      if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      {
         return 0;
      }

      //don't camp unless all the gens in simgroup are destroyed
      if (%this.targetObjectId.getGroup().powerCount > 0)
      {
      	return 0;
      }

      //if we're too far away, don't camp - roberto
		if ( VectorDist(%this.targetObjectId.position, %client.player.position) > 75 )
		{
			return 0;
		}
   }

   //do a quick check to disqualify this objective if it can't meet the minimum weight
   if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
   {
      if (%this.targetObjectId > 0 && %this.issuedByClientId == %client.controlByHuman)
      {
         if ($AIWeightHumanIssuedCommand < %minWeight)
            return 0;
      }
      else
         return 0;
   }

   %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

   //if defensive  the object has been destroyed, reduce the weight / if offensive increase the weight if destroyed (Camping !)
   if (%this.targetObjectId > 0 && %this.defense > 0)
   {
      //see if we were forced on the objective
      if (%this.issuedByClientId == %client.controlByHuman && %weight < $AIWeightHumanIssuedCommand)
      {
         %weight = $AIWeightHumanIssuedCommand;
      }
      //else see if the object has been destroyed - slight modification here - Lagg... 11-7-2003
      //else if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
      else if (!isObject(%this.targetObjectId) || %this.targetObjectId.isDisabled())
      {
         %weight -= 320;
      }
      //---MISSION SPECIFIC 2---//
      //1.Copy this entire function to your mission extras script.
      //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
      //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
      //---MISSION SPECIFIC 2---//
   }
   else if (%this.targetObjectId > 0 && %this.offense > 0)
   {
      //see if we were forced on the objective
      if (%this.issuedByClientId == %client.controlByHuman && %weight < $AIWeightHumanIssuedCommand)
      {
			%weight = $AIWeightHumanIssuedCommand;
      }
      // - slight modification here - Lagg... 11-7-2003
      else if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() !$= "Destroyed")
      {
         return 0;//don't camp if not destroyed
      }

      if (%client.player.getInventory("InventoryDeployable") > 0)
      {
         return 0;
      }
   }

   //add or decrease weight if the objective has inBase or inRoom fields - roberto
   //------------------------------------------------------------------------------
   if (%this.inBase) 
   {
      if (!%client.inBase)
      {
         if (Game.class $= "SiegeGame")
         {
            %weight -= 100;
         }
      }
      else
      {
         if (%weight <= 4000) { %weight += 100; }
      }
   }

   if (%this.inRoom)
   {
      if (!%client.inRoom)
      {
         %weight -= 100;
      }
      else
      {
         if (%weight <= 4000) { %weight += 100; }
      }
   }
   //------------------------------------------------------------------------------

   return %weight + %weightModifier;
}

function AIODefendLocation::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIDefendLocation);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIODefendLocation::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//-------------------------------------------------------------------------------------------------------------- AI Defend Location ---

function AIDefendLocation::initFromObjective(%task, %objective, %client)
{
	//initialize the task vars from the objective
	%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
	%task.targetObject = %objective.targetObjectId;
	if (%objective.Location !$= "")
	   %task.location = %objective.location;
	else
	   %task.location = %objective.targetObjectId.getWorldBoxCenter();

   %task.equipment = %objective.equipment;
	%task.buyEquipmentSet = %objective.buyEquipmentSet;
	%task.desiredEquipment = %objective.desiredEquipment;
	%task.issuedByClient = %objective.issuedByClientId;
	%task.chat = %objective.chat;
        %task.mode = %objective.mode;

        //added for base camping if on offensive
        %task.defense = %objective.defense;
        %task.offense = %objective.offense;

   //ctfEmergency is automatically false if the field doesn't exist
   if (%objective.ctfEmergency $= "")
   {
      %task.ctfEmergency = false;
   }
   else
   {
      %task.ctfEmergency = %objective.ctfEmergency; //if objective must only be completed if in danger of losing
   }

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;

	//initialize other task vars
	%task.sendMsg = true;
	%task.sendMsgTime = 0;
   
        %task.engageTarget = -1;
}

function AIDefendLocation::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(15);
   %client.inPerimeter = false;
	//%client.needEquipment = AINeedEquipment(%task.equipment, %client);

   if (%task.equipment !$= "")
      %equipmentList = %task.equipment;
   else
      %equipmentList = %task.desiredEquipment;
   %client.needEquipment = AINeedEquipment(%equipmentList, %client);


	//even if we don't *need* equipemnt, see if we should buy some... 
	if (! %client.needEquipment && %task.buyEquipmentSet !$= "")
	{
		//see if we could benefit from inventory
		%needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
		%result = AIFindClosestInventory(%client, %needArmor);
		%closestInv = getWord(%result, 0);
		%closestDist = getWord(%result, 1);
		if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
		{
		   %result = AIFindClosestEnemy(%client, 200, $AIClientLOSTimeout);
	           %closestEnemy = getWord(%result, 0);
		   %closestEnemydist = getWord(%result, 1);

			if (%closestEnemy <= 0 || (%closestEnemyDist > %closestDist * 1.5))
				%client.needEquipment = true;
		}
	}

	//mark the current time for the buy inventory state machine
	%task.buyInvTime = getSimTime();

	//set a flag to determine if the objective should be re-aquired when the object is destroyed/repaired - modified Lagg...
	%task.reassignOnDestroyed = false;

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0)
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
		}
	}
    //--------- End teleporters check ---------
}

function AIDefendLocation::retire(%task, %client)
{
	%task.engageVehicle = -1;
	%client.setTargetObject(-1);
}

function AIDefendLocation::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;

   %player = %client.player;
   if (!isObject(%player))
      return;

   %hasMissile = (%player.getInventory("MissileLauncher") > 0) && (%player.getInventory("MissileLauncherAmmo") > 0);

   //if we're defending with a missile launcher, our first priority is to take out vehicles...
   //see if we're already attacking a vehicle...
   if (%task.engageVehicle > 0 && isObject(%task.engageVehicle) && %hasMissile)
   {
      //set the weight
      %task.setWeight(%task.baseWeight);
      return;
   }

   //search for a new vehicle to attack
   %task.engageVehicle = -1;
   %losTimeout = $AIClientMinLOSTime + ($AIClientLOSTimeout * %client.getSkillLevel());
   %result = AIFindClosestEnemyPilot(%client, 300, %losTimeout);
   %pilot = getWord(%result, 0);
   %pilotDist = getWord(%result, 1);

   //if we've got missiles, and a vehicle to attack...
   if (%hasMissile && AIClientIsAlive(%pilot))
   {
      %task.engageVehicle = %pilot.vehicleMounted;
      %client.needEquipment = false;
   }

   //otherwise look for a regular enemy to fight... added for sniper defender - Lagg... 4-16-2004
   else
   {
      //if we are sniper extend the range
      if (%client.player.getInventory(SniperRifle) > 0 && %client.player.getInventory(EnergyPack) > 0)
         %result = AIFindClosestEnemyToLoc(%client, %task.location, 500, $AIClientLOSTimeout);
      ////if we are offense camping cheat a little, ignore LOS :) - Lagg...
      //else if (%task.offense)
	 //%result = AIFindClosestEnemyToLoc(%client, %task.location, 150, %losTimeout, true);
      //if we have mortar than extend the range
      else if (%client.player.getInventory(Mortar) > 0 && %client.player.getInventory(MortarAmmo) > 0)
         %result = AIFindClosestEnemyToLoc(%client, %task.location, 400, $AIClientLOSTimeout, false);
      //otherwise, do the search normally.
      else
	 %result = AIFindClosestEnemyToLoc(%client, %task.location, 150, %losTimeout, false);

      %closestEnemy = getWord(%result, 0);
      %closestdist = getWord(%result, 1);
	   
      //see if we found someone
      if (%closestEnemy > 0)
         %task.engageTarget = %closestEnemy;
      else
      {
         %task.engageTarget = -1;

	 //see if someone is near me...
	 %result = AIFindClosestEnemy(%client, 150, %losTimeout);
         %closestEnemy = getWord(%result, 0);
         %closestdist = getWord(%result, 1);
	 if (%closestEnemy <= 0 || %closestDist > 150)
	    %client.setEngageTarget(-1);
      }
   }

   %task.setWeight(%task.baseWeight); 
}

function AIDefendLocation::monitor(%task, %client)
{
   //if the defend location task has an object, set the "reset" flag
   if (%task == %client.objectiveTask && isObject(%task.targetObject))
   {
      //if (%task.targetObject.getDamageState() !$= "Destroyed")//modified for offensive camping (heh heh) - Lagg... 11-9-2003
      if (%task.targetObject.isDisabled() && %task.defense > 0)
         %task.reassignOnDestroyed = true;
      else if (%task.targetObject.isEnabled() && %task.offense > 0)
	 %task.reassignOnDestroyed = true;

      if (%task.reassignOnDestroyed)
      {
         AIUnassignClient(%client);
	 Game.AIChooseGameObjective(%client);
         return;
      }
   }

   //buy the equipment
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
	 %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(15);
	 %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
         //if this task is the objective task, choose a new objective
	 if (%task == %client.objectiveTask)
	 {
	    AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	 }
	 return;
      }
   }
   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();

   //chat
   if (%task.sendMsg)
   {
		if (%task.sendMsgTime == 0)
			%task.sendMsgTime = getSimTime();
		else if (getSimTime() - %task.sendMsgTime > 7000)
		{
			%task.sendMsg = false;
		   if (%client.isAIControlled())
			{
				if (%task.chat !$= "")
				{
					%chatMsg = getWord(%task.chat, 0);
					%chatTemplate = getWord(%task.chat, 1);
					if (%chatTemplate !$= "")
						AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
					else
						AIMessageThread(%task.chat, %client, -1);
				}
				else if (%task.targetObject > 0)
				{
					%type = %task.targetObject.getDataBlock().getName();
					if (%type $= "Flag")
						AIMessageThreadTemplate("DefendBase", "ChatSelfDefendFlag", %client, -1);
					else if (%type $= "GeneratorLarge")
						AIMessageThreadTemplate("DefendBase", "ChatSelfDefendGenerator", %client, -1);
					else if (%type $= "StationVehicle")
						AIMessageThreadTemplate("DefendBase", "ChatSelfDefendVehicle", %client, -1);
					else if (%type $= "SensorLargePulse")
						AIMessageThreadTemplate("DefendBase", "ChatSelfDefendSensors", %client, -1);
					else if (%type $= "SensorMediumPulse")
						AIMessageThreadTemplate("DefendBase", "ChatSelfDefendSensors", %client, -1);
					else if (%type $= "TurretBaseLarge")
						AIMessageThreadTemplate("DefendBase", "ChatSelfDefendTurrets", %client, -1);
				}
			}
		}
	}

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	           %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            //don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk); }
else { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeExpress); }

            return;
         }
      }
      else
         %task.useTeleport = false;
   }
	else if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
   //--------------------------------------------------- monitor telporter - end -

   //first, check for a vehicle to engage
   if (%task.engageVehicle > 0 && isObject(%task.engageVehicle))
   {
      %client.stop();
      %client.clearStep();
      %client.setEngageTarget(-1);
      %client.setTargetObject(%task.engageVehicle, 300, "Missile");
   }
   else
   {
      //clear the target vehicle...
      %client.setTargetObject(-1);

      //see if we're engaging a player
      if (%client.getEngageTarget() > 0)
      {
         //too far, or killed the enemy - return home
         %distance = %client.getPathDistance(%task.location);
         if (%client.getStepStatus() !$= "InProgress" || %distance > 75)
         {
            //%client.setEngageTarget(-1);
            //%client.stepMove(%task.location, 8.0);
			//don't move like a maniac when near lava - roberto
			if (%client.inDanger) { %client.stepMove(%task.location, 8.0, $AIModeWalk); }
			else { %client.stepMove(%task.location, 8.0, $AIModeExpress); }
         } 
      }
	   
      //else see if we have a target to begin attacking
      if (%task.engageTarget > 0 && AIClientIsAlive(%task.engageTarget))
      {
         if (%client.player.getInventory(SniperRifle) > 0 && %client.player.getInventory(EnergyPack) > 0)
	 {
            %client.stepRangeObject(%task.engageTarget.player.getWorldBoxCenter(), "BasicSniperShot", 150, 500, %task.location);
            %client.aimAt(%task.engageTarget.player.getWorldBoxCenter(), 8000);
            %client.setEngageTarget(%task.engageTarget);
         }
         else
	    %client.stepEngage(%task.engageTarget);
      }
	      
      //else move to a random location around where we are defending
      else if (%client.getStepName() !$= "AIStepIdlePatrol")
      {
         %dist = VectorDist(%client.player.getWorldBoxCenter(), %task.location);
	 if (%dist < 10)
	 {
            //dissolve the human control link and re-evaluate the weight
            if (%task == %client.objectiveTask)
	    {
	       if (aiHumanHasControl(%task.issuedByClient, %client))
	       {
	          aiReleaseHumanControl(%client.controlByHuman, %client);

		  //should re-evaluate the current objective weight
		  %inventoryStr = AIFindClosestInventories(%client);
		  %client.objectiveWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
               }
            }

            %client.stepIdle(%task.location);
         }
         else
	    //%client.stepMove(%task.location, 8.0);
         //don't move like a maniac when near lava - roberto
		if (%client.inDanger) { %client.stepMove(%task.location, 8.0, $AIModeWalk); }
		else { %client.stepMove(%task.location, 8.0, $AIModeExpress); }
      }
   }

   //see if we're supposed to be engaging anyone...
   if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
      %client.setEngageTarget(%client.shouldEngage);
}

//---------------------------------------------------------------------------------------------------------------------------- AIO Defense Patrol ---
//this is a new objective in beta. bot will walk from location to location doing the default defense stuff.
//*** note to lagg redo this for more than 1 path like you did for vehicle objectives *** (ahh No need Too - Lagg 9/20/2006)

//Requires a simgroup titled T<%team#>PatrolPath<#> (example: T1PatrolPath1)  including 'markers' to set as locations to defend.
//When placing this AIOObjectiveMarket in the AI Editor you must add a dynamic field named "Path" and put a value of "1" or "2" or any
//number you want. BUT ONLY ONE NUMBER. Exit the AI Editor and creat a simgroup for this path with the NUMBER that you placed as the
//value of the path. EXAMPLE: T1PatrolPath1 or T2PatrolPath4 stands for team1patrolpath1, team2patrolpath4 only one simgroup per AIO
//Objective marker inside that simgroup you place markers at the exact location the bot will walk to and stand gaurd for a rondom
//amount of time and then will choose another location from the simgroup at ramdom. Place as many location markers in thgere as you
//lik.But make sure the location is pathable.

function AIODefensePatrol::weight(%this, %client, %level, %minWeight, %inventoryStr)
{

   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }

   if (Game.class $= "CTFGame")
   {
		//if we're in danger of losing and this is an emergency task only run if needed  - roberto
		if (%this.ctfEmergency && $emergencyFlagDefender[%client.team] == false)
		{
			return 0;
		}
		//in CTF: run this only if our generators were destroyed enough time - roberto
		else if (%this.urgentGenDefender && $AIGenTotalDestroyed[%client.team] < $AIDefendGenStart)
		{
			return 0;
		}

      //in ctf don't defend stuff if we're over 9000 miles away - roberto
		if (VectorDist(%this.location, %client.player.position) > 200)
		{
			return 0;
		}

      //if the enemy team is full of bots and we have humans in our team, VERY low priority - roberto
      if ( isObject(%this.targetObjectId) && $Host::AdaptiveCTFBots )
      {
         %objType = %this.targetObjectId.getDataBlock().getName();

         if (%objType $= "Flag")
         {
            %enemyTeam = 1;
            if (%client.team == 1) { %enemyTeam = 2; }

            if ( $AIHumansInTeam[%client.team] >= 1 && $AIHumansInTeam[%enemyTeam] <= 0 )
            {
               return 1500;
            }
         }
      }
   }

   // if were playing CnH, check who owns this
   if (%this.targetObjectId > 0  && %this.defense > 0)
   {
      if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team != %client.team)
      {
         return 0;
      }

      //don't defend unless the entire group is powered
      if (%this.targetObjectId.getGroup().powerCount <= 0)
      {
         return 0;
      }
   }
   else if (%this.targetObjectId > 0  && %this.offense > 0)
   {
      if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
         return 0;

      //don't camp unless base is down
      if (%this.targetObjectId.getGroup().powerCount > 0)
         return 0;
   }
   

   //---MISSION SPECIFIC 1---//
   //1.Copy this entire function to your mission extras script.
   //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
   //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
   //---MISSION SPECIFIC 1---//

	//do a quick check to disqualify this objective if it can't meet the minimum weight
	if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	{
		if (%this.targetObjectId > 0 && %this.issuedByClientId == %client.controlByHuman)
		{
			if ($AIWeightHumanIssuedCommand < %minWeight)
				return 0;
		}
		else
			return 0;
	}

	%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);


   //add more and more weight if the gens are totally raped - roberto
   if (Game.class $= "CTFGame")
   {
      if (%this.urgentGenDefender)
      {
         if (%weight < 3825)
         {
            %addWeightOnce = $AIDefendGenStart + $AIDefendGenStart;

            if ($AIGenTotalDestroyed[%client.team] >= %addWeightOnce)
            {
               %weight += 150;

               %addWeightTwice = $AIDefendGenStart + $AIDefendGenStart + $AIDefendGenStart;
               if ($AIGenTotalDestroyed[%client.team] >= %addWeightOnce)
               {
                  %weight += 150;
               }
            }
         }
      }
   }

        //if defensive  the object has been destroyed, reduce the weight / if offensive increase the weight if destroyed (Camping !)
	if (%this.targetObjectId > 0 && %this.defense > 0)
	{

		//see if we were forced on the objective
		if (%this.issuedByClientId == %client.controlByHuman && %weight < $AIWeightHumanIssuedCommand)
			%weight = $AIWeightHumanIssuedCommand;

		//else see if the object has been destroyed - slight modification here - Lagg... 11-7-2003
	   //else if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
           else if (!isObject(%this.targetObjectId) || %this.targetObjectId.isDisabled())
			%weight -= 320;

      //---MISSION SPECIFIC 2---//
      //1.Copy this entire function to your mission extras script.
      //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
      //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
      //---MISSION SPECIFIC 2---//
	}
        else if (%this.targetObjectId > 0 && %this.offense > 0)
	{

		//see if we were forced on the objective
		if (%this.issuedByClientId == %client.controlByHuman && %weight < $AIWeightHumanIssuedCommand)
			%weight = $AIWeightHumanIssuedCommand;

		// - slight modification here - Lagg... 11-7-2003
                else if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() !$= "Destroyed")
			return 0;//don't camp if not destroyed
                if (%client.player.getInventory("InventoryDeployable") > 0)
                   return 0;
	}

   //add or decrease weight if the objective has inBase or inRoom fields - roberto
   //------------------------------------------------------------------------------
   if (%this.inBase)
   {
      if (!%client.inBase)
      {
         if (Game.class $= "SiegeGame")
         {
            %weight -= 100;
         }
      }
      else
      {
         if (%weight <= 4000) { %weight += 100; }
      }
   }

   if (%this.inRoom)
   {
      if (!%client.inRoom)
      {
         %weight -= 100;
      }
      else
      {
         if (%weight <= 4000) { %weight += 100; }
      }
   }
   //------------------------------------------------------------------------------
	
	return %weight;
}

function AIODefensePatrol::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIDefensePatrol);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIODefensePatrol::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//--------------------------------------------------------------------------------------------------------------------------------- AI Defense Patrol ---

function AIDefensePatrol::initFromObjective(%task, %objective, %client)
{
   //initialize the task vars from the objective
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
   if (%objective.Location !$= "")
      %task.location = %objective.location;
   else
      %task.location = %objective.targetObjectId.getWorldBoxCenter();

   %task.equipment = %objective.equipment;
   %task.desiredEquipment = %objective.desiredEquipment;
   %task.buyEquipmentSet = %objective.buyEquipmentSet;

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;

   //ctfEmergency is automatically false if the field doesn't exist
   if (%objective.ctfEmergency $= "")
   {
      %task.ctfEmergency = false;
   }
   else
   {
      %task.ctfEmergency = %objective.ctfEmergency; //if objective must only be completed if in danger of losing
   }

   //urgentGenDefender is automatically false if the field doesn't exist
   if (%objective.urgentGenDefender $= "")
   {
      %task.urgentGenDefender = false;
   }
   else
   {
      %task.urgentGenDefender = %objective.urgentGenDefender; //if objective must only be completed if gens were destroyed enough times
   }
   
   %task.path = %objective.Path;

   if (%task.buyEquipmentSet $= "randomSet")
   {
      //pick a random equipment set...
	      %randNum = getRandom();
	      if (%randNum < 0.2)
              {
                    %task.desiredEquipment = "Plasma PlasmaAmmo";
                    %task.buyEquipmentSet = "HeavyShieldOff";
              }
	      else if (%randNum < 0.4)
              {
                    %task.desiredEquipment = "Plasma PlasmaAmmo";
                    %task.buyEquipmentSet = "MediumEnergySet";
              } 
	      else if (%randNum < 0.6)
              {
                    %task.desiredEquipment = "Plasma PlasmaAmmo";
                    %task.buyEquipmentSet = "LightCloakSet";
              }
              else if (%randNum < 0.8)
              {
                    %task.desiredEquipment = "Plasma PlasmaAmmo";
                    %task.buyEquipmentSet = "HeavyShieldSet";
              }
              else
              {
                    %task.desiredEquipment = "Plasma PlasmaAmmo";
                    %task.buyEquipmentSet = "HeavyRepairSet";
              }
   }

   %task.issuedByClient = %objective.issuedByClientId;
   %task.chat = %objective.chat;

   //initialize other task vars
   %task.sendMsg = true;
   %task.sendMsgTime = 0;
   
   %task.engageTarget = -1;

   %task.timeCheck = true;
}

function AIDefensePatrol::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(15);
   %client.inPerimeter = false;
   %client.needEquipment = AINeedEquipment(%task.desiredEquipment, %client);

   //even if we don't *need* equipemnt, see if we should buy some... 
   if (! %client.needEquipment && %task.buyEquipmentSet !$= "")
   {
      //see if we could benefit from inventory
      %needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
      %result = AIFindClosestInventory(%client, %needArmor);
      %closestInv = getWord(%result, 0);
      %closestDist = getWord(%result, 1);
      if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
      {
         %result = AIFindClosestEnemy(%client, 200, $AIClientLOSTimeout);
	 %closestEnemy = getWord(%result, 0);
	 %closestEnemydist = getWord(%result, 1);

	 if (%closestEnemy <= 0 || (%closestEnemyDist > %closestDist * 1.5))
            %client.needEquipment = true;
      }
   }

   //mark the current time for the buy inventory state machine
   %task.buyInvTime = getSimTime();

   //set a flag to determine if the objective should be re-aquired when the object is destroyed
   %task.reassignOnDestroyed = false;

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0 && isObject(%task.targetObject) )
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
		}
	}
    //--------- End teleporters check ---------
}

function AIDefensePatrol::retire(%task, %client)
{
   %task.engageVehicle = -1;
   %client.setTargetObject(-1);
}

function AIDefensePatrol::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;//this will reaccess on destroyed if targetobjectid > 0

   %player = %client.player;
   if (!isObject(%player))
      return;

   %hasMissile = (%player.getInventory("MissileLauncher") > 0) && (%player.getInventory("MissileLauncherAmmo") > 0);

   //if we're defending with a missile launcher, our first priority is to take out vehicles...
   //see if we're already attacking a vehicle...
   if (%task.engageVehicle > 0 && isObject(%task.engageVehicle) && %hasMissile)
   {
      //set the weight
      %task.setWeight(%task.baseWeight);
		return;
   }

   //search for a new vehicle to attack
   %task.engageVehicle = -1;
   %losTimeout = $AIClientMinLOSTime + ($AIClientLOSTimeout * %client.getSkillLevel());
   %result = AIFindClosestEnemyPilot(%client, 300, %losTimeout);
   %pilot = getWord(%result, 0);
   %pilotDist = getWord(%result, 1);

   //if we've got missiles, and a vehicle to attack...
   if (%hasMissile && AIClientIsAlive(%pilot))
   {
      %task.engageVehicle = %pilot.vehicleMounted;
      %client.needEquipment = false;
   }

   //otherwise look for a regular enemy to fight...
   else
   {
      %result = AIFindClosestEnemyToLoc(%client, %task.location, 100, %losTimeout);
      %closestEnemy = getWord(%result, 0);
      %closestdist = getWord(%result, 1);
	   
      //see if we found someone
      if (%closestEnemy > 0)
         %task.engageTarget = %closestEnemy;
      else
      {
         %task.engageTarget = -1;

	 //see if someone is near me...
	 %result = AIFindClosestEnemy(%client, 100, %losTimeout);
		   %closestEnemy = getWord(%result, 0);
		   %closestdist = getWord(%result, 1);
			if (%closestEnemy <= 0 || %closestDist > 70)
				%client.setEngageTarget(-1);
      }
   }

   //in ctf, if we're in danger of losing and this is an emergency task only run if needed  - roberto
   if (%task.ctfEmergency && $emergencyFlagDefender[%client.team] == false)
   {
      %task.setWeight(0);
   }
   //in CTF: run this only if our generator was destroyed enough time - roberto
   else if (%task.urgentGenDefender && $AIGenTotalDestroyed[%client.team] < $AIDefendGenStart)
   {
      %task.setWeight(0);
   }
   else //this is not a speial defense patrol task, continue as usual
   {
       %task.setWeight(%task.baseWeight); 
   }
}

function AIDefensePatrol::monitor(%task, %client)
{
   //first, buy the equipment
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
         %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(15);
	 %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
         //if this task is the objective task, choose a new objective
	 if (%task == %client.objectiveTask)
	 {
	    AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	 }
	 return;
      }
   }
   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();

   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
         if (%client.isAIControlled())
	 {
	    if (%task.chat !$= "")
	    {
	       %chatMsg = getWord(%task.chat, 0);
	       %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
                  AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, -1);
	    }
	    else if (%task.targetObject > 0)
	    {
	       %type = %task.targetObject.getDataBlock().getName();
	       if (%type $= "Flag")
	          AIMessageThreadTemplate("DefendBase", "ChatSelfDefendFlag", %client, -1);
	       else if (%type $= "GeneratorLarge")
		  AIMessageThreadTemplate("DefendBase", "ChatSelfDefendBase", %client, -1);
	       else if (%type $= "StationVehicle")
		  AIMessageThreadTemplate("DefendBase", "ChatSelfDefendVehicle", %client, -1);
	       else if (%type $= "SensorLargePulse")
		  AIMessageThreadTemplate("DefendBase", "ChatSelfDefendSensors", %client, -1);
	       else if (%type $= "SensorMediumPulse")
		  AIMessageThreadTemplate("DefendBase", "ChatSelfDefendSensors", %client, -1);
	       else if (%type $= "TurretBaseLarge")
		  AIMessageThreadTemplate("DefendBase", "ChatSelfDefendTurrets", %client, -1);
	    }
         }
      }
   }

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	           %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            //don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk); }
else { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeExpress); }

            return;
         }
      }
      else
         %task.useTeleport = false;
   }
	else if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
   //--------------------------------------------------- monitor telporter - end -       

   //if the defend location task has an object, set the "reset" flag
   if (%task == %client.objectiveTask && isObject(%task.targetObject))
   {
      //if (%task.targetObject.getDamageState() !$= "Destroyed")
      if (%task.targetObject.isDisabled())
         %task.reassignOnDestroyed = true;
      else
      {
         if (%task.reassignOnDestroyed)
         {
            AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	    return;
         }
      }
   }

   //first, check for a vehicle to engage
   if (%task.engageVehicle > 0 && isObject(%task.engageVehicle))
   {
      %client.stop();
      %client.clearStep();
      %client.setEngageTarget(-1);
      %client.setTargetObject(%task.engageVehicle, 300, "Missile");
   }
   else
   {
      //clear the target vehicle...
      %client.setTargetObject(-1);

      //see if we're engaging a player
      if (%client.getEngageTarget() > 0)
      {
         //too far, or killed the enemy - return home
	 if (%client.getStepStatus() !$= "InProgress" || %distance > 100)
	 {
	    %client.setEngageTarget(-1);
	    //%client.stepMove(%task.location, 4.0);
		//don't move like a maniac when near lava - roberto
		if (%client.inDanger) { %client.stepMove(%task.location, 4.0, $AIModeWalk); }
		else { %client.stepMove(%task.location, 4.0, $AIModeExpress); }
	 } 
      }
	   
      //else see if we have a target to begin attacking
      else if (%task.engageTarget > 0)
         %client.stepEngage(%task.engageTarget);
	      
      //else move to a location taken from the T#PatrolPath# simgroup
      else
      {
         %dist = VectorDist(%client.player.getWorldBoxCenter(), %task.location);
	 if (%dist < 6)
	 {
            //random time update task location - Lagg...
            if (%task.timeCheck)
            {
               %task.watTime = getSimTime();
               %task.ranTime = getRandom(0, 20) * 1000;//random time to stepidle at defense points
               %task.timeCheck = false;
            }
            if (getSimTime() > (%task.watTime + %task.ranTime))
            {
               if (Game.class !$= "SiegeGame")
                  %patrolSpots = nameToId("T" @ %client.team @ "PatrolPath" @ %task.path);
               else
               {
                  %T = %game.offenseTeam == 1 ? 1 : 2;
                  %patrolSpots = nameToId("T" @ %T @ "PatrolPath" @ %task.path);
               }
               %count = %patrolSpots.getCount();
               %ranSpot = mFloor(getRandom(0, %count - 1));
               %task.location = %patrolSpots.getObject(%ranSpot).position;
               %task.timeCheck = true;
            }
            //dissolve the human control link and re-evaluate the weight
	    if (%task == %client.objectiveTask)
	    {
	       if (aiHumanHasControl(%task.issuedByClient, %client))
	       {
	          aiReleaseHumanControl(%client.controlByHuman, %client);

		  //should re-evaluate the current objective weight
		  %inventoryStr = AIFindClosestInventories(%client);
		  %client.objectiveWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
               }
            }
	    %client.stepIdle(%task.location);
	 }
	 else
            %client.stepMove(%task.location, 4.0, $AIModeWalk);
      }
   }

   //see if we're supposed to be engaging anyone...
   if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
      %client.setEngageTarget(%client.shouldEngage);
}

//----------------------------------------------------------------------------------------------- AIO Deploy Equipment ---
//modified here so bot will continue deploying if has equip and not get bumped
//can be set at a realistic weight now, like 3400               - Lagg... 4-12-2003
//if you want this to be around a deployed MPB, just set the dynamic field "mode = MPB" and he will deploy only
//if a team deployed MPB is within 150 meters of this objectives location.

//If the function ShapeBaseImageData::testInvalidDeployConditions() changes at all, those changes need to be reflected here

function AIODeployEquipment::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //make sure the deploy objective is valid
   if (%this.isInvalid)
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }

   if (%this.mode $= "MPB")
   {      
      %result = AIFindDeployedMPB(%client);
      %clM = getWord(%result, 0);
      if (!isObject(%clM) || VectorDist(%clM.getWorldBoxCenter(), %this.position) > 150)
         return 0;
   }

   //---MISSION SPECIFIC 1---//
   //1.Copy this entire function to your mission extras script.
   //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
   //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
   //---MISSION SPECIFIC 1---//

   if (Game.class $= "CTFGame")
   {
      //for GenDefend mode, only deploy equipment if our gen was destroyed last time and we still didn't deploy everything we needed - roberto
      if (%this.GenDefend == true)
      {
          if ($AIGenDisabledCount[%client.team] <= 0)
          {
          	   return 0; //stop if we already deployed everything for gen defense
          }
      }
   }

   // this checks for CnH and works with the offense/defense tags associated with each AIObjective Marker
   //if offensive the deply is assumed to be camping a destroyed object - Lagg-Alot... 7/18/2009
   if (%this.targetObjectId > 0  && %this.defense > 0)
   {
      if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team != %client.team)
         return 0;

      //don't defend unless the entire group is powered
      if (%this.targetObjectId.getGroup().powerCount <= 0)
         return 0;
   }
   else if (%this.targetObjectId > 0  && %this.offense > 0)
   {
      if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
         return 0;

      //don't camp if target object is not destroyed
      if (%this.targetObjectId.getDamageState() !$= "Destroyed")
         return 0;
   }

   //first, make sure we haven't deployed too many...
   if (%this.equipment $= "TurretOutdoorDeployable" || %this.equipment $= "TurretIndoorDeployable")
      %maxAllowed = countTurretsAllowed(%this.equipment);
   else
      %maxAllowed = $TeamDeployableMax[%this.equipment];

   if ($TeamDeployedCount[%client.team, %this.equipment] >= %maxAllowed)
      return 0;

   //if badly damaged and we have a repair pack, don't waste time
   %damage = %client.player.getDamagePercent();
   if (%client.player.getInventory("RepairPack") > 0 && %damage > 0.3)
      return 0;

   //now make sure there are no other items in the way...
   InitContainerRadiusSearch(%this.location, $MinDeployableDistance, $TypeMasks::VehicleObjectType |
  	                                             $TypeMasks::MoveableObjectType |
  	                                             $TypeMasks::StaticShapeObjectType |
  	                                             $TypeMasks::TSStaticShapeObjectType | 
  	                                             $TypeMasks::ForceFieldObjectType |
  	                                             $TypeMasks::ItemObjectType | 
  	                                             $TypeMasks::PlayerObjectType | 
  	                                             $TypeMasks::TurretObjectType);
      %objSearch = containerSearchNext();

      //make sure we're not invalidating the deploy location with the client's own player object
      if (%objSearch == %client.player)
         %objSearch = containerSearchNext();

      //did we find an object which would block deploying the equipment?
      if (isObject(%objSearch))
         return 0;

   //now run individual checks based on the equipment type...
   if (%this.equipment $= "TurretIndoorDeployable")
   {
      //check if there's another turret close to the deploy location
      InitContainerRadiusSearch(%this.location, $TurretIndoorSpaceRadius, $TypeMasks::StaticShapeObjectType);
      %found = containerSearchNext();
      if (isObject(%found))
      {
         %foundName = %found.getDataBlock().getName();
 	 if ((%foundName $= TurretDeployedFloorIndoor) || (%foundName $= "TurretDeployedWallIndoor") ||
           (%foundName $= "TurretDeployedCeilingIndoor") || (%foundName $= "TurretDeployedOutdoor"))
 	    return 0;
      }
 
      //now see if there are too many turrets in the area...
      %highestDensity = 0;
      InitContainerRadiusSearch(%this.location, $TurretIndoorSphereRadius, $TypeMasks::StaticShapeObjectType);
      %found = containerSearchNext();
      while (isObject(%found))
      {
         %foundName = %found.getDataBlock().getName();
 	 if ((%foundName $= "TurretDeployedFloorIndoor") || (%foundName $= "TurretDeployedWallIndoor") ||
           (%foundName $= "TurretDeployedCeilingIndoor") || (%foundName $= "TurretDeployedOutdoor"))
 	 {
 	    //found one
 	    %numTurretsNearby++;//+++++++++++++++++++++++++++++++++++++++++ ahh review this - Lagg... +++
            %nearbyDensity = testNearbyDensity(%found, $TurretIndoorSphereRadius);
 	    if (%nearbyDensity > %highestDensity)
 	       %highestDensity = %nearbyDensity;
         }
 	 %found = containerSearchNext();
      }
 
      if (%numTurretsNearby > %highestDensity)
         %highestDensity = %numTurretsNearby;
 
      //now see if the area is already saturated
      if (%highestDensity > $TurretIndoorMaxPerSphere)
         return 0;
   }
   //--------------------left off here - Lagg...+++++++++++++++++++++++++
 
 	else if (%this.equipment $= "TurretOutdoorDeployable")
 	{
 		//check if there's another turret close to the deploy location
 	   InitContainerRadiusSearch(%this.location, $TurretOutdoorSpaceRadius, $TypeMasks::StaticShapeObjectType);
 	   %found = containerSearchNext();
 	   if (isObject(%found))
 		{
 			%foundName = %found.getDataBlock().getName();     
 			if ((%foundName $= "TurretDeployedFloorIndoor") || (%foundName $= "TurretDeployedWallIndoor") || (%foundName $= "TurretDeployedCeilingIndoor") || (%foundName $= "TurretDeployedOutdoor"))
 				return 0;
 		}
 
 		//now see if there are too many turrets in the area...
 	   %highestDensity = 0;
 	   InitContainerRadiusSearch(%this.location, $TurretOutdoorSphereRadius, $TypeMasks::StaticShapeObjectType);
 	   %found = containerSearchNext();
 	   while (isObject(%found))
 	   {
 	      %foundName = %found.getDataBlock().getName();
 	      if ((%foundName $= "TurretDeployedFloorIndoor") || (%foundName $= "TurretDeployedWallIndoor") || (%foundName $= "TurretDeployedCeilingIndoor") || (%foundName $= "TurretDeployedOutdoor"))
 	      {
 				//found one
 				%numTurretsNearby++;
 
 				%nearbyDensity = testNearbyDensity(%found, $TurretOutdoorSphereRadius);
 				if (%nearbyDensity > %highestDensity)
 					%highestDensity = %nearbyDensity;     
 	      }
 	     %found = containerSearchNext();
 	   }
 
 		if (%numTurretsNearby > %highestDensity)
 			%highestDensity = %numTurretsNearby;
 
 		//now see if the area is already saturated
 		if (%highestDensity > $TurretOutdoorMaxPerSphere)
 			return 0;
 	}

	//check equipment requirement
	%needEquipment = AINeedEquipment(%this.equipment, %client);

        //if don't need equipment, see if we've past the "point of no return", and should continue regardless
 	if (!%needEquipment && %this.clientLevel1 == %client)
 	{
                
 		
 		//if we're too far from the inv to go back, or we're too close to the deploy location, force continue
 		////if (%closestDist > 50 && VectorDist(%client.player.getWorldBoxCenter(), %task.location) < 50) - changed - Lagg...
                if (%closestDist > 25)
 		{
 			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
 			if (%weight < $AIWeightContinueDeploying)
 				%weight = $AIWeightContinueDeploying;

 			return %weight;
 		}
 	}

	//if this bot is linked to a human who has issued this command, up the weight
	if (%this.issuedByClientId == %client.controlByHuman)
	{
		//make sure we have the potential to reach the minWeight
		if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
		{
			if ($AIWeightHumanIssuedCommand < %minWeight)
				return 0;
			else
				%weight = $AIWeightHumanIssuedCommand;
		}
		else
		{
			// calculate the default...
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
			if (%weight < $AIWeightHumanIssuedCommand)
				%weight = $AIWeightHumanIssuedCommand;
		}
	}
	else
	{
		//make sure we have the potential to reach the minWeight
		if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			return 0;

		// calculate the default...
		%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	}

   if (!%needEquipment)
   {
      %weight += 400;
   }

   //add or decrease weight if the objective has inBase or inRoom fields - roberto
   //------------------------------------------------------------------------------
   if (%this.inBase)
   {
      if (%client.inBase)
      {
         if (%weight <= 4000) { %weight += 100; }
      }
      else
      {
         if (Game.class $= "SiegeGame")
         {
            %weight -= 100;
         }
      }
   }

   if (%this.inRoom)
   {
      if (%client.inRoom)
      {
         if (%weight <= 4000) { %weight += 200; }
      }
      else
      {
         %weight -= 100;
      }
   }
   //------------------------------------------------------------------------------

   //if we have humans in our team, decrease the weight for deploying turrets 
   if (Game.class $= "CTFGame")
   {
      if ( $Host::AdaptiveCTFBots )
      {
         %enemyTeam = 1;
         if (%client.team == 1) { %enemyTeam = 2; }

         //only if the enemy team is full of bots, so it will be fair
         if ( $AIHumansInTeam[%client.team] >= 1 && $AIHumansInTeam[%enemyTeam] <= 0 )
         {
            if (%this.equipment $= "TurretOutdoorDeployable" || %this.equipment $= "TurretIndoorDeployable")
            {
               if ( !$AITeamLosing[%client.team] || $Host::BotDifficulty !$= "Easy" ) //if medium/hard difficulty, or if we're winning on easy, decrease
               {
                  %weight -= 350;
               }
               else
               {
                  %weight -= 250; //easy difficulty but not winning
               }
            }
         }
      }
   }

	return %weight;
}

function AIODeployEquipment::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIDeployEquipment);
   %task = %client.objectiveTask;
   %task.initFromObjective(%this, %client);
}

function AIODeployEquipment::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//---------------------------------------------------------------------------------------------- AI Deploy Equipment ---

function AIDeployEquipment::initFromObjective(%task, %objective, %client)
{
	//initialize the task vars from the objective
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.location = %objective.location;
   %task.position = %objective.position;
   %task.equipment = %objective.equipment;
	%task.buyEquipmentSet = %objective.buyEquipmentSet;
	%task.desiredEquipment = %objective.desiredEquipment;
	%task.issuedByClient = %objective.issuedByClientId;
	%task.chat = %objective.chat;
    %task.mode = %objective.mode;
    %task.GenDefend = %objective.GenDefend; //should we deploy this after our gen was destroyed last time? - roberto

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;


	//initialize other task vars
	%task.sendMsg = true;
	%task.sendMsgTime = 0;
        %task.useTeleport = false;

	//use the Y-axis of the rotation as the desired direction of deployement,
	//and calculate a walk to point 3 m behind the deploy point. 
	%task.deployDirection = MatrixMulVector("0 0 0 " @ getWords(%objective.getTransform(), 3, 6), "0 1 0");
	%task.deployDirection = VectorNormalize(%task.deployDirection);
}

function AIDeployEquipment::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(15);
	
   %client.needEquipment = AINeedEquipment(%task.equipment, %client);
   
   //mark the current time for the buy inventory state machine
   %task.buyInvTime = getSimTime();

   %task.passes = 0;
   %task.deployAttempts = 0;
   %task.checkObstructed = false;
   %task.waitMove = 0;

   //add some for picking up deployables from ground - Laggg... 2-26-2004
   %task.pickupPack = -1;
   %task.usingInv = false;

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0)
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
		}
	}
    //--------- End teleporters check ---------
}

function AIDeployEquipment::retire(%task, %client)
{
}

function AIDeployEquipment::weight(%task, %client)
{
	//update the task weight
	if (%task == %client.objectiveTask)
		%task.baseWeight = %client.objectiveWeight;

   %task.setWeight(%task.baseWeight);
}

function findTurretDeployPoint(%client, %location, %attempt)
{
   %player = %client.player;
	if (!isObject(%player))
		return "0 0 0";

	%feetPos = posFromTransform(%player.getTransform());
	%temp = VectorSub(%location, %feetPos);
   %temp2 = getWord(%temp, 0) @ " " @ getWord(%temp, 1) @ " 0";
	%facingVector = VectorNormalize(%temp2);
	%aimPoint = VectorAdd(%feetPos, %facingVector);
	//assume that there will be 10 attempts
	%height = getWord(%location, 2) + 1.0 - (0.2 * %attempt);
   %aimAt = getWord(%aimPoint, 0) @ " " @ getWord(%aimPoint, 1) @ " " @ %height;
   return %aimAt;
}

//modified range to deploy to make easier for outdoor inventory deployments - Lagg... - 4-12-2003
function AIDeployEquipment::monitor(%task, %client)
{
   //first, make sure we haven't deployed too many...
   if (%task.equipment $= "TurretOutdoorDeployable" || %task.equipment $= "TurretIndoorDeployable")
      %maxAllowed = countTurretsAllowed(%task.equipment);
   else
      %maxAllowed = $TeamDeployableMax[%task.equipment];

   if ($TeamDeployedCount[%client.team, %task.equipment] >= %maxAllowed)
   {
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         AIUnassignClient(%client);
	 Game.AIChooseGameObjective(%client);
         return;
      }
   }

   //make sure this was not deployed already
   InitContainerRadiusSearch(%task.location, $MinDeployableDistance, $TypeMasks::StaticShapeObjectType);
   %found = containerSearchNext();
   if (isObject(%found))
   {
      if (%found.team == %client.team)
      {
         if (%task == %client.objectiveTask)
	 {
            //error("AIDeployEquipment::monitor - this objective was completed/deployed already :(");
            AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	    return;
	 }
      }
   }

   //first, see if we still need the deployable pack - Lagg... 2-25-2004
   %itemType = getWord(%task.equipment, 0);
   if (%client.player.getInventory(%itemType) > 0)
   {
      //error("AIDeployEquipment - we have Item Type = " @ %itemType SPC getTaggedString(%client.name));
      %client.needEquipment = false;
      %task.setMonitorFreq(15);
   }

   //then, buy the equipment or find it if you can :) - Lagg... 2-25-2004
   else if (%client.needEquipment)
   {
      // check to see if there's a deployable pack nearby
      %closestItem = -1;
      %closestItemDist = 32767;
      %itemType = getWord(%task.equipment, 0);

      //search the AIItemSet for a deployable pack (someone might have dropped one...)
      %armor = %client.player.getArmorSize();
      %itemCount = $AIItemSet.getCount();
      for (%i = 0; %i < %itemCount; %i++)
      {
         %item = $AIItemSet.getObject(%i);
         if (%item.getDataBlock().getName() $= %itemType && !%item.isHidden())
	 {
            %cant = false;
            if ((%itemType $= "InventoryDeployable" || %itemType $= "TurretOutdoorDeployable" || %itemType $= "TurretIndoorDeployable") && %armor $= "Light")
            {
               //error("AIDeployEquipment - we found on floor but LIGHT ARMOR = " @ %itemType SPC getTaggedString(%client.name));
               %cant = true;
            }
            %dist = %client.getPathDistance(%item.getWorldBoxCenter());
	    if (%dist > 0 && %dist < %closestItemDist && !%cant)
	    {
               %closestItem = %item;
	       %closestItemDist = %dist;
	    }
         }
      }

      //choose whether we're picking up the closest pack, or buying from an inv station...
      if ((isObject(%closestItem) && %closestItem != %task.pickupPack) || (%task.buyInvTime != %client.buyInvTime))
      {
         %task.pickupPack = %closestItem;

	 //initialize the inv buying
	 %task.buyInvTime = getSimTime();
	 AIBuyInventory(%client, %task.equipment, %task.buyEquipmentSet, %task.buyInvTime);

	 //now decide which is closer
	 if (isObject(%closestItem))
	 {
            //added a little here to check for power - Lagg... - 3-25-2003
	    if (isObject(%client.invToUse) && (%client.invToUse.isEnabled()) && (%client.invToUse.isPowered()))
	    {
               %dist = %client.getPathDistance(%client.invToUse.getWorldBoxCenter());
	       if (%dist < %closestItemDist)
	          %task.usingInv = true;
	       else
		  %task.usingInv = false;
            }
	    else
               %task.usingInv = false;
	 }
	 else
	    %task.usingInv = true;
      }

      //now see if we found a closer deployable pack
      if (!%task.usingInv && isObject(%task.pickupPack))
      {
         %client.stepMove(%task.pickupPack.position, 0.25);
         %distToPack = %client.getPathDistance(%task.pickupPack.position);
         if (%distToPack < 10 && %client.player.getMountedImage($BackpackSlot) > 0)
	    %client.player.throwPack();

         //error("AIDeployEquipment - Picking Up Deployable Pack = " @ %itemType SPC getTaggedString(%client.name));

	 //and we're finished until we actually have the new pack... we should - Lagg...
	 return;
      }
      else
      {
         //error("AIDeployEquipment - we are using inventory Item Type = " @ %itemType SPC getTaggedString(%client.name));
         %task.setMonitorFreq(5);
         %result = AIBuyInventory(%client, %task.equipment, %task.buyEquipmentSet, %task.buyInvTime);
	 if (%result $= "InProgress")
            return;
	 else if (%result $= "Finished")
	 {
	    %task.setMonitorFreq(30);
            %client.needEquipment = false;

            //if we made it past the inventory buying, reset the inv time
	    %task.buyInvTime = getSimTime();
         }
	 else if (%result $= "Failed")
	 {
            //if this task is the objective task, choose a new objective
	    if (%task == %client.objectiveTask)
            {
	       AIUnassignClient(%client);
	       Game.AIChooseGameObjective(%client);
	    }
	    return;
	 }
      }
   }

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            //don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk); }
else { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeExpress); }

            return;
         }
      }
      else
         %task.useTeleport = false;
   }
	else if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
   //--------------------------------------------------- monitor telporter - end -

   //make sure we still have equipment
		%client.needEquipment = AINeedEquipment(%task.equipment, %client);
      if (%client.needEquipment)
      {
         //if this task is the objective task, choose a new objective
         if (%task == %client.objectiveTask)
			{
            AIUnassignClient(%client);
	         Game.AIChooseGameObjective(%client);
				return;
			}
      }

	//chat
	if (%task.sendMsg)
	{
		if (%task.sendMsgTime == 0)
			%task.sendMsgTime = getSimTime();
		else if (getSimTime() - %task.sendMsgTime > 7000)
		{
			%task.sendMsg = false;
		   if (%client.isAIControlled())
			{
				if (%task.chat !$= "")
				{
					%chatMsg = getWord(%task.chat, 0);
					%chatTemplate = getWord(%task.chat, 1);
					if (%chatTemplate !$= "")
						AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
					else
						AIMessageThread(%task.chat, %client, -1);
				}
			}
		}
	}

   //see if we're supposed to be engaging anyone...
   if (AIClientIsAlive(%client.shouldEngage))
   {
	   %hasLOS = %client.hasLOSToClient(%client.shouldEngage);
	   %losTime = %client.getClientLOSTime(%client.shouldEngage);
	   if (%hasLOS || %losTime < 1000)
         %client.setEngageTarget(%client.shouldEngage);
      else
         %client.setEngageTarget(-1);
   }
   else
      %client.setEngageTarget(-1);

	//calculate the deployFromLocation
	%factor = -1 * (4 - (%task.passes * 0.5));//changed to 4m behind to improve proformance --- Lagg... 3-15-2003
   %task.deployFromLocation = VectorAdd(%task.location,VectorScale(%task.deployDirection, %factor));

	//see if we're within range of the deploy location
   %clLoc = %client.player.position;
   %distance = VectorDist(%clLoc, %task.deployFromLocation);
	%dist2D = VectorDist(%client.player.position, getWords(%task.deployFromLocation, 0, 1) SPC getWord(%client.player.position, 2));

	//set the aim when we get near the target...  this will be overwritten when we're actually trying to deploy
	if (%distance < 10 && %dist2D < 10)
      %client.aimAt(%task.location, 1000);

   if ((%client.pathDistRemaining(20) > %distance + 0.25) || %dist2D > 0.5)
	{
		%task.deployAttempts = 0;
		%task.checkObstructed = false;
		%task.waitMove = 0;
	//don't move like a maniac when near lava - roberto
	if (%client.inDanger) { %client.stepMove(%task.deployFromLocation, 0.25, $AIModeWalk); }
	else { %client.stepMove(%task.deployFromLocation, 0.25, $AIModeExpress); }
	   %task.setMonitorFreq(15);
		return;
	}
   
	if (%task.deployAttempts < 10 && %task.passes < 5 && !AIClientIsAlive(%client.getEngageTarget()))
	{
		//dissolve the human control link
		if (%task == %client.objectiveTask)
			aiReleaseHumanControl(%client.controlByHuman, %client);

	   %task.setMonitorFreq(3);
      %client.stop();
		if (%task.deployAttempts == 0)
			%deployPoint = %task.location;
		else
	      %deployPoint = findTurretDeployPoint(%client, %task.location, %task.deployAttempts);
      if(%deployPoint !$= "")
      {
         // we have possible point
         %task.deployAttempts++;
         %client.aimAt(%deployPoint, 2000);

			//try to deploy the backpack
			%client.deployPack = true;
         %client.lastDeployedObject = -1;
         %client.player.use(Backpack);
         
         // check if pack deployed
         if (isObject(%client.lastDeployedObject))
			{
				//we deployed or tried to deploy equipment because objective's GenDefend is true
				//keep decreasing the value. At 0 we won't deploy GenDefend objectives. - Roberto
				if (%task.GenDefend == true) 
				{
					$AIGenDisabledCount[%client.team] = $AIGenDisabledCount[%client.team] - 1;

					if ($AIGenDisabledCount[%client.team] < 0)
					{
						$AIGenDisabledCount[%client.team] = 0; 
					}
					echo("-AI TACTICS- Pack deployed. AIGenDisabledCount for team " @ %client.team @ " is now " @ $AIGenDisabledCount[%client.team]);
				}

				//see if there's a "repairObject" objective for the newly deployed thingy...
				if (%task == %client.objectiveTask)
				{
					%deployedObject = %client.lastDeployedObject;

					//search the current objective group and search for a "repair Object" task...
					%objective = %client.objective;

					//delete any previously associated "AIORepairObject" objective
					if (isObject(%objective.repairObjective))
					{
						AIClearObjective(%objective.repairObjective);
						%objective.repairObjective.delete();
						%objective.repairObjective = "";
                                                clearObjectiveFromTable(%objective.repairObjective);//new function below to stop objective spam - Lagg... 1-27-2004
					}

					//add the repair objective
	            %objective.repairObjective = new AIObjective(AIORepairObject)
		                              {
											      dataBlock = "AIObjectiveMarker";
		                                 weightLevel1 = %objective.weightLevel1 - 50;
		                                 weightLevel2 = 0;
		                                 description = "Repair the " @ %deployedObject.getDataBlock().getName();
                                                 mode = %task.mode;
													targetObjectId = %deployedObject;
													issuedByClientId = %client;
		                                 offense = false;
													defense = true;
		                                 equipment = "RepairPack";
		                              };
					%objective.repairObjective.deployed = true;
					%objective.repairObjective.setTransform(%objective.getTransform());
					%objective.repairObjective.group = %objective.group;
                                        MissionCleanup.add(%objective.repairObjective);

   //if is siege game we have to do this right - Lagg... 11-3-2003
   if (Game.class $= "SiegeGame")
   {
      //the objectives in $ObjectiveQ[1] are all offense objectives, $ObjectiveQ[2] has the defensive ones..
      if (%client.team == game.offenseTeam)
         $ObjectiveQ[1].add(%objective.repairObjective);
      else
         $ObjectiveQ[2].add(%objective.repairObjective);
   }
   else
      $ObjectiveQ[%client.team].add(%objective.repairObjective);


                              //changed to fix Siege gameType wrong Q error - Lagg... 11-5-2003
			      //MissionCleanup.add(%objective.repairObjective);
	            //$ObjectiveQ[%client.team].add(%objective.repairObjective);

					//finally, unassign the client so he'll go do something else...
			      AIUnassignClient(%client);
					Game.AIChooseGameObjective(%client);
				}

				//finished
				return;
			}
      }
	}
	else if (!%task.checkObstructed)
	{
		%task.checkObstructed = true;

	   //see if anything is in our way
	   InitContainerRadiusSearch(%task.location, 4, $TypeMasks::MoveableObjectType | $TypeMasks::VehicleObjectType |
																					                      $TypeMasks::PlayerObjectType);
	   %objSrch = containerSearchNext();
		if (%objSrch == %client.player)
		   %objSrch = containerSearchNext();
	   if (%objSrch)
			AIMessageThread("ChatMove", %client, -1);
	}
	else if (%task.waitMove < 5 && %task.passes < 5)
	{
		%task.waitMove++;

		//try another pass at deploying 
		if (%task.waitMove == 5)
		{
			%task.waitMove = 0;
			%task.passes++;
			%task.deployAttempts = 0;

			//see if we're *right* underneath the deploy point
			%deployDist2D = VectorDist(getWords(%client.player.position, 0, 1) @ "0", getWords(%task.location, 0, 1) @ "0");
			if (%deployDist2D < 0.25)
			{
				%client.pressjump();
                                %client.pressJet();
				%client.deployPack = true;
	         %client.player.use(Backpack);

	         // check if pack deployed
	         if(%client.player.getMountedImage($BackpackSlot) == 0)
				{
					//we deployed or tried to deploy equipment because objective's GenDefend is true
					//keep decreasing the value. At 0 we won't deploy GenDefend objectives. - Roberto
					if (%task.GenDefend == true)
					{
						$AIGenDisabledCount[%client.team] = $AIGenDisabledCount[%client.team] - 1;

						if ($AIGenDisabledCount[%client.team] < 0)
						{
							$AIGenDisabledCount[%client.team] = 0; 
						}
						echo("-AI TACTICS- Pack deployed. AIGenDisabledCount for team " @ %client.team @ " is now " @ $AIGenDisabledCount[%client.team]);
					}

					//don't add a "repairObject" objective for ceiling turrets
					if (%task == %client.objectiveTask)
					{
						AIUnassignClient(%client);
						Game.AIChooseGameObjective(%client);
					}
				}
			}
		}
	}
	else
	{
		//find a new assignment - and remove this one from the Queue
      if (%task == %client.objectiveTask)
		{
			error(getTaggedString(%client.name) SPC ", from team" SPC %client.team SPC ", is invalidating objective:" SPC %client.objectiveTask.equipment SPC ", UNABLE TO DEPLOY EQUIPMENT");
                        //error("client = " @ getTaggedString(%client.name));
			%client.objective.isInvalid = true;

			//we deployed or tried to deploy equipment because objective's GenDefend is true
			//keep decreasing the value. At 0 we won't deploy GenDefend objectives. - Roberto
			if (%task.GenDefend == true)
			{
				$AIGenDisabledCount[%client.team] = $AIGenDisabledCount[%client.team] - 1;

				if ($AIGenDisabledCount[%client.team] < 0)
				{
					$AIGenDisabledCount[%client.team] = 0; 
				}
				echo("-AI TACTICS- Pack deployed. AIGenDisabledCount for team " @ %client.team @ " is now " @ $AIGenDisabledCount[%client.team]);
			}

			AIUnassignClient(%client);
			Game.AIChooseGameObjective(%client);
		}
	}
}

//----------------------------------------------------------------------------------------------- AIO Destroy Object --- (Last Update 3-30-2004)
//place objective marker in spot you want bot to stand and destroy for sentry turrets and hard to
//reach spots. basically same as attackobject.                                                - Lagg... 3-26-2003

function AIODestroyObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
   {
      return 0;
   }

   //don't attack team 0 stuffs
   if (%this.targetObjectId.team <= 0)
   {
      return 0;      
   }
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }

   //no need to attack if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
   {
      return 0;
   }
   else
   {
      //if this bot is linked to a human who has issued this command, up the weight
		if (%this.issuedByClientId == %client.controlByHuman)
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			{
				if ($AIWeightHumanIssuedCommand < %minWeight)
					return 0;
				else
					%weight = $AIWeightHumanIssuedCommand;
			}
			else
			{
				// calculate the default...
				%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
				if (%weight < $AIWeightHumanIssuedCommand)
					%weight = $AIWeightHumanIssuedCommand;
			}
		}
		else
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
				return 0;

			// calculate the default...
                        %this.desiredEquipment = "Plasma PlasmaAmmo";
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

         //---MISSION SPECIFIC 1---//
         //1.Copy this entire function to your mission extras script.
         //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
         //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
         //---MISSION SPECIFIC 1---//
		}

      //if we are playing DnD, check if objective was completed already, if not give some extra :) - Lagg... 3-29-2004
      if (Game.class $= "DnDGame")
      {
         if (%this.targetObjectId.objectiveCompleted)
            %weight -= 400;
         else
            %weight += 200;
      }

		//increase weight if we have LOS on the object
      %targetObjDist = VectorDist(%client.player.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter());
      if (%targetObjDist < 125)
      {
         //check for LOS
         %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType | $TypeMasks::ForceFieldObjectType | $TypeMasks::VehicleObjectType;
         %hasLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %this.targetObjectId.getWorldBoxCenter(), %mask, 0);

         if (Game.class $= "CTFGame")
         {
         	if (%weight <= 4000)
         	{
         	 	%weight += 250;
         	}

         	 if (%targetObjDist < 75)
         	 {
         	 	%weight += 100;
         	 }

	         if (%client.isCloakerOrHeavy && %weight <= 3890)
	         {
	         	//cloakers and heavies get very high weight
	         	%weight += 400;
	         }
	         else if (%client.player.getMountedImage($BackpackSlot) == 0 && %targetObjDist < 50 && %weight <= 3890)
	         {
	         	//if we're very close to the object and we have no pack, no use in grabbing the flag, just go destroy the object
	         	%weight += 400;
	         }

	         if ( !%this.targetObjectId.isPowered() && %weight <= 4000)
	         {
	         	%weight += 175;
	         }
         }

         if (%hasLOS && %weight <= 4000)
         {
            %weight += 250;
         }
      }
      else if (%targetObjDist < 150 && Game.class $= "CTFGame")
      {
      	%weight += 50;

         //cloakers, heavies, or bots with no packs get bonus weight
         if (%client.isCloakerOrHeavy || %client.player.getMountedImage($BackpackSlot) == 0)
         {
         	%weight += 100;
         }

         if ( !%this.targetObjectId.isPowered() && %weight <= 4000)
         {
         	%weight += 100;
         }
      }

      //add or decrease weight if the objective has inBase or inRoom fields - roberto
      //------------------------------------------------------------------------------
      if (%this.inBase)
      {
         if (%client.inBase)
         {
            if (%weight <= 4000) { %weight += 100; }
         }
      }

      if (%this.inRoom)
      {
         if (%client.inRoom)
         {
            if (%weight <= 4000) { %weight += 200; }
         }
      }
      //------------------------------------------------------------------------------

      return %weight;
   }
}

function AIODestroyObject::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIdestroyObject);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIODestroyObject::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//------------------------------------------------------------------------------------------------ AI Destroy Object ---
function AIDestroyObject::initFromObjective(%task, %objective, %client)
{
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
   %task.equipment = %objective.equipment;
   %task.buyEquipmentSet = %objective.buyEquipmentSet;
   %task.desiredEquipment = %objective.desiredEquipment;
   %task.issuedByClient = %objective.issuedByClientId;
   %task.location = %objective.position;
   %task.mode = %objective.mode;

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;

   //initialize other task vars
   %task.sendMsg = true;
   %task.sendMsgTime = 0;
   %task.useTeleport = false;
}

function AIDestroyObject::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(15);

   //added here so Bot buys a random set if no set was specified in objective dataBlock - Lagg... 2-14-2003
      if (%task.buyEquipmentSet $= "randomSet")
      {
         //pick a random equipment set...
         %randNum = getRandom();

         //if this is an attack objective that really needs the blaster use different sets
         if (getWord(%task.equipment, 0) $= "Blaster")
         {
            if (%randNum < 0.35)
            {
               %task.desiredEquipment = "Blaster Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "HeavySensorJamSetBlaster"; 
            }
            else if (%randNum < 0.7)
            {
               %task.desiredEquipment = "Blaster Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "HeavyShieldOffBlaster"; 
            } 
            else
            {
               %task.desiredEquipment = "Blaster Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "MediumSensorJamSetBlaster"; 
            }
         }
         else
         {
            //light cloakers and medium sensor jamming bots get a higher priority - roberto
            if (%randNum < 0.40)
            {
               %task.desiredEquipment = "Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "LightCloakSet";
            }
            else if (%randNum < 0.70)
            {
               %task.desiredEquipment = "SensorJammerPack Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "MediumEnergySet";
            } 
            else if (%randNum < 0.80)
            {
               %task.desiredEquipment = "Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "HeavyShieldSet";
            }
            else if (%randNum < 0.90)
            {
               %task.desiredEquipment = "Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "HeavyShieldOff";
            }
            else
            {
               %task.desiredEquipment = "Plasma PlasmaAmmo";
               %task.buyEquipmentSet = "MediumMissileSet";
            }
         }
      }

   if (%task.equipment !$= "")
   {
      %equipmentList = %task.equipment;
   }
   else
   {
      %equipmentList = %task.desiredEquipment;
   }
   %client.needEquipment = AINeedEquipment(%equipmentList, %client);


   //even if we don't *need* equipemnt, see if we should buy some... 
   if (! %client.needEquipment && %task.buyEquipmentSet !$= "")
   {
      //see if we could benefit from inventory
      %needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
      %result = AIFindClosestInventory(%client, %needArmor);
      %closestInv = getWord(%result, 0);
      %closestDist = getWord(%result, 1);
      if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
      {
         //find where we are
         %clientPos = %client.player.getWorldBoxCenter();
         %objPos = %task.targetObject.getWorldBoxCenter();
         %distToObject = %client.getPathDistance(%objPos);
			
	 if (%distToObject < 0 || %closestDist < 50)// 50m to inventory in this case
         {
            //error("AIDestroyObject::assume - ! %needequipment but should buy some " @ gettaggedString(%client.name));
	    %client.needEquipment = true;
         }
      }
   }

   //mark the current time for the buy inventory state machine
   %task.buyInvTime = getSimTime();

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0 && isObject(%task.targetObject) )
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
		}
	}
    //--------- End teleporters check ---------
}

function AIDestroyObject::retire(%task, %client)
{
   %client.setTargetObject(-1);
}

function AIDestroyObject::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;

   //let the monitor decide when to stop attacking
   %task.setWeight(%task.baseWeight);
}

function AIDestroyObject::monitor(%task, %client)
{
   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            //don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk); }
else { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeExpress); }

            return;
         }
      }
      else
         %task.useTeleport = false;
   }
	else if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
   //--------------------------------------------------- monitor telporter - end -

   //first, buy the equipment - if we really really need some :) - Lagg... - 4-6-2003
   %hasMortar = (%client.player.getInventory("Mortar") > 0) && (%client.player.getInventory("MortarAmmo") > 0);
   %hasPlasma = (%client.player.getInventory("Plasma") > 0) && (%client.player.getInventory("PlasmaAmmo") > 0);
   %hasDisc = (%client.player.getInventory("Disc") > 0) && (%client.player.getInventory("DiscAmmo") > 0) && (!%client.player.getInventory("Blaster") > 0);
   if (%hasMortar || %hasPlasma || %hasDisc)
   {
      %client.needEquipment = false;
   }

   //bots that need to use blasters in very tight rooms must have a pack, blaster, and a plasma weapon - roberto
   %needToBuy = (%client.player.getMountedImage($BackpackSlot) == 0) || (%client.player.getInventory("Plasma") < 1);

   if ( (getWord(%task.equipment, 0) $= "Blaster") && (%client.player.getInventory("Blaster") < 1 || %needToBuy) )
   {
         %client.needEquipment = true;
   }

   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
			%equipmentList = %task.equipment;
		else
			%equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
		{
		   %task.setMonitorFreq(15);
			%client.needEquipment = false;
		}
		else if (%result $= "Failed")
		{
	      //if this task is the objective task, choose a new objective
	      if (%task == %client.objectiveTask)
	      {
	         AIUnassignClient(%client);
	         Game.AIChooseGameObjective(%client);
	      }
			return;
		}
   }
	//if we made it past the inventory buying, reset the inv time
	%task.buyInvTime = getSimTime();
   
	//chat
	if (%task.sendMsg)
	{
		if (%task.sendMsgTime == 0)
			%task.sendMsgTime = getSimTime();
		else if (getSimTime() - %task.sendMsgTime > 7000)
		{
			%task.sendMsg = false;
		   if (%client.isAIControlled())
			{
				if (%task.chat !$= "")
				{
					%chatMsg = getWord(%task.chat, 0);
					%chatTemplate = getWord(%task.chat, 1);
					if (%chatTemplate !$= "")
						AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
					else
						AIMessageThread(%task.chat, %client, -1);
				}
				else if (%task.targetObject > 0)
				{
					%type = %task.targetObject.getDataBlock().getName();
					if (%type $= "GeneratorLarge")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackGenerator", %client, -1);
					else if (%type $= "SensorLargePulse")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
					else if (%type $= "SensorMediumPulse")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
					else if (%type $= "TurretBaseLarge")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackTurrets", %client, -1);
					else if (%type $= "StationVehicle")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackVehicle", %client, -1);
				}
			}
		}
	}

   //set the target object
   %outOfStuff = (AIAttackOutofAmmo(%client));
   if (isObject(%task.targetObject) && %task.targetObject.getDamageState() !$= "Destroyed" && !%outOfStuff)
   {
      %close = false;
      %distToTargObj = VectorDist(%task.targetObject.position, %client.player.position);

      if (%distToTargObj < 4)
      {
         %close = true;
      }

      //bots that must destroy generators very close to them and must use a blaster get a pass
      if (getWord(%task.equipment, 0) $= "Blaster" && %client.player.getInventory("Blaster") >= 1)
      {
         %close = false;
      }

      %type = %task.targetObject.getDataBlock().getName();
      if (%type $= "StationVehicle" || %type $= "MPBTeleporter")
      {

         %targetLOS = "false";
         %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::ForceFieldObjectType;
         //%targetLOS = !containerRayCast(%client.player.getWorldBoxCenter(), %task.targetObject.getWorldBoxCenter(), %mask, 0);
         %targetLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %task.targetObject.getWorldBoxCenter(), %mask, 0);

         //use the Y-axis of the rotation as the desired direction of approach,
         //and calculate a walk to point 1m in front of the trigger point - Lagg...
 
         %aprchDirection = MatrixMulVector("0 0 0 " @ getWords(%task.targetObject.getTransform(), 3, 6), "0 1 0");
         %aprchDirection = VectorNormalize(%aprchDirection);
         %factor = 10; //10m in front of station vehicle is the right spot to stand                        
         //%aprchFromLocation = VectorAdd(%task.targetObject.position,VectorScale(%aprchDirection, %factor));
         %task.location = VectorAdd(%task.targetObject.position,VectorScale(%aprchDirection, %factor));
   		//don't move like a maniac when near lava - roberto
   		if (%client.inDanger) { %client.stepMove(%task.location, 0.25, $AIModeWalk); }
   		else { %client.stepMove(%task.location, 0.25, $AIModeExpress); }

         if (VectorDist(%task.targetObject.position, %client.player.position) < 15 && %targetLOS)
         {
            %client.setDangerLocation(%task.targetObject.position, 20);
         }
      }
      //in case lazy people don't want to move aiobjective marker
      else if (VectorDist(%task.targetObject.getWorldBoxCenter(), %task.location) < 4)
      {
         if (! %client.targetInRange())
         {
            %client.stepMove(%task.targetObject.getWorldBoxCenter(), 8.0, $AIModeWalk);
         }
      }
      else
      {
   		//don't move like a maniac when near lava - roberto
   		if (%client.inDanger) { %client.stepMove(%task.location, 0.25, $AIModeWalk); }
   		else { %client.stepMove(%task.location, 0.25, $AIModeExpress); }
      }

      %targetLOS = "false";
      %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::ForceFieldObjectType;
      //%targetLOS = !containerRayCast(%client.player.getWorldBoxCenter(), %task.targetObject.getWorldBoxCenter(), %mask, 0);
      %targetLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %task.targetObject.getWorldBoxCenter(), %mask, 0);


      if (%targetLOS && !%close)
      {
         %client.setTargetObject(%task.targetObject, 40, "Destroy");
      }
      else
      {
          %client.setTargetObject(-1);
      }
   }
   else
   {
      %client.setTargetObject(-1);

      //aim at the floor if we need to repair self
      if (%client.player.getDamagePercent() > 0 && %client.player.getInventory(RepairPack) > 0)
      {
         %pos = %client.player.getWorldBoxCenter();
         %posX = getWord(%pos, 0);
         %posY = getWord(%pos, 1);
         %posZ = getWord(%pos, 2) - 1;
         %pos = %posX SPC %posY SPC %posZ;
         %client.aimAt(%pos, 500);
      }

      
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         AIUnassignClient(%client);
         Game.AIChooseGameObjective(%client);
      }
   }
}

//----------------------------------------------------------------------------------------------- AIO Escort Player ---

//modified escorter armor type allowed, rewritten all aivehicle escort stuff, included vehicleobjecttypes
//as potential targets if lazed by escortie.                                              - Lagg... - 4-12-2003

function AIOEscortPlayer::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client) || ! AIClientIsAlive(%this.targetClientId))
      return 0;

   //can't escort yourself
   if (%client == %this.targetClientId)
      return 0;

   if (%client.player.holdingFlag)
      return 0;

   //if vehicle mounted escort check for empty seats - Lagg... - 5-17-2003
   %targMounted = %this.targetClientId.player.ismounted();
   if (%targMounted > 0)
   {
   	  //the escorted driver is too far away - roberto
   	  if ( VectorDist(%this.targetClientId.player.position, %client.player.position) > 500 )
   	  {
   	  	return 0;
   	  }

      //%this.forceClientId = %client;//this should stop em getting bumped
      %vehicle = %this.targetClientId.vehicleMounted;
      %node = findAIEmptySeat(%vehicle, %client.player);

      if (%node < 0 && %client.vehicleMounted != %this.targetClientId.vehicleMounted)
      {
         return 0;
      }
      else if (%client.vehicleMounted == %this.targetClientId.vehicleMounted)  //lets keep bombardiers, gunners in their seats
      {
         return 10000;
      }
   }

   //make sure the class is appropriate unless we are in a vehicle :) - Lagg...
   if (%this.forceClientId <= 0 && %this.issuedByClientId != %client.controlByHuman)
   {
      %targArmor = %this.targetClientId.player.getArmorSize();
      %myArmor = %client.player.getArmorSize();
      %targMounted = %this.targetClientId.player.ismounted();                  

      //changed here to allow bigger escorts - Lagg... 1-31-2003
      if (%targArmor $= "Light" && %myArmor $= "Heavy" && !%targMounted)
	      return 0;
   }

   //can't bump a forced client from level 1
   if (%this.forceClientId > 0 && %this.forceClientId != %client && %level == 1)
   {
      return 0;
   }

   %escortToPlayerDist = vectorDist(%client.Player.getWorldBoxCenter(), %this.targetClientId.player.getWorldBoxCenter());

   //lets check the range on this one if not forced from command map
   if (%this.forceClientId <= 0 && %escortToPlayerDist > 200)
   {
      return 0;
   }

   //if this bot is linked to a human who has issued this command, up the weight
   if (%this.issuedByClientId == %client.controlByHuman)
   {
      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
      {
         if ($AIWeightHumanIssuedEscort < %minWeight)
            return 0;
         else
            %weight = $AIWeightHumanIssuedEscort;
      }
      else
      {
         // calculate the default...
         %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
         if (%weight < $AIWeightHumanIssuedEscort)
            %weight = $AIWeightHumanIssuedEscort;
      }
   }
   else
   {
      //if we are in base the target client must be as well unless forced by a human - Lagg... 9/14/09
      if (!%this.targetClientId.inBase && %client.inBase)
      {
         return 0;
      }

      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
      {
         return 0;
      }

      // calculate the default...
      %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

      //if we're escorts waiting near the pilot's waitspot, increase weight - roberto
      if (%targMounted)
      {
      	if (%this.targetClientId.waitSpot !$= "")
      	{
      		if ( vectorDist(%client.Player.position, %this.targetClientId.waitSpot) < 35 )
      		{
				%weight += 150;
      		}
      	}
      }
   }

   return %weight;
}

function AIOEscortPlayer::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIEscortPlayer);
   %client.objectiveTask.initFromObjective(%this, %client);

   //echo("AIOEscortPlayer::assignClient - " @ getTaggedString(%client.name));

}

function AIOEscortPlayer::unassignClient(%this, %client)
{
   //error("AIOEscortPlayer::unassignClient - " @ getTaggedString(%client.name));

   //make sure we are standing up right// still testing this - Lagg...
   if (AIClientIsAlive(%client))
      %client.player.setActionThread("root");

   //kill the escort objective
   if (%client.escort)
   {
      AIClearObjective(%client.escort);
      %client.escort.delete();
      %client.escort = "";
   }
   %client.repairObject = -1;
   %client.setTargetObject(-1);
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//----------------------------------------------------------------------------------------------- AI Escort Player ---

function AIEscortPlayer::initFromObjective(%task, %objective, %client)
{
   %task.baseWeight = %client.objectiveWeight;
   %task.targetClient = %objective.targetClientId;
   %task.equipment = %objective.equipment;
   %task.buyEquipmentSet = %objective.buyEquipmentSet;
   %task.desiredEquipment = %objective.desiredEquipment;
   %task.mode = %objective.mode;
   %task.issuedByClient = %objective.issuedByClientId;
   %task.forceClient = %objective.forceClientId;
   %task.chat = %objective.chat;

   //initialize other task vars
   %task.sendMsg = true;
   %task.sendMsgTime = 0;
}

function AIEscortPlayer::assume(%task, %client)
{
   %task.setWeightFreq(15);//was 15
   %task.setMonitorFreq(15);//was 15
   %task.rangedTarget = false;
   %task.engageVehicle = -1;

   if (%task == %client.objectiveTask && %client == %task.forceClient &&
     %task.issuedByClient == %task.targetClient && !%task.targetClient.player.isMounted())
      %client.needEquipment = false;
   else
   {
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
         %equipmentList = %task.desiredEquipment;

      %client.needEquipment = AINeedEquipment(%equipmentList, %client);

      if (! %client.needEquipment)
         //%client.stepEscort(%task.targetClient);
         %client.stepMove(%task.targetClient.player.position, 8, $AIModeExpress);
   }

   //mark the current time for the buy inventory state machine
   %task.buyInvTime = getSimTime();
}

function AIEscortPlayer::retire(%task, %client)
{
   //clear the attack object flags
   %task.engageVehicle = -1;
   %client.setTargetObject(-1);

   if (aiClientIsAlive(%client))
   {
      %client.clearStep();
      if(%client.player.isMounted())
         AIDisembarkVehicle(%client);

      //clear the target object
      %client.setTargetObject(-1);

      //tailgunner stop repairing
      if (%client.player.getImageState($BackpackSlot) $= "Activate")
      {
         %client.player.use("RepairPack");
         %client.player.cycleWeapon();
      }
   }
}

function AIEscortPlayer::weight(%task, %client)
{
   //update the task weight
   %task.baseWeight = %client.objectiveWeight;

   //make sure we still have someone to escort
   if (!AiClientIsAlive(%task.targetClient))
   {
      %task.setWeight(0);
      AIUnassignClient(%client);
      Game.AIChooseGameObjective(%client);
      return;
   }

   //always shoot at the closest person to the client being escorted
   %targetPos = %task.targetClient.player.getWorldBoxCenter();
   %losTimeout = $AIClientMinLOSTime + ($AIClientLOSTimeout * %client.getSkillLevel());

   //if mounted in a HAPC or TAILGUNNER position act like a tailgunner first
   if (%client.player.isMounted())
   {
      %task.engageVehicle = -1;
      %vehicle = %client.vehicleMounted;
      %type = %vehicle.getDataBlock().getName();
      if (%type $= "HAPCFlyer" || %type $= "BomberFlyer")
      {        
         //search for a new vehicle to attack
         //%task.engageVehicle = -1;
         %losTimeout = $AIClientMinLOSTime + ($AIClientLOSTimeout * %client.getSkillLevel());
         %result = AIFindClosestEnemyPilot(%client, 400, %losTimeout, true);// - ignores los
         //%result = AIFindClosestEnemyPilot(%client, 400, %losTimeout);//uses los
         %pilot = getWord(%result, 0);

         //if we've got a vehicle to attack
         if (AIClientIsAlive(%pilot))
            %task.engageVehicle = %pilot.vehicleMounted;

         //no pilot then look for players
         if (firstWord(%pilot) <= 0)
            %result = AIFindClosestEnemyToLoc(%client, %targetPos, 300, %losTimeout);
      }
   }
   else
      %result = AIFindClosestEnemyToLoc(%client, %targetPos, 250, %losTimeout);//was 200 - Lagg...

   %task.engageTarget = getWord(%result, 0);

   if (!AIClientIsAlive(%task.engageTarget))
   {
      if (AIClientIsAlive(%task.targetClient.lastDamageClient, %losTimeout) && getSimTime() - %task.targetClient.lastDamageTime < %losTimeout)
         %task.engageTarget = %task.targetClient.lastDamageClient;
   }
   if (!AIClientIsAlive(%task.engageTarget))
   {
      %myPos = %client.player.getWorldBoxCenter();
		%result = AIFindClosestEnemy(%client, 50, %losTimeout);
      %task.engageTarget = getWord(%result, 0);
   }

   //if both us and the person we're escorting are in a vehicle, set the weight high!
   if (%task.targetClient.player.isMounted() && %client.player.isMounted())
   {
      %vehicle = %client.vehicleMounted;

      //if (%vehicle > 0 && isObject(%vehicle) && %vehicle.getDamagePercent() < 0.8) *** TESTING HERE *** To try and stop UE when Tank Blows up
      if (%vehicle > 0 && isObject(%vehicle))
      {
         %task.setWeight($AIWeightVehicleMountedEscort);
         AIProcessVehicle(%client); //call the vehicle process routine - Lagg...
      }
   }
   
   //set the task weight (so important)
   %task.setWeight(%task.baseWeight);

   //find out if our escortee is lazing a target... 
   //%task.missileTarget = -1;//moved down a bit
   %targetCount = ServerTargetSet.getCount();
   for (%i = 0; %i < %targetCount; %i++)
   {
      %targ = ServerTargetSet.getObject(%i);
      if (%targ.sourceObject == %task.targetClient.player)
      {
         //find out which item is being targetted...
	 %targPoint = %targ.getWorldBoxCenter();
         InitContainerRadiusSearch(%targPoint, 10, $TypeMasks::TurretObjectType | $TypeMasks::StaticShapeObjectType |
           $TypeMasks::VehicleObjectType);
	 %task.missileTarget = containerSearchNext();
	 break;
      }

      //moved to here so he says on target once found
      %task.missileTarget = -1;
   }
}

function AIEscortPlayer::monitor(%task, %client)
{
   //make sure we still have someone to escort
   if (!AiClientIsAlive(%task.targetClient))
   {
      AIUnassignClient(%client);
      Game.AIChooseGameObjective(%client);
      return;
   }
   //lets check the range on this one 
   if (vectorDist(%client.Player.getWorldBoxCenter(), %task.targetClient.player.getWorldBoxCenter()) > 300)
   {
      AIUnassignClient(%client);
      Game.AIChooseGameObjective(%client);
      return;
   }

   //for this we send chat first
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
         if (%client.isAIControlled())
	  {
	    if (%task.chat !$= "")
	    {
	       %chatMsg = getWord(%task.chat, 0);
	       %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
	          AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, -1, true);
	    }
	    else
               AIMessageThreadTemplate("ChatTaskCover", "ChatTaskOnIt", %client, -1);
	 }
      }
   }

   //then buy the equipment
   if (%client.needEquipment && !%client.player.isMounted())
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
         %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(15);//was 15
         %client.needEquipment = false;
	      //%client.stepEscort(%task.targetClient);
         //%client.stepMove(%task.targetClient.player.position, 8, $AIModeExpress);
      }
      else if (%result $= "Failed")
      {
         if (%client.player.getArmorSize() $= "Heavy")
         {
            //if this task is the objective task, choose a new objective
	    AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
            return;
         }
         else
         {
            %task.setMonitorFreq(15);
	    %client.needEquipment = false;
         }
      }
   }

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();

   %hasMissile = (%client.player.getInventory("MissileLauncher") > 0) &&
     (%client.player.getInventory("MissileLauncherAmmo") > 0);
     
   //see if our target is mounted in a vehicle...
   if (%task.targetClient.player.isMounted())
   {
      //get the vehicle in question
      %vehicle = %task.targetClient.vehicleMounted;

      //find the passenger seat the bot will take
      if (!%client.isMounted)//added - *
         %node = findAIEmptySeat(%vehicle, %client.player);

      //make sure there is still an empty seat and/or we are in the same vehicle as escortee
      if ((%node < 0 && !%client.vehicleMounted) || (%node < 0 && %client.vehicleMounted != %task.targetClient.vehicleMounted))
      {
         //if this task is the objective task, choose a new objective
	 AIUnassignClient(%client);
	 Game.AIChooseGameObjective(%client);
         return;
      }
                 
      //find the passenger seat location...
      %slotPosition = %vehicle.getSlotTransform(%node);
      
      //make sure we're in the correct armor - assault tanks cannot have a heavy...
      if (%vehicle.getDataBlock().getName() $= "AssaultVehicle")
      {
         //if the bot is in a heavy, go get equipment - Lagg...
	 if (%client.player.getArmorSize() $= "Heavy")
	 {
            if (%task == %client.objectiveTask)
            {
               if (! %task.sendMsg && isObject(%vehicle))
               {
                  %task.sendMsg = true;
                  %task.sendMsgTime = getSimTime();
                  %task.chat = "ChatNeedHold";
               }               
               %task.equipment = "Medium FlareGrenade RepairPack";
	       %task.buyEquipmentSet = "MediumRepairSet";
               %client.needEquipment = true;
               return;
	    }
	 }

         //throw away any packs that won't fit
         if (hasLargePack(%client.player))
            %client.player.throwPack();
      }

      //make sure we're in the correct armor - bomber turrets cannot have a heavy...
      if (%vehicle.getDataBlock().getName() $= "BomberFlyer")
      {
         //bd seat
         %nodeBd = %vehicle.getMountNodeObject(1);

         //tg seat
         %nodeTg = %vehicle.getMountNodeObject(2);

         //if we are a tailgunner with a repair pack lets use it :) - Lagg...
         if (%nodeTg == %client.player)
         {
            //first, check for a vehicle to engage
            if (%task.engageVehicle > 0 && isObject(%task.engageVehicle))
            {
               %client.setEngageTarget(-1);

               //check the range by skill level
               %dis = VectorDist(%client.player.position, %task.engageVehicle.getWorldBoxCenter());
               if (%dis > 200 + (100 * %client.getSkillLevel()))
               {
                  %client.setTargetObject = -1;
                  %task.engageVehicle = -1;
               }

               //destroy the boggy at 6 o'clock
               //%client.aimAt(%task.engageVehicle.getWorldBoxCenter(), 300);//no work - Lagg...
               %client.setTargetObject(%task.engageVehicle, 300, "Destroy");
            }
            else
            {
               //clear the target vehicle...
               %client.setTargetObject(-1);

               //make sure we're still shooting...
               %client.setEngageTarget(%task.engageTarget);

               //see if we're supposed to be engaging any players...
               if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
                  %client.setEngageTarget(%client.shouldEngage);
            
               if (%vehicle.getDamagePercent() > 0 && %client.player.getInventory(RepairPack) > 0 && %client.getEngageTarget() <= 0)
	       {
                  if (isObject(%vehicle) && (%client.player.getImageState($BackpackSlot) !$= "Activate"))               
                     %client.player.use("RepairPack");
                  else if (isObject(%vehicle) && isObject(%client.player.getMountedImage($WeaponSlot))
                    && %client.player.getMountedImage($WeaponSlot).getName() $= "RepairGunImage")
                  {
                     %client.aimAt(%task.targetClient.player.getWorldBoxCenter());
                     %client.repairObject = %vehicle;
                     %client.setTargetObject(%vehicle, 12, "Repair");
                  }
                  else
                  {
                     %client.repairObject = -1;
                     %client.setTargetObject(-1);
                     %client.pressFire(-1);
                  }
               }
               else
               {               
                  %client.repairObject = -1;
                  %client.setTargetObject(-1);
               }
            }
	 } 

         //we may need a tailgunner if we are following a human - Lagg...
         if (! %client.escort && %nodeTg <= 0 && %task.mode !$= "Tailgunner" &&
           !%task.targetClient.isAiControlled())
         {
            
            //create the escort objective (require flares in this case...)
            if (! %task.sendMsg && isObject(%vehicle))
            {
               %task.sendMsg = true;
               %task.sendMsgTime = getSimTime();
               %task.chat = "ChatNeedTailgunner";
            }

            %client.escort = new AIObjective(AIOEscortPlayer)
            {
               dataBlock = "AIObjectiveMarker";
                        weightLevel1 = $AIWeightVehicleMountedEscort;
                        weightLevel2 = 0;
                        description = "Escort " @ getTaggedString(%client.name);
                        targetClientId = %client;
                        issuedByHuman        = true;
                        mode = "Tailgunner";
                        offense = true;
                        chat = "ChatNeedRide";
                        equipment = "FlareGrenade";
	                buyEquipmentSet = "HeavyRepairSet";
            };
            MissionCleanup.add(%client.escort);
            //if is siege game we have to do this right - Lagg... 11-3-2003
            if (Game.class $= "SiegeGame")
            {
               //the objectives in $ObjectiveQ[1] are all offense objectives, $ObjectiveQ[2] has the defensive ones..
               if (%client.team == game.offenseTeam)
                  $ObjectiveQ[1].add(%client.escort);
               else
                  $ObjectiveQ[2].add(%client.escort);
            }
            else
               $ObjectiveQ[%client.team].add(%client.escort);
         }

         //throw away any packs that won't fit
         if (hasLargePack(%client.player))
            %client.player.throwPack();
      }
      //client escort is in a havoc
      else
      {
         //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ ** HERE! ***
         //if we have a sensor jammer pack, use it near enemy turrets - 4/19/2009
         if (%client.player.getInventory(SensorJammerPack) > 0 && %client.player.getEnergyPercent() > 0.4)
         {
            %result = findClosestEnemyTarget(%client, %vehicle);//new function above, also tied to functions in ai.cs file
            %clTarg = getWord(%result,0);
            %clTargDist = getWord(%result, 1);

            if (%clTarg > 0 && %clTargDist < 300)
            {
               //error("AIEscortPlayer::monitor - we have SJPACK and we have turret/sensor in range turn on pack!");

               //we can check for a quick LOS to the turret with function - checkLOSPoint(%pt, %pt1) it returns true or False

               //ok we have an enemy turret or sensor and it is close, if we have LOS turn on Pack

               if (%client.player.getImageState($BackpackSlot) $= "Idle")
                  %client.player.use("Backpack");
            }
         }
             //else if (%client.player.getImageState($BackpackSlot) $= "activate" || %myEnergy < 0.4)
					           //%client.player.use("Backpack");
               

         //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ ** ^^^  THERE ^^^ ***

         //check for a vehicle to engage
         if (%task.engageVehicle > 0 && isObject(%task.engageVehicle))
         {
            %client.setEngageTarget(-1);

            //check the range by skill level
            %dis = VectorDist(%client.player.position, %task.engageVehicle.getWorldBoxCenter());
            if (%dis > 200 + (100 * %client.getSkillLevel()))
            {
               %client.setTargetObject = -1;
               %task.engageVehicle = -1;
            }
            else
            {

               //destroy the boggy at 6 o'clock
               %client.aimAt(%task.engageVehicle.getWorldBoxCenter(), 300);
               %client.setTargetObject(%task.engageVehicle, 300, "Destroy");
            }
         }
         else
         {
            //clear the target vehicle...
            %client.setTargetObject(-1);

            //make sure we're still shooting...
            %client.setEngageTarget(%task.engageTarget);

            //see if we're supposed to be engaging any players...
            if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
               %client.setEngageTarget(%client.shouldEngage);
            
            if (%vehicle.getDamagePercent() > 0 && %client.player.getInventory(RepairPack) > 0 && %client.getEngageTarget() <= 0)
	    {
               if (isObject(%vehicle) && (%client.player.getImageState($BackpackSlot) !$= "Activate"))               
                  %client.player.use("RepairPack");
               else if (isObject(%vehicle) && isObject(%client.player.getMountedImage($WeaponSlot))
                 && %client.player.getMountedImage($WeaponSlot).getName() $= "RepairGunImage")
               {
                  %client.aimAt(%task.targetClient.player.getWorldBoxCenter());
                  %client.repairObject = %vehicle;
                  %client.setTargetObject(%vehicle, 12, "Repair");
               }
               else
               {
                  %client.repairObject = -1;
                  %client.setTargetObject(-1);
                  %client.pressFire(-1);
               }
            }
            else
            {               
               %client.repairObject = -1;
               %client.setTargetObject(-1);
            }
         }
      }
		
      if (%client.player.isMounted())
      {
         %task.setMonitorFreq(15);//testing here

         %vehicle = %task.targetClient.vehicleMounted;

         //make sure it's the same vehicle :)
         if (%client.vehicleMounted != %vehicle)
            AIDisembarkVehicle(%client);
      }
      else if (!%client.player.isMounted())
      {
         //check for empty seat
         if (!%client.isMounted)//added - *
            %node = findAIEmptySeat(%vehicle, %client.player);

         //make sure there is still an empty seat and/or we are in the same vehicle as escortee
         if ((%node < 0 && !%client.vehicleMounted) || (%node < 0 && %client.vehicleMounted != %vehicle))
         {
            //if (%task == %client.objectiveTask)
	    AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
            return;
         }
                 
         //find the passenger seat location...
         %slotPosition = %vehicle.getSlotTransform(%node);

         if (%vehicle.getDataBlock().getName() $= "BomberFlyer")
         {
            if (%nodeBd <= 0 && %client.player.getArmorSize() $= "Heavy" && %task.mode !$= "Tailgunner")
            {
               if (! %task.sendMsg && isObject(%vehicle))
               {
                  %task.sendMsg = true;
                  %task.sendMsgTime = getSimTime();
                  %task.chat = "ChatNeedHold";                     
               }
               %task.equipment = "Medium";
	       %task.buyEquipmentSet = "MediumMissileSet";
               %client.needEquipment = true;
               return;
            }
            if (%nodeBd > 0 && %nodeTg <= 0 && %client.player.getInventory(FlareGrenade) <= 0)
            {
               if (! %task.sendMsg && isObject(%vehicle))
               {
                  %task.sendMsg = true;
                  %task.sendMsgTime = getSimTime();
                  %task.chat = "ChatNeedHold";
               }
               %task.mode = "Tailgunner";
               %task.equipment = "FlareGrenade RepairPack";
	       %task.buyEquipmentSet = "HeavyRepairSet";
               %client.needEquipment = true;
               return;
            }
         }
         //mount the vehicle - heavily modified - Lagg... 10-16-2003
         //%client.stepMove(%slotPosition, 0.25, $AIModeMountVehicle);

         //check for empty seat
         if (!%client.isMounted)//added - *
            %node = findAIEmptySeat(%vehicle, %client.player);

         //if we are in water than we must be smart
         if (%client.inWater)
         {              
            %vPos2D = getWord(%vehicle.getPosition(), 0) SPC getWord(%vehicle.getPosition(), 1) SPC "0";
            %myPos2D = getWord(%client.player.position, 0) SPC getWord(%client.player.position, 1) SPC "0";
            if (vectorDist(%myPos2D, %vPos2D) < 2.5)
            {
               %client.pressJump();
               %client.stepMove(%vehicle.getWorldBoxCenter(), 0.25, $AIModeGainHeight);
               %client.mountVehicle = %vehicle;//force ai to mount used in ai.cs
            }
            else
               %client.stepMove(%vehicle.position, 0.25, $AIModeExpress);
         }
         
         //then check if it is a bomber
         else if (%vehicle.getdataBlock().getName() $= "BomberFlyer")
         {
            %plyrToWaitDist = vectorDist(%client.Player.getWorldBoxCenter(), %task.targetClient.objectiveTask.waitSpot);
            %plyrToVehDist = vectorDist(%client.Player.getWorldBoxCenter(), %vehicle.getWorldBoxCenter());

            if (%node < 2)
            {
               //if (vectorDist(%client.Player.getWorldBoxCenter(), %vehicle.getWorldBoxCenter()) < 20 && %task.targetClient.waitTimeWasReset)
               if (%plyrToWaitDist < 70 && %plyrToVehDist < 38 && %task.targetClient.waitTimeWasReset)
               {
                  //%speed = VectorLen(%client.player.getVelocity());
                  %task.setMonitorFreq(1);//testing here

                     %client.player.setActionThread("sitting", true, true);
                     %client.stepMove(%slotPosition, 0.25, $AIModeMountVehicle);
                     %client.player.setActionThread("sitting", true, true);
               }
               else
               {
                  %task.setMonitorFreq(15);//testing here

                  %client.player.setActionThread("root");

                  //%client.stepMove(%vehicle.getWorldBoxCenter(), 18, $AIModeExpress);
                  if (vectorDist(%client.Player.getWorldBoxCenter(), %task.targetClient.objectiveTask.waitSpot) > 80)
                  {
                     %client.stepMove(%task.targetClient.objectiveTask.waitSpot, 0.25, $AIModeExpress);
                  }
                  else
                  {
                     %client.stepIdle(%task.targetClient.objectiveTask.waitSpot);
                  }
               }
            }
            else
            {
               //if (vectorDist(%client.Player.getWorldBoxCenter(), %vehicle.getWorldBoxCenter()) < 20 && %task.targetClient.waitTimeWasReset)
               if (%plyrToWaitDist < 70 && %plyrToVehDist < 38 && %task.targetClient.waitTimeWasReset)
               {
	               %task.setMonitorFreq(15);//testing here
	               %client.player.setActionThread("root");
	               %client.stepMove(%slotPosition, 0.25, $AIModeMountVehicle);//*** TESTING HERE ***
               }
               else
               {
                  %task.setMonitorFreq(15);//testing here
                  %client.player.setActionThread("root");
                  //%client.stepMove(%vehicle.getWorldBoxCenter(), 18, $AIModeExpress);
                  if (vectorDist(%client.Player.getWorldBoxCenter(), %task.targetClient.objectiveTask.waitSpot) > 80)
                  {
                     %client.stepMove(%task.targetClient.objectiveTask.waitSpot, 0.25, $AIModeExpress);
                  }
                  else
                  {
                     %client.stepIdle(%task.targetClient.objectiveTask.waitSpot);
                  }
               }
            }                     
         }
         //vehicle is a Tank 
         else if (%vehicle.getdataBlock().getName() $= "AssaultVehicle")
         {               
            %plyrToVehDist = vectorDist(%client.Player.getWorldBoxCenter(), %vehicle.getWorldBoxCenter());
            %plyrToWaitDist = vectorDist(%client.Player.getWorldBoxCenter(), %task.targetClient.objectiveTask.waitSpot);

            //if (vectorDist(%client.Player.getWorldBoxCenter(), %vehicle.getWorldBoxCenter()) < 18 && %task.targetClient.waitTimeWasReset)
            if (%plyrToWaitDist < 70 && %plyrToVehDist < 38 && %task.targetClient.waitTimeWasReset)
            {
               //%speed = VectorLen(%client.player.getVelocity());
               %task.setMonitorFreq(1);//testing here

                  %client.player.setActionThread("sitting", true, true);
                  %client.stepMove(%slotPosition, 0.25, $AIModeMountVehicle);//*** TESTING HERE ***
                  %client.player.setActionThread("sitting", true, true);
            }
            else
            {
               %task.setMonitorFreq(15);//testing here
               %client.player.setActionThread("root");
               //%client.stepMove(%vehicle.getWorldBoxCenter(), 16, $AIModeExpress);
               if (vectorDist(%client.Player.getWorldBoxCenter(), %task.targetClient.objectiveTask.waitSpot) > 80)
               {
                  %client.stepMove(%task.targetClient.objectiveTask.waitSpot, 0.25, $AIModeExpress);
               }
               else
               {
                  %client.stepIdle(%task.targetClient.objectiveTask.waitSpot);
               }
            }
         }
         //else must be a havoc so mount regularly (not so fast) move to a good spot and wait
         else
         {
			//just try to mount the vehicle
			//%client.stepMove(getWords(%vehicle.getTransform(), 0, 2), 0.25, $AIModeMountVehicle);//*** TESTING HERE ***
         %plyrToVehDist = vectorDist(%client.Player.getWorldBoxCenter(), %vehicle.getWorldBoxCenter());
         %plyrToWaitDist = vectorDist(%client.Player.getWorldBoxCenter(), %task.targetClient.objectiveTask.waitSpot);

			if (%plyrToWaitDist < 70 && %plyrToVehDist < 38 && %task.targetClient.waitTimeWasReset)
			{
				%client.stepMove(getWords(%vehicle.getTransform(), 0, 2), 0.25, $AIModeMountVehicle);
			}
			else
			{
				if (vectorDist(%client.Player.getWorldBoxCenter(), %task.targetClient.objectiveTask.waitSpot) > 80)
				{
					%client.stepMove(%task.targetClient.objectiveTask.waitSpot, 0.25, $AIModeExpress);
				}
				else
				{
					%client.stepIdle(%task.targetClient.objectiveTask.waitSpot);
				}
			}
         }
      }
   }
   else//------------------------------------------------------------------//target client is not mounted
   {
      //lets end the vehicle conversation stuff - Lagg...
      if (%task == %client.objectiveTask)
      {
         %task.sendMsg = false;
         %task.chat = "";
      }

      //disembark if we're mounted, but our target isn't (anymore)
      if (%client.player.isMounted())
         AIDisembarkVehicle(%client);

      //kill the escort (tailgunner) objective
      if (%client.escort)
      {
         AIClearObjective(%client.escort);
         %client.escort.delete();
         %client.escort = "";
      }

      //don't block the vehicle pad if we are escorting a pilot
      if (%task.targetClient.needVehicle)
      {
         %clVs = AIFindClosestVStation(%client);
         if (%clVs > 0)
         {
            %closestVs = getWord(%clVs, 0);
            %closestVsDist = getWord(%clVs, 1);
            if (%closestVsDist < 7)
               %client.setDangerLocation(%task.targetClient.player.location, 10);
            else if (%closestVsDist < 10)
               %client.stop();
         }
      }

      //END VEHICLE STUFF----------------------------------------------------- Lagg...
   
      //see if we're supposed to be mortaring/missiling something...
      //%client.player.setActionThread("root", true);


      if (!isObject(%task.engageTarget) && isobject(%task.missileTarget) && %task.missileTarget.getDamageState() !$= "Destroyed" && (%hasMortar || %hasMissile))
      {
         if (%task.rangedTarget)
         {
            %client.stop();
	    %client.clearStep();
	    %client.setEngageTarget(-1);

            %type = %task.missileTarget.getDataBlock().getName();
	    if (%type $= "SensorLargePulse" && %hasMissile)
               %client.setTargetObject(%task.missileTarget, 300, "MissileNoLock");
	    else if (%type $= "SensorMediumPulse" && %hasMissile)
               %client.setTargetObject(%task.missileTarget, 300, "MissileNoLock");
	    else if (%type $= "TurretBaseLarge" && %hasMissile)
               %client.setTargetObject(%task.missileTarget, 300, "MissileNoLock");
            else if (%type $= "MobileBaseVehicle" && %hasMissile)
               %client.setTargetObject(%task.missileTarget, 300, "MissileNoLock");
	    else
               %client.setTargetObject(%task.missileTarget, 300, "Mortar");
         }
         else if (%client.getStepName() !$= "AIStepRangeObject")
         {
            if (%hasMortar || %hasMissile)
	       %client.stepRangeObject(%task.missileTarget, "BasicTargeter", 100, 300);
	    else
	       %client.stepRangeObject(%task.missileTarget, "DefaultRepairBeam", 20, 30);
         }
         else if (%client.getStepStatus() $= "Finished")
         %task.rangedTarget = true;
      }

      //move to waiting spot for a ride if we are escorting a pilot
      else if (%task.targetClient.needVehicle || %task.targetClient.vehicleMounted)
      {
         if (vectorDist(%client.player.getWorldBoxcenter(), %task.targetClient.objectiveTask.waitSpot) < 20)
            %client.stepIdle(%task.targetClient.objectiveTask.waitSpot);
         else
            %client.stepMove(%task.targetClient.objectiveTask.waitSpot, 0.25, $AIModeExpress);
      }
      else
      {
        if ((%client.repairObject != %task.targetClient.vehicleMounted) || !%client.player.isMounted())
        {
           %task.rangedTarget = false;
           %client.setTargetObject(-1);
        }
        if (%client.getStepName() !$= "AIStepEscort")
           %client.stepEscort(%task.targetClient);

        //make sure we're still shooting...
        %client.setEngageTarget(%task.engageTarget);

        //see if we're supposed to be engaging anyone...
        if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
           %client.setEngageTarget(%client.shouldEngage);
      }
   }
}

//----------------------------------------------------------------------------------------------- AIO Laze Object --- (last update 3-30-2004)
//fixed here so bots will acually use the targetting lazer for the right team :) - Lagg... - 4-12-2003
//modified last laze time and distance to client we laze for.
   
function AIOLazeObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //if we are in water forget it
   if (%client.inWater)
      return 0;

   //not if we are in the base
   if (%client.inBase)
      return 0;

   //lets check the range on this one
   if (vectorDist(%client.Player.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter()) < 100)
      return 0;
      
	//see if it's already being lazed
	%numTargets = ServerTargetSet.getCount();
	for (%i = 0; %i < %numTargets; %i++)
	{
		%targ = ServerTargetSet.getObject(%i);
		if (%targ.sourceObject != %client.player)
		{
			%targDist = VectorDist(%targ.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter());
			if (%targDist < 100)//was 10
			{
				%this.lastLazedTime = getSimTime();
				%this.lastLazedClient = %targ.sourceObject.client;
				break;
 			}
		}
	}
   if (%this.issuedByClientId > 0 && %this.issuedByClinetId == %client)
   {
       echo("cant laze target for your self IDIOT !");
       return 0;
   }

   //no need to laze if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
	else if (%this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      return 0;      
	else if (getSimTime() - %this.lastLazedTime <= 10000 && %this.lastLazedClient == %client)
		return 0;
   else
	{
	   //set the base weight
	   switch (%level)
	   {
	      case 1:
	         %weight = %this.weightLevel1;
	      case 2:
	         %weight = %this.weightLevel2;
	      case 3:
	         %weight = %this.weightLevel3;
	      default:
	         %weight = %this.weightLevel4;
	   }
   
	   //check Affinity
	   if (ClientHasAffinity(%this, %client))
	      %weight += 100;
      
		//for now, do not deviate from the current assignment to laze a target, if you don't
		//already have a targeting laser.
		%needEquipment = AINeedEquipment(%this.equipment, %client);
		if (!%needEquipment)
			%weight += 100;
		else if (!aiHumanHasControl(%client.controlByHuman, %client))
			return 0;

		//see if this client is close to the issuing client
		if (%this.issuedByClientId > 0)
		{
			if (! AIClientIsAlive(%this.issuedByClientId))
				return 0;

		   %distance = %client.getPathDistance(%this.issuedByClientId.player.getWorldBoxCenter());
			if (%distance < 0)
				%distance = 32767;

		   //see if we're over 300 m
		   if (%distance > 300)
		      return 0;

                   //see if we're within 200 m
		   if (%distance < 200)
		      %weight += 30;

		   //see if we're within 90 m
		   if (%distance < 90)
		      %weight += 30;

		   //see if we're within 45 m
		   if (%distance < 45)
		      %weight += 30;
		}

		//now, if this bot is linked to a human who has issued this command, up the weight
		if (%this.issuedByClientId == %client.controlByHuman && %weight < $AIWeightHumanIssuedCommand)
			%weight = $AIWeightHumanIssuedCommand;

		return %weight;
	}
}

function AIOLazeObject::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AILazeObject);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIOLazeObject::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//----------------------------------------------------------------------------------------------- AI Laze Object ---

function AILazeObject::initFromObjective(%task, %objective, %client)
{
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
   %task.equipment = %objective.equipment;
	%task.buyEquipmentSet = %objective.buyEquipmentSet;
	%task.desiredEquipment = %objective.desiredEquipment;
	%task.issuedByClient = %objective.issuedByClientId;
	%task.msgAck = true;
	%task.msgFire = true;

   //initialize the range variable
   %task.needToRangeTime = getSimTime() + 15000;
}

function AILazeObject::assume(%task, %client)
{
   %task.setWeightFreq(30);
   %task.setMonitorFreq(30);
   %client.needEquipment = AINeedEquipment(%task.equipment, %client);
   
   //clear the target object, and range it
   %client.setTargetObject(-1);
	if (! %client.needEquipment)
	   %client.stepRangeObject(%task.targetObject, "BasicTargeter", 80, 300, %task.issuedByClient.player.getWorldBoxCenter());

	//set up some task vars
	%task.celebrate = false;
	%task.waitTimerMS = 0;

	//mark the current time for the buy inventory state machine
	%task.buyInvTime = getSimTime();
}

function AILazeObject::retire(%task, %client)
{
   %client.setTargetObject(-1);
}

function AILazeObject::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;

   //let the monitor decide when to stop lazing
   %task.setWeight(%task.baseWeight);
}

function AILazeObject::monitor(%task, %client)
{
   //error("AILazeObject::monitor - " @ getTaggedString(%client.name));

   //first, buy the equipment
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
	 %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(30);
	 %client.needEquipment = false;
         %client.stepRangeObject(%task.targetObject, "BasicTargeter", 80, 300);
      }
      else if (%result $= "Failed")
      {
         //if this task is the objective task, choose a new objective
	 if (%task == %client.objectiveTask)
	 {
            AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	 }
	 return;
      }
   }

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();
   
   //set the target object
   if (isObject(%task.targetObject) && %task.targetObject.getDamageState() !$= "Destroyed")
   {
      //make sure we still have equipment
		%client.needEquipment = AINeedEquipment(%task.equipment, %client);
      if (%client.needEquipment)
      {
         //if this task is the objective task, choose a new objective
         if (%task == %client.objectiveTask)
	 {
            AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	    return;
	 }
      }
      
      //look to see if anyone else is also targetting...
      %foundTarget = false;
      %numTargets = ServerTargetSet.getCount();
      for (%i = 0; %i < %numTargets; %i++)
      {
         %targ = ServerTargetSet.getObject(%i);
	 if (%targ.sourceObject != %client.player)
	 {
	    %targDist = VectorDist(%targ.getWorldBoxCenter(), %task.targetObject.getWorldBoxCenter());
	    if (%targDist < 100)//was 10 - Lagg...
	    {
	       %foundTarget = true;
	       break;
	    }
	 }
      }

      if (%foundTarget)
      {
         //if this task is the objective task, choose a new objective
	 if (%task == %client.objectiveTask)
         {
	    AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	    return;
	 }
      }
      else if (%client.getStepStatus() $= "Finished")
      {
         //dissolve the human control link
	 if (%task == %client.objectiveTask)
	    aiReleaseHumanControl(%client.controlByHuman, %client);

         //modified here in case bot gets stuck trying to get LOS to target - Lagg... - 4-21-2003 - 3-30-2004
	 %currentTime = getSimTime();
	 if (%currentTime > %task.needToRangeTime)
	 {
	    //force him to get LOS every 10 seconds...
            %task.needToRangeTime = %currentTime + 15000;
            %client.setDangerLocation(%client.player.getWorldBoxCenter(), 30);
            return;

         }

         %client.setTargetObject(%task.targetObject, 300, "Laze");
	 %task.celebrate = true;
	 %task.waitTimerMS = 0;

	 //make sure we only say "fire..." once
	 if (%task.msgFire)
	 {
	    AIMessageThread("ChatTargetFire", %client, -1);
	    %task.msgFire = false;
	 }
      }
      else
      {
         %client.stepRangeObject(%task.targetObject, "BasicTargeter", 80, 300, %task.issuedByClient.player.getWorldBoxCenter());//added here - Lagg...
         %client.aimAt(%task.targetObject.getWorldBoxCenter(), 1000);
         %client.setTargetObject(-1);
      }
   }
   else
   {
      %client.setTargetObject(-1);

      if (%task.celebrate)
      {
         if (%task.waitTimerMS == 0)
	 {
	    //add in a "woohoo"!  :)
	    //choose the animation range
	    %minCel = 3;
	    %maxCel = 8;

	    //pick a random sound
	    if (getRandom() > 0.25)
					%sound = "gbl.awesome";
				else if (getRandom() > 0.5)
					%sound = "gbl.thanks";
				else if (getRandom() > 0.75)
					%sound = "gbl.nice";
				else
					%sound = "gbl.rock";
			  	%randTime = mFloor(getRandom() * 500) + 1;
			   schedule(%randTime, %client, "AIPlayAnimSound", %client, %task.targetObject.getWorldBoxCenter(), %sound, %minCel, %maxCel, 0);

				//set the timer
            %task.waitTimerMS = getSimTime();
			}

			//else see if the celebration period is over
			else if (getSimTime() - %task.waitTimerMS > 3000)
				%task.celebrate = false;
		}
		else
		{
	      //if this task is the objective task, choose a new objective
	      if (%task == %client.objectiveTask)
			{
	         AIUnassignClient(%client);
				Game.AIChooseGameObjective(%client);
			}
		}
   }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO MAttack Location +++
//modified version of AIAttackLocation, could be called AIMortarSpamLocation :) - Lagg...

function AIOMAttackLocation::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }
      
   //now, if this bot is linked to a human who has issued this command, up the weight
   if (%this.issuedByClientId == %client.controlByHuman)
   {
      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
      {
         if ($AIWeightHumanIssuedCommand < %minWeight)
	    return 0;
	 else
	    %weight = $AIWeightHumanIssuedCommand;
      }
      else
      {
         // calculate the default...
	 %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	 if (%weight < $AIWeightHumanIssuedCommand)
	    %weight = $AIWeightHumanIssuedCommand;
      }
   }
   else
   {
      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
         return 0;

      // calculate the default...
      %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
   }

   //---MISSION SPECIFIC 1---//
   //1.Copy this entire function to your mission extras script.
   //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
   //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
   //---MISSION SPECIFIC 1---//

      //add or decrease weight if the objective has inBase or inRoom fields - roberto
      //------------------------------------------------------------------------------
      if (%this.inBase)
      {
         if (%client.inBase)
         {
            if (%weight <= 4000) { %weight += 100; }
         }
      }

      if (%this.inRoom)
      {
         if (%client.inRoom)
         {
            if (%weight <= 4000) { %weight += 200; }
         }
      }
      //------------------------------------------------------------------------------

   return %weight;
}

function AIOMAttackLocation::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIMAttackLocation);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIOMAttackLocation::unassignClient(%this, %client)
{
   //error("AIOMAttackLocation::unassignClient - " @ getTaggedString(%client.name));

   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//----------------------------------------------------------------------------------------------- AI MAttack Location ---

function AIMAttackLocation::initFromObjective(%task, %objective, %client)
{
        //initialize the task vars from the objective
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.location = %objective.location;
   %task.equipment = %objective.equipment;
	%task.buyEquipmentSet = %objective.buyEquipmentSet;
	%task.desiredEquipment = %objective.desiredEquipment;
	%task.issuedByClient = %objective.issuedByClientId;
	%task.chat = %objective.chat;

   %task.snipeLocation = "";
   %task.hideLocation = "";
   %task.moveToPosition = true;
   %task.moveToSnipe = false;
   %task.nextSnipeTime = 0;

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;

   //MPB teleporter
   %task.useTeleport = false;

	//initialize other task vars
	%task.sendMsg = false;
	%task.sendMsgTime = 0;
   %task.engageTarget = -1;
}

function AIMAttackLocation::assume(%task, %client)
{
   %task.setWeightFreq(30);
   %task.setMonitorFreq(30);
	%client.needEquipment = AINeedEquipment(%task.equipment, %client);
   %client.divertposition = ""; //needed so we try to find markers again when re-taking this objective - roberto

	//even if we don't *need* equipemnt, see if we should buy some... 
	if (! %client.needEquipment && %task.buyEquipmentSet !$= "")
	{
		//see if we could benefit from inventory
		%needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
		%result = AIFindClosestInventory(%client, %needArmor);
		%closestInv = getWord(%result, 0);
		%closestDist = getWord(%result, 1);
		if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
		{
			%result = AIFindClosestEnemyToLoc(%client, %task.location, 100, $AIClientLOSTimeout);
	      %closestEnemy = getWord(%result, 0);

			if (%closestEnemy <= 0 && %closestDist < 100)
				%client.needEquipment = true;
		}
	}

	//mark the current time for the buy inventory state machine
	%task.buyInvTime = getSimTime();

	%task.snipeLocation = "";
	%task.hideLocation = "";
	%task.moveToPosition = true;
	%task.moveToSnipe = false;
	%task.nextSnipeTime = 0;
}

function AIMAttackLocation::retire(%task, %client)
{
}

function AIMAttackLocation::weight(%task, %client)
{
	//update the task weight
	if (%task == %client.objectiveTask)
		%task.baseWeight = %client.objectiveWeight;

	//if we're a sniper, we're going to cheat, and see if there are clients near the attack location
        %losTimeout = $AIClientMinLOSTime + ($AIClientLOSTimeout * %client.getSkillLevel());
	%distToLoc = VectorDist(%client.player.getWorldBoxCenter(), %task.location);
	if (((%client.player.getInventory(SniperRifle) > 0 && %client.player.getInventory(EnergyPack) > 0) ||
            (%client.player.getInventory(Mortar) > 0 && %client.player.getInventory(MortarAmmo) > 0)) && %distToLoc > 100)
		%result = AIFindClosestEnemyToLoc(%client, %task.location, 100, $AIClientLOSTimeout, true);

	//otherwise, do the search normally.  (cheat ignores LOS)...
	else
		%result = AIFindClosestEnemyToLoc(%client, %task.location, 100, %losTimeout, false);

   %closestEnemy = getWord(%result, 0);
   %closestdist = getWord(%result, 1);
   %task.setWeight(%task.baseWeight);
   
   //see if we found someone
   if (%closestEnemy > 0)
      %task.engageTarget = %closestEnemy;
   else
      %task.engageTarget = -1;
}

//---------------------------------------------------------------------
//modified range for bot sniper to attack location - Lagg... 4-12-2003
//more modifications for sparky's tele, diverting locations, and making the bot properly get a target and move from a hiding position to an attack position - Roberto May, 2020

function AIMAttackLocation::monitor(%task, %client)
{
   //first, buy the equipment - if we really really need some :) - Lagg... - 4-6-2003
   %hasMortar = (%client.player.getInventory("Mortar") > 0 && %client.player.getInventory("MortarAmmo") > 0);
   %hasSniper = (%client.player.getInventory("SniperRifle") > 0 && %client.player.getInventory("EnergyPack") > 0);

   if (%hasMortar || %hasSniper)
   {
      %client.needEquipment = false;
   }


   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
    %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(30);
    %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
    //if this task is the objective task, choose a new objective
    if (%task == %client.objectiveTask)
    {
       AIUnassignClient(%client);
       Game.AIChooseGameObjective(%client);
    }
    return;
      }
   }

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();

   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
    if (%client.isAIControlled())
    {
       if (%task.chat !$= "")
       {
          %chatMsg = getWord(%task.chat, 0);
          %chatTemplate = getWord(%task.chat, 1);
          if (%chatTemplate !$= "")
             AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
          else
        AIMessageThread(%task.chat, %client, -1);
       }
       else
          AIMessageThreadTemplate("AttackBase", "ChatSelfAttack", %client, -1);
    }
      }
   }

   //how far are we from the location we're attacking
   %myPos = %client.player.getWorldBoxCenter();
   %distance = %client.getPathDistance(%task.location);
   if (%distance < 0)
   {
      %distance = 32767;
   }

   if (%hasSniper)
   {
      //first, find an LOS location
      if (%task.snipeLocation $= "")
      {
         %task.snipeLocation = %client.getLOSLocation(%task.location, 150, 450);
         %task.hideLocation = %client.getHideLocation(%task.location, VectorDist(%task.location, %task.snipeLocation), %task.snipeLocation, 1); 

         //We are not near distance to hide, see if we must take longer routes to it or not - roberto
         //Divert attacker path - Roberto
         //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
         //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.
         %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));

         if (%DivertPathExists && %distToHide > 325) //must be further than 325m from hide location to go to a divert location
         {
            //Pick a location to move to if we don't have one
            if (%client.divertPosition $= "")
            {
               %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
               %locCount = %DivertLocations.getCount();

               //get the marker that is closest to the hide position
               for (%i = 0; %i < %locCount; %i++)
               {
                  %currMarker = %DivertLocations.getObject(%i);
                  %markerToHideDist = VectorDist(%currMarker.position, %task.hidelocation);

                  //if that marker from DivertLocations is closer to the hide location than we are this is our marker
                  if (%markerToHideDist < %distToHide)
                  {
                     %client.divertPosition = %currMarker.position;
                     break;
                  }
               }

               //If we still haven't found a close marker just go to the hide location
               if (%client.divertPosition $= "")
               {
                  %client.divertPosition = %task.hidelocation;
               }
            }
            else //we have a divert position, is it still closer to our desired location than we are?
            {
               %divertCanContinue = ( VectorDist(%client.divertPosition, %task.hidelocation) < %distToHide );

               //We are way off track, abort divert and move right to desired location
               if (!%divertCanContinue) 
               {
                  %client.divertPosition = %task.hidelocation;
               }
            }

            //By now we should have a marker as destination or the hide location, Mooooooo!
            %client.setTargetObject(-1);

            //don't move like a maniac when near lava - roberto
            if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
            else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
         }
         //NO DIVERTLOCATIONS SIMGROUP or we are close to hide location, just move near hide location
         else
         {
            //don't move like a maniac when near lava - roberto
            if (%client.inDanger || %distToHide < 10) { %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk); }
            else { %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); }

            %client.divertposition = ""; //stop diverting our path
         }

         %task.moveToPosition = true;
      }
      else
      {
         %energy = %client.player.getEnergyPercent();
         %distToSnipe = VectorDist(%task.snipelocation, %client.player.getWorldBoxCenter());
         %distToHide = VectorDist(%task.hidelocation, %client.player.getWorldBoxCenter());

         //until we're in position, we can move using the AIModeExpress, after that, we only want to walk...
         if (%task.moveToPosition)
         {
            if (%distToHide < 4.0)
            {
               //dissolve the human control link
               if (%task == %client.objectiveTask)
               {
                  if (aiHumanHasControl(%task.issuedByClient, %client))
                  {
                     aiReleaseHumanControl(%client.controlByHuman, %client);
                     //should re-evaluate the current objective weight
                     %inventoryStr = AIFindClosestInventories(%client);
                     %client.objectiveWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
                  }
               }

               %task.moveToPosition = false;
            }
            else
            {
               //We are not near distance to hide, see if we must take longer routes to it or not - roberto
               //Divert attacker path - Roberto
               //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
               //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.
               %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));

               if (%DivertPathExists && %distToHide > 325) //must be further than 325m from hide location to go to a divert location
               {
                  //Pick a location to move to if we don't have one
                  if (%client.divertPosition $= "")
                  {
                     %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
                     %locCount = %DivertLocations.getCount();

                     //get the marker that is closest to the hide position
                     for (%i = 0; %i < %locCount; %i++)
                     {
                        %currMarker = %DivertLocations.getObject(%i);
                        %markerToHideDist = VectorDist(%currMarker.position, %task.hidelocation);

                        //if that marker from DivertLocations is closer to the hide location than we are this is our marker
                        if (%markerToHideDist < %distToHide)
                        {
                           %client.divertPosition = %currMarker.position;
                           break;
                        }
                     }

                     //If we still haven't found a close marker just go to the hide location
                     if (%client.divertPosition $= "")
                     {
                        %client.divertPosition = %task.hidelocation;
                     }
                  }
                  else //we have a divert position, is it still closer to our desired location than we are?
                  {
                     %divertCanContinue = ( VectorDist(%client.divertPosition, %task.hidelocation) < %distToHide );

                     //We are way off track, abort divert and move right to desired location
                     if (!%divertCanContinue) 
                     {
                        %client.divertPosition = %task.hidelocation;
                     }
                  }

                  //By now we should have a marker as destination or the hide location, Mooooooo!
                  %client.setTargetObject(-1);

                  //don't move like a maniac when near lava - roberto
                  if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
                  else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
               }
               //NO DIVERTLOCATIONS SIMGROUP or we are close to hide location, just move near hide location
               else
               {
                  //don't move like a maniac when near lava - roberto
                  if (%client.inDanger || %distToHide < 10) { %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk); }
                  else { %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); }

                  %client.divertposition = ""; //stop diverting our path
               }
            }
         }
         else if (%task.moveToSnipe)
         {
            if (%energy > 0.75 && %client.getStepStatus() $= "Finished")
            {
               %client.stepMove(%task.snipeLocation, 4.0, $AIModeWalk);

               if (%task.engageTarget > 0)
               {
                  %client.setEngageTarget(%task.engageTarget);
               }
               else
               {
                  %task.moveToSnipe = false;
               }
            }
            else if (%energy < 0.4)
            {
                %client.setEngageTarget(-1);
                %client.stepMove(%task.hideLocation, 4.0);
                %task.nextSnipeTime = getSimTime() + 1000 + (getRandom() * 4000);
                %task.moveToSnipe = false;
            }
         }
         else if (%energy > 0.5 && getSimTime() > %task.nextSnipeTime)
         { 
            if (%task.engageTarget <= 0)
            {
               //%client.aimAt(%task.location, 8000);
               %targetNearLocation = AIFindClosestEnemyToLoc(%client, %task.snipeLocation, 450, $AIClientLOSTimeout, false); //last argument specifies that it doesn't ignore LOS
               %closestEnemyToLoc = getWord(%targetNearLocation, 0);

               if (AIClientIsAlive(%closestEnemyToLoc))
               {
                  %task.engageTarget = %closestEnemyToLoc;
                  %client.setEngageTarget(%task.engageTarget);
                  %client.stepRangeObject(%task.engageTarget.player.getWorldBoxCenter(), "BasicSniperShot", 150, 250, %task.snipelocation);
                  %client.aimAt(%task.engageTarget.player.getWorldBoxCenter(), 8000);
                  %task.moveToSnipe = true;
               }
            }
            else if (AIClientIsAlive(%task.engageTarget))
            {
               %client.setEngageTarget(%task.engageTarget);
               %client.stepRangeObject(%task.engageTarget.player.getWorldBoxCenter(), "BasicSniperShot", 150, 250, %task.snipelocation);
               %client.aimAt(%task.engageTarget.player.getWorldBoxCenter(), 8000);
               %task.moveToSnipe = true;
            }
         }
         else
         {
            if (%client.inDanger || %distToHide < 15.0) 
            { 
               %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk);
            }
            else 
            { 
               %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); 
            }
         }
      }
   }
   //---------------------------------------------------------------------------------------------------------------------
   //see if we can mortar spam somebody
   else if (%hasMortar)
   {
      //first, find an LOS location
      if (%task.snipeLocation $= "")
      {

         %task.snipeLocation = %client.getLOSLocation(%task.location, 150, 350);
         %task.hideLocation = %client.getHideLocation(%task.location, VectorDist(%task.location, %task.snipeLocation), %task.snipeLocation, 1); 

         //We are not near distance to hide, see if we must take longer routes to it or not - roberto
         //Divert attacker path - Roberto
         //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
         //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.
         %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));

         if (%DivertPathExists && %distToHide > 325) //must be further than 325m from hide location to go to a divert location
         {
            //Pick a location to move to if we don't have one
            if (%client.divertPosition $= "")
            {
               %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
               %locCount = %DivertLocations.getCount();

               //get the marker that is closest to the hide position
               for (%i = 0; %i < %locCount; %i++)
               {
                  %currMarker = %DivertLocations.getObject(%i);
                  %markerToHideDist = VectorDist(%currMarker.position, %task.hidelocation);

                  //if that marker from DivertLocations is closer to the hide location than we are this is our marker
                  if (%markerToHideDist < %distToHide)
                  {
                     %client.divertPosition = %currMarker.position;
                     break;
                  }
               }

               //If we still haven't found a close marker just go to the hide location
               if (%client.divertPosition $= "")
               {
                  %client.divertPosition = %task.hidelocation;
               }
            }
            else //we have a divert position, is it still closer to our desired location than we are?
            {
               %divertCanContinue = ( VectorDist(%client.divertPosition, %task.hidelocation) < %distToHide );

               //We are way off track, abort divert and move right to desired location
               if (!%divertCanContinue) 
               {
                  %client.divertPosition = %task.hidelocation;
               }
            }

            //By now we should have a marker as destination or the hide location, Mooooooo!
            %client.setTargetObject(-1);

            //don't move like a maniac when near lava - roberto
            if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
            else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
         }
         //NO DIVERTLOCATIONS SIMGROUP or we are close to hide location, just move near hide location
         else
         {
            //don't move like a maniac when near lava - roberto
            if (%client.inDanger || %distToHide < 10) { %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk); }
            else { %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); }

            %client.divertposition = ""; //stop diverting our path
         }

         %task.moveToPosition = true;
      }
      else
      {
         %energy = %client.player.getEnergyPercent();
         %distToSnipe = VectorDist(%task.snipelocation, %client.player.getWorldBoxCenter());
         %distToHide = VectorDist(%task.hidelocation, %client.player.getWorldBoxCenter());

         //until we're in position, we can move using the AIModeExpress, after that, we only want to walk...
         if (%task.moveToPosition)
         {
            if (%distToHide < 4.0)
            {
               //dissolve the human control link
               if (%task == %client.objectiveTask)
               {
                  if (aiHumanHasControl(%task.issuedByClient, %client))
                  {
                     aiReleaseHumanControl(%client.controlByHuman, %client);
                     //should re-evaluate the current objective weight
                     %inventoryStr = AIFindClosestInventories(%client);
                     %client.objectiveWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
                  }
               }

               %task.moveToPosition = false;
            }
            else
            {
               //We are not near distance to hide, see if we must take longer routes to it or not - roberto
               //Divert attacker path - Roberto
               //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
               //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.
               %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));

               if (%DivertPathExists && %distToHide > 325) //must be further than 325m from hide location to go to a divert location
               {
                  //Pick a location to move to if we don't have one
                  if (%client.divertPosition $= "")
                  {
                     %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
                     %locCount = %DivertLocations.getCount();

                     //get the marker that is closest to the hide position
                     for (%i = 0; %i < %locCount; %i++)
                     {
                        %currMarker = %DivertLocations.getObject(%i);
                        %markerToHideDist = VectorDist(%currMarker.position, %task.hidelocation);

                        //if that marker from DivertLocations is closer to the hide location than we are this is our marker
                        if (%markerToHideDist < %distToHide)
                        {
                           %client.divertPosition = %currMarker.position;
                           break;
                        }
                     }

                     //If we still haven't found a close marker just go to the hide location
                     if (%client.divertPosition $= "")
                     {
                        %client.divertPosition = %task.hidelocation;
                     }
                  }
                  else //we have a divert position, is it still closer to our desired location than we are?
                  {
                     %divertCanContinue = ( VectorDist(%client.divertPosition, %task.hidelocation) < %distToHide );

                     //We are way off track, abort divert and move right to desired location
                     if (!%divertCanContinue) 
                     {
                        %client.divertPosition = %task.hidelocation;
                     }
                  }

                  //By now we should have a marker as destination or the hide location, Mooooooo!
                  %client.setTargetObject(-1);

                  //don't move like a maniac when near lava - roberto
                  if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
                  else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
               }
               //NO DIVERTLOCATIONS SIMGROUP or we are close to hide location, just move near hide location
               else
               {
                  //don't move like a maniac when near lava - roberto
                  if (%client.inDanger || %distToHide < 10) { %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk); }
                  else { %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); }

                  %client.divertposition = ""; //stop diverting our path
               }
            }
         }
         else if (%task.moveToSnipe)
         {
            if (%energy > 0.75 && %client.getStepStatus() $= "Finished")
            {
               %client.stepMove(%task.snipeLocation, 4.0, $AIModeWalk);

               if (%task.engageTarget > 0)
               {
                  %client.setEngageTarget(%task.engageTarget);
               }
               else
               {
                  %task.moveToSnipe = false;
               }
            }
            else if (%energy < 0.4)
            {
                %client.setEngageTarget(-1);
                %client.stepMove(%task.hideLocation, 4.0);
                %task.nextSnipeTime = getSimTime() + 1000 + (getRandom() * 4000);
                %task.moveToSnipe = false;
            }
         }
         //if we have a repair pack and we're fully healed and the shooting delay passed, or if the shooting delay passed and we don't have a repair pack
         else if ( ((%client.player.getDamagePercent() <= 0 && %client.player.getInventory(RepairPack) > 0) || (%client.player.getInventory(RepairPack) <= 0)) && getSimTime() > %task.nextSnipeTime)
         { 
            if (%task.engageTarget <= 0)
            {
               //%client.aimAt(%task.location, 8000);
               %targetNearLocation = AIFindClosestEnemyToLoc(%client, %task.snipeLocation, 450, $AIClientLOSTimeout, false); //last argument specifies that it doesn't ignore LOS
               %closestEnemyToLoc = getWord(%targetNearLocation, 0);

               if (AIClientIsAlive(%closestEnemyToLoc))
               {
                  %task.engageTarget = %closestEnemyToLoc;
                  %client.setEngageTarget(%task.engageTarget);
                  %client.stepRangeObject(%task.engageTarget.player.getWorldBoxCenter(), "MortarShot", 40, 400, %task.snipelocation);
                  //%client.aimAt(%task.engageTarget.player.getWorldBoxCenter(), 1000);
                  %task.moveToSnipe = true;
               }
            }
            else if (AIClientIsAlive(%task.engageTarget))
            {
               %client.setEngageTarget(%task.engageTarget);
               %client.stepRangeObject(%task.engageTarget.player.getWorldBoxCenter(), "MortarShot", 40, 400, %task.snipelocation);
               //%client.aimAt(%task.engageTarget.player.getWorldBoxCenter(), 1000);
               %task.moveToSnipe = true;
            }
         }
         else
         {
            if (%client.inDanger || %distToHide < 10.0) 
            { 
               %client.stepMove(%task.hideLocation, 4.0, $AIModeWalk);
            }
            else 
            { 
               %client.stepMove(%task.hideLocation, 4.0, $AIModeExpress); 
            }

            //if we're not fighting a player aim at the floor to repair ourselves if we have a repair pack 
            if ( (%client.getEngageTarget() <= 0) && ((%client.player.getDamagePercent() > 0) && (%client.player.getInventory(RepairPack) > 0)) )
            {
               %pos = %client.player.getWorldBoxCenter();
               %posX = getWord(%pos, 0);
               %posY = getWord(%pos, 1);
               %posZ = getWord(%pos, 2) - 1;
               %pos = %posX SPC %posY SPC %posZ;
               %client.aimAt(%pos, 500);
            }
         }
      }
   }
   //----------------------------------------------------------------------------------------------------------------

   //see if we're supposed to be engaging anyone...
   if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
   {
      %client.setEngageTarget(%client.shouldEngage);
   }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO MDestroy Object +++
//place objective marker in spot you want bot to stand and destroy for sentry turrets and hard to
//reach spots. basically same as destroy object but with mortars :)           - Lagg... 3-26-2003

function AIOMDestroyObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
   {
      return 0;      
   }
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }
      
   //no need to attack if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
   {
      return 0;
   }
   else
	{
      //if there is a force field blocking the vpad, because those are dynamically created - roberto
      if ( %this.targetObjectId.getDataBlock().getName() $= "StationVehicle" )
      {
         InitContainerRadiusSearch(%client.player.getWorldBoxCenter(), 12, $TypeMasks::ForceFieldObjectType);
         %objSearch = containerSearchNext();
         if (%objSearch > 0)
         {
            %ffDis = %client.getPathDistance(%objSearch.getWorldBoxCenter());
            %myDis = %client.getPathDistance(%this.position);
            if (%ffDis < %myDis )
            {  
               if (%objSearch.isPowered() && %objSearch.team != %client.team)
               {
                  return 0;
               }
            }
         }
      }

		//if this bot is linked to a human who has issued this command, up the weight
		if (%this.issuedByClientId == %client.controlByHuman)
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			{
				if ($AIWeightHumanIssuedCommand < %minWeight)
					return 0;
				else
					%weight = $AIWeightHumanIssuedCommand;
			}
			else
			{
				// calculate the default...
				%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
				if (%weight < $AIWeightHumanIssuedCommand)
					%weight = $AIWeightHumanIssuedCommand;
			}
		}
		else
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
				return 0;

			// calculate the default...
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

         //---MISSION SPECIFIC 1---//
         //1.Copy this entire function to your mission extras script.
         //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
         //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
         //---MISSION SPECIFIC 1---//
		}

      //if we are playing DnD, check if objective was completed already, if not give some extra :) - Lagg... 3-29-2004
      if (Game.class $= "DnDGame")
      {
         if (%this.targetObjectId.objectiveCompleted)
            %weight -= 400;
         else
            %weight += 200;
      }

		//increase weight if we have LOS on the object
      %targetObjDist = VectorDist(%client.player.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter());
      if (%targetObjDist < 125)
      {
         //check for LOS
         %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType | $TypeMasks::ForceFieldObjectType | $TypeMasks::VehicleObjectType;
         %hasLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %this.targetObjectId.getWorldBoxCenter(), %mask, 0);

         if (Game.class $= "CTFGame")
         {
         	if (%weight < 4000)
         	{
         	 	%weight += 250;
         	}

         	 if (%targetObjDist < 75)
         	 {
         	 	%weight += 100;
         	 }

	         if (%client.isCloakerOrHeavy && %weight <= 3890)
	         {
	         	//cloakers and heavies get very high weight
	         	%weight += 400;
	         }
	         else if (%client.player.getMountedImage($BackpackSlot) == 0 && %targetObjDist < 50 && %weight <= 3890)
	         {
	         	//if we're very close to the object and we have no pack, no use in grabbing the flag, just go destroy the object
	         	%weight += 400;
	         }

	         if ( !%this.targetObjectId.isPowered() && %weight <= 4000)
	         {
	         	%weight += 175;
	         }
         }

         if (%hasLOS && %weight <= 4000)
         {
            %weight += 250;
         }
      }
      else if (%targetObjDist < 150 && Game.class $= "CTFGame")
      {
      	%weight += 50;

         //cloakers, heavies, or bots with no packs get bonus weight
         if (%client.isCloakerOrHeavy || %client.player.getMountedImage($BackpackSlot) == 0)
         {
         	%weight += 100;
         }

         if ( !%this.targetObjectId.isPowered() && %weight <= 4000)
         {
         	%weight += 100;
         }
      }

      //add or decrease weight if the objective has inBase or inRoom fields - roberto
      //------------------------------------------------------------------------------
      if (%this.inBase)
      {
         if (%client.inBase)
         {
            if (%weight <= 4000) { %weight += 100; }
         }
      }

      if (%this.inRoom)
      {
         if (%client.inRoom)
         {
            if (%weight <= 4000) { %weight += 200; }
         }
      }
      //------------------------------------------------------------------------------

      return %weight;
   }
}

function AIOMDestroyObject::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIMDestroyObject);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIOMDestroyObject::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";

   //error("AIOMDestroyObject::unassignClient - " @ getTaggedString(%client.name));
}

//----------------------------------------------------------------------------------------------- AI MDestroy Object ---

function AIMDestroyObject::initFromObjective(%task, %objective, %client)
{
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
   %task.equipment = %objective.equipment;
	%task.buyEquipmentSet = %objective.buyEquipmentSet;
	%task.desiredEquipment = %objective.desiredEquipment;
	%task.issuedByClient = %objective.issuedByClientId;
        %task.location = %objective.location;
   %task.mode = %objective.mode;

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;

	//initialize other task vars
	%task.sendMsg = true;
	%task.sendMsgTime = 0;
   %task.useTeleport = false;

   if (%task.equipment $= "")
   {
      //pick a random equipment set...
      %randNum = getRandom();
      if (%randNum < 0.2)
      {
         %task.desiredEquipment = "Plasma PlasmaAmmo";
         %task.buyEquipmentSet = "HeavyEnergySet";
      }
      else if (%randNum < 0.4)
      {
         %task.desiredEquipment = "Plasma PlasmaAmmo";
         %task.buyEquipmentSet = "HeavyAmmoSet";
      } 
      else if (%randNum < 0.6)
      {
         %task.desiredEquipment = "SensorJammerPack Plasma PlasmaAmmo";
         %task.buyEquipmentSet = "HeavyEnergyDefault";
      }
      else if (%randNum < 0.8)
      {
         %task.desiredEquipment = "Plasma PlasmaAmmo";
         %task.buyEquipmentSet = "HeavyShieldOff"; 
      }
      else
      {
         %task.desiredEquipment = "Plasma PlasmaAmmo";
         %task.buyEquipmentSet = "HeavyRepairSet";
      }
   }
}

function AIMDestroyObject::assume(%task, %client)
{
   %task.setWeightFreq(20);
   %task.setMonitorFreq(20);

   %client.needEquipment = AINeedEquipment(%task.equipment, %client);

	//even if we don't *need* equipemnt, see if we should buy some... 
	if (! %client.needEquipment && %task.buyEquipmentSet !$= "")
	{
		//see if we could benefit from inventory
		%needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
		%result = AIFindClosestInventory(%client, %needArmor);
		%closestInv = getWord(%result, 0);
		%closestDist = getWord(%result, 1);
		if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
		{
			//find where we are
			%clientPos = %client.player.getWorldBoxCenter();
			%objPos = %task.targetObject.getWorldBoxCenter();
			%distToObject = %client.getPathDistance(%objPos);
			
			if (%distToObject < 0 || %closestDist < 100)//%distToObject)
				%client.needEquipment = true;
		}
	}

	//mark the current time for the buy inventory state machine
	%task.buyInvTime = getSimTime();

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0 && isObject(%task.targetObject) )
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
		}
	}
    //--------- End teleporters check ---------
}

function AIMDestroyObject::retire(%task, %client)
{
   %client.setTargetObject(-1);
}

function AIMDestroyObject::weight(%task, %client)
{
	//update the task weight
	if (%task == %client.objectiveTask)
		%task.baseWeight = %client.objectiveWeight;

   //let the monitor decide when to stop attacking
   %task.setWeight(%task.baseWeight);
}

function AIMDestroyObject::monitor(%task, %client)
{
   //first, buy the equipment - if we really really need some :) - Lagg... - 4-6-2003
   %hasMortar = (%client.player.getInventory("Mortar") > 0) && (%client.player.getInventory("MortarAmmo") > 0);

   if (%hasMortar)
      %client.needEquipment = false;

   if (%client.needEquipment)
   {
	   %task.setMonitorFreq(5);
		if (%task.equipment !$= "")
			%equipmentList = %task.equipment;
		else
			%equipmentList = %task.desiredEquipment;

      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(20);
         %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
         //if this task is the objective task, choose a new objective
	 if (%task == %client.objectiveTask)
	 {
            AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	 }
	 return;
      }
   }
	//if we made it past the inventory buying, reset the inv time
	%task.buyInvTime = getSimTime();
   
	//chat
	if (%task.sendMsg)
	{
		if (%task.sendMsgTime == 0)
			%task.sendMsgTime = getSimTime();
		else if (getSimTime() - %task.sendMsgTime > 7000)
		{
			%task.sendMsg = false;
		   if (%client.isAIControlled())
			{
				if (%task.chat !$= "")
				{
					%chatMsg = getWord(%task.chat, 0);
					%chatTemplate = getWord(%task.chat, 1);
					if (%chatTemplate !$= "")
						AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
					else
						AIMessageThread(%task.chat, %client, -1);
				}
				else if (%task.targetObject > 0)
				{
					%type = %task.targetObject.getDataBlock().getName();
					if (%type $= "GeneratorLarge")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackGenerator", %client, -1);
					else if (%type $= "SensorLargePulse")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
					else if (%type $= "SensorMediumPulse")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
					else if (%type $= "TurretBaseLarge")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackTurrets", %client, -1);
					else if (%type $= "StationVehicle")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackVehicle", %client, -1);
				}
			}
		}
	}

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            //don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk); }
else { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeExpress); }

            return;
         }
      }
      else
         %task.useTeleport = false;
   }
	else if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
   //--------------------------------------------------- monitor telporter - end -

   //set the target object
   %outOfStuff = (AIAttackOutofAmmo(%client));
   if (isObject(%task.targetObject) && %task.targetObject.getDamageState() !$= "Destroyed" && !%outOfStuff)
   {
      %dist = VectorDist(%task.targetObject.position, %client.player.position);
      if (%dist < 15)
      {
         if (%task.targetObject.getDataBlock().getName() $= "StationVehicle")
            %client.setDangerLocation(%task.targetObject.position, 20);
         else if (%task.targetObject.getDataBlock().getName() $= "MPBTeleporter")
            %client.setDangerLocation(%task.targetObject.position, 20);
      }
      //in case lazy people don't want to move aiobjective marker
      else if (VectorDist(%task.targetObject.getWorldBoxCenter(), %task.location) < 4)
      {
         if (! %client.targetInRange())
         {
            %client.stepMove(%task.targetObject.getWorldBoxCenter(), 8.0, $AIModeWalk);
         }
      }
      else
      {
         //don't move like a maniac when near lava - roberto
         if (%client.inDanger) { %client.stepMove(%task.location, 0.25, $AIModeWalk); }
         else { %client.stepMove(%task.location, 0.25, $AIModeExpress); }
      }


      %targetLOS = "false";
      %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::ForceFieldObjectType;
      %targetLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %task.targetObject.getWorldBoxCenter(), %mask, 0);

      if (%targetLOS && %dist > 40 && %dist < 300)
      {
         %client.setTargetObject(%task.targetObject, 300, "Mortar");
         //echo("AIMDestroyObject::monitor - Should be set to MDestroy object here");
      }
      else if (%targetLOS && %dist < 40)
      {
         %client.setTargetObject(%task.targetObject, 100, "Destroy");
         //echo("AIMDestroyObject::monitor - Should be set to MDestroy object here");
      }
      else
      {
          %client.setTargetObject(-1);
   		//don't move like a maniac when near lava - roberto
   		if (%client.inDanger) { %client.stepMove(%task.location, 0.25, $AIModeWalk); }
   		else { %client.stepMove(%task.location, 0.25, $AIModeExpress); }
      } 
   }
   else
   {
      %client.setTargetObject(-1);

      //aim at the floor if we need to repair self so we don't repair enemy equipment
      if (%client.player.getDamagePercent() > 0 && %client.player.getInventory(RepairPack) > 0)
      {
         %pos = %client.player.getWorldBoxCenter();
         %posX = getWord(%pos, 0);
         %posY = getWord(%pos, 1);
         %posZ = getWord(%pos, 2) - 1;
         %pos = %posX SPC %posY SPC %posZ;
         %client.aimAt(%pos, 500);
      }

      
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         AIUnassignClient(%client);
         Game.AIChooseGameObjective(%client);
      }
   }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO Missile Object +++
//                                                                            - Lagg... 4-12-2003

function AIOMissileObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      return 0;

   //don't attack team 0 stuffs
   if (%this.targetObjectId.team <= 0)
      return 0;      
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

  //check the zone
  if (%client.inBase)
  {
     return 0;
  }

   //lets check the range on this one
   if (vectorDist(%client.Player.getWorldBoxCenter(), %this.getWorldBoxCenter()) < 50)
      return 0;
    
   //no need to attack if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
   else
	{
		//if this bot is linked to a human who has issued this command, up the weight
		if (%this.issuedByClientId == %client.controlByHuman)
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			{
				if ($AIWeightHumanIssuedCommand < %minWeight)
					return 0;
				else
					%weight = $AIWeightHumanIssuedCommand;
			}
			else
			{
				// calculate the default...
				%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
				if (%weight < $AIWeightHumanIssuedCommand)
					%weight = $AIWeightHumanIssuedCommand;
			}
		}
		else
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
				return 0;

			// calculate the default...
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
		}

      //if we are playing DnD, check if objective was completed already, if not give some extra :) - Lagg... 3-29-2004
      if (Game.class $= "DnDGame")
      {
         if (%this.targetObjectId.objectiveCompleted)
            %weight -= 400;
         else
            %weight += 200;
      }

      return %weight + $emergencyTurretMissile[%client.team]; //return increased missile weight if needed
   }
}

function AIOMissileObject::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIMissileObject);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIOMissileObject::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//----------------------------------------------------------------------------------------------- AI Missile Object ---
//                                                                            - Lagg... 4-12-2003

function AIMissileObject::initFromObjective(%task, %objective, %client)
{
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
   %task.location = %objective.location;
   %task.equipment = %objective.equipment;
	%task.buyEquipmentSet = %objective.buyEquipmentSet;
	%task.desiredEquipment = %objective.desiredEquipment;
	%task.issuedByClient = %objective.issuedByClientId;
   %task.mode = %objective.mode;

	//initialize other task vars
	%task.sendMsg = true;
	%task.sendMsgTime = 0;
        %task.useTeleport = false;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;

   %task.useTeleport = false;
}

function AIMissileObject::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(5);
   %task.needToRangeTime = getSimTime() + 20000;
   %client.needEquipment = AINeedEquipment(%task.equipment, %client);
   %client.divertposition = ""; //needed so we try to find markers again when re-taking this objective - roberto

	//even if we don't *need* equipemnt, see if we should buy some... 
	if (! %client.needEquipment && %task.buyEquipmentSet !$= "")
	{
		//see if we could benefit from inventory
		%needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
		%result = AIFindClosestInventory(%client, %needArmor);
		%closestInv = getWord(%result, 0);
		%closestDist = getWord(%result, 1);
		if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
		{
			//find where we are
			%clientPos = %client.player.getWorldBoxCenter();
			%objPos = %task.targetObject.getWorldBoxCenter();
			%distToObject = %client.getPathDistance(%objPos);
			
			if (%distToObject < 0 || %closestDist < 100)//%distToObject)-modified - Lagg...
				%client.needEquipment = true;
		}
	}

	//mark the current time for the buy inventory state machine
	%task.buyInvTime = getSimTime();

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0 && isObject(%task.targetObject) )
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//error("-Missile object assume- %task.usetele = " @ %task.useTele @ " %task.tele = " @ %task.tele);

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
			//error("-Missile object assume- Sparky's teleporter is closer than MPB.");
		}
	}
    //--------- End teleporters check ---------
}

function AIMissileObject::retire(%task, %client)
{
   %client.setTargetObject(-1);
}

function AIMissileObject::weight(%task, %client)
{

    if (VectorDist(%task.location, %client.Player.getWorldBoxCenter) < 41)
    {
       echo(" Too Close to missile object set baseweight (0)");
       %task.baseWeight = 0;
    }
	else if (%task == %client.objectiveTask)
	{
		%task.baseWeight = %client.objectiveWeight;
	}

   //let the monitor decide when to stop attacking
   %task.setWeight(%task.baseWeight);
}

function AIMissileObject::monitor(%task, %client)
{
   //first, buy the equipment
   if (%client.needEquipment)
   {
	   %task.setMonitorFreq(5);
		if (%task.equipment !$= "")
			%equipmentList = %task.equipment;
		else
			%equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
      {
         ////force a nervous reaction every 15 sec - Lagg...
         //if (getSimTime() - %task.buyInvTime > 15000)
         //{
            //%client.setDangerLocation(%client.player.getWorldBoxCenter(), 20);
           // %task.buyInvTime = getSimTime();
           // %client.buyInvTime = %task.buyInvTime;
         //}
         return;
      }
      else if (%result $= "Finished")
		{
		   %task.setMonitorFreq(15);
			%client.needEquipment = false;
		}
		else if (%result $= "Failed")
		{
	      //if this task is the objective task, choose a new objective
	      if (%task == %client.objectiveTask)
	      {
	         AIUnassignClient(%client);
	         Game.AIChooseGameObjective(%client);
	      }
			return;
		}
   }
	//if we made it past the inventory buying, reset the inv time
	%task.buyInvTime = getSimTime();
   
	//chat
	if (%task.sendMsg)
	{
		if (%task.sendMsgTime == 0)
			%task.sendMsgTime = getSimTime();
		else if (getSimTime() - %task.sendMsgTime > 7000)
		{
			%task.sendMsg = false;
		   if (%client.isAIControlled())
			{
				if (%task.chat !$= "")
				{
					%chatMsg = getWord(%task.chat, 0);
					%chatTemplate = getWord(%task.chat, 1);
					if (%chatTemplate !$= "")
						AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
					else
						AIMessageThread(%task.chat, %client, -1);
				}
				else if (%task.targetObject > 0)
				{
					%type = %task.targetObject.getDataBlock().getName();
					if (%type $= "GeneratorLarge")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackGenerator", %client, -1);
					else if (%type $= "SensorLargePulse")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
					else if (%type $= "SensorMediumPulse")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
					else if (%type $= "TurretBaseLarge")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackTurrets", %client, -1);
					else if (%type $= "StationVehicle")
						AIMessageThreadTemplate("AttackBase", "ChatSelfAttackVehicle", %client, -1);
				}
			}
		}
	}

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            //don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk); }
else { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeExpress); }

            return;
         }
      }
      else
         %task.useTeleport = false;
   }
	else if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
   //--------------------------------------------------- monitor telporter - end -

   //set the target object
   if (isObject(%task.targetObject) && %task.targetObject.getDamageState() !$= "Destroyed")
   {
      //make sure we still have equipment
		%client.needEquipment = AINeedEquipment(%task.equipment, %client);
      if (%client.needEquipment)
      {
         //if this task is the objective task, choose a new objective
         if (%task == %client.objectiveTask)
			{
            AIUnassignClient(%client);
	         Game.AIChooseGameObjective(%client);
				return;
			}
      }

      %clientPos = %client.player.getWorldBoxCenter();
      %targetPos = %task.targetObject.getWorldBoxCenter();
      %distance = %client.getPathDistance(%targetPos);
		if (%distance < 0)
			%distance = 32767;

      //next move to within 300 
      if (%distance > 300)
      {
      //Divert attacker path - Roberto
      //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
      //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.

         %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));
         %distanceVec = 30000;

         if (%DivertPathExists)
         {
         	%distanceVec = VectorDist(%client.player.position, %targetPos);
         }

         if (%distanceVec > 325 && %distanceVec < 30000) //must be further than 325m from targetobject to go to a divert location
         {
            //Pick a location to move to if we don't have one
            if (%client.divertPosition $= "")
            {
            %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
            %locCount = %DivertLocations.getCount();

            //get the marker that is closest to the target object
            for (%i = 0; %i < %locCount; %i++)
            {
               %currMarker = %DivertLocations.getObject(%i);
               %markerToObjDist = VectorDist(%currMarker.position, %targetPos);

               //if that marker from DivertLocations is closer to the target object than we are this is our marker
               if (%markerToObjDist < %distanceVec)
               {
                  %client.divertPosition = %currMarker.position;
                  break;
               }
            }

            //If we still haven't found a close marker just go near the object instead
            if (%client.divertPosition $= "")
            {
               %client.divertPosition = %targetPos;
            }
         }
		 else //we have a divert position, is it still closer to our desired location than we are?
		 {
			%divertCanContinue = ( VectorDist(%client.divertPosition, %targetPos) < %distanceVec );

			//We are way off track, abort divert and move right to desired location
			if (!%divertCanContinue) 
			{
				%client.divertPosition = %targetPos;
			}
		 }

         //By now we should have a marker as destination or the target object itself, Mooooooo!
         %client.setTargetObject(-1);
		//don't move like a maniac when near lava - roberto
		if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
		else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
         }
         //NO DIVERTLOCATIONS SIMGROUP or we are close to target object, just move near the object
         else
         {
            %client.setTargetObject(-1);
            %client.stepMove(%task.targetObject.getWorldBoxCenter(), 15);

            %client.divertposition = ""; //stop diverting our path
         }
      }
      else
      {
         %client.divertposition = ""; //stop diverting our path
         //move to LOS location to objective marker(not to target)
         //(that makes the LOS location adjustable!)
         %firePos = %client.getLOSLocation(%task.location, 50, 290);
         %client.stepMove(%firePos);
         //check for LOS
         %missileLOS = "false";
         %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType | $TypeMasks::ForceFieldObjectType;
         %missileLOS = !containerRayCast(%client.player.getWorldBoxCenter(), %task.location, %mask, 0);
         %inRange = %client.getPathDistance(%task.location);

         //modified here in case bot gets stuck trying to get LOS to target - Lagg... - 4-21-2003
	 %currentTime = getSimTime();
	 if (%currentTime > %task.needToRangeTime)
	 {
	    //force a rangeObject every 20 seconds...
            %task.needToRangeTime = %currentTime + 20000;
            %client.setDangerLocation(%firePos, 30);
            return;

         }
      
         if ((%inRange > 49) && (%inRange < 291) && %missileLOS)
         {
            %client.setTargetObject(%task.targetObject, 300, "Missile");
	    //dissolve the human control link
	    if (%task == %client.objectiveTask)
               {
	          aiReleaseHumanControl(%client.controlByHuman, %client);
                  %client.stop();
               }
            return;
         }
         else if ((%inRange > 49) && (%inRange < 291) && !%missileLOS && (%client.getPathDistance(%client.player.getWorldBoxCenter(), %firePos) < 10))
         {     
            //if this task is the objective task, choose a new objective
            if (%task == %client.objectiveTask)
            {
               %task.baseWeight = 0;
               %task.setWeight(%task.baseWeight);
               return;
            }
         }
         else if (%client.getStepStatus() !$= "Finished")
         {
            %client.stepMove(%firePos);
            return;
         }
      }     
   }
   else
   {     
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         %client.setTargetObject(-1);
         %client.stop();
         AIUnassignClient(%client);
         Game.AIChooseGameObjective(%client);
      }
   }
}

//------------------------------------------------------------------------------------------------- AIO Mortar Object ---
//Modified so bot gets LOS to target before firing. Fixed a lot of stuff here.          - Lagg...

function AIOMortarObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      return 0;
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

  //check the zone
  if (%client.inBase)
  {
     return 0;
  }
      
   //no need to attack if the object is already destroyed
   if (%this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
   else
   {
      //if this bot is linked to a human who has issued this command, up the weight
      if (%this.issuedByClientId == %client.controlByHuman)
      {
         //make sure we have the potential to reach the minWeight
	 if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	 {
	    if ($AIWeightHumanIssuedCommand < %minWeight)
	       return 0;
	    else
	       %weight = $AIWeightHumanIssuedCommand;
	 }
	 else
	 {
	    // calculate the default...
	    %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	    if (%weight < $AIWeightHumanIssuedCommand)
	       %weight = $AIWeightHumanIssuedCommand;
	 }
      }
      else
      {
         //error("AIOMortarObject::weight - %minWeight = " @ %minWeight);//---------------------------------------------- NOTES HERE LAGG ***
         //make sure we have the potential to reach the minWeight
         if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	    return 0;

         //check equipment requirement / if we are too far from inventory forget it
	 %needsEquipment = AINeedEquipment(%this.equipment, %client);
         if (%needsEquipment)
         {
            %needArmor = AIMustUseRegularInvStation(%this.equipment, %client);
            %result = AIFindClosestInventory(%client, %needArmor);
            %closestInv = getWord(%result, 0);
            %closestDist = getWord(%result, 1);

            if (%closestDist > 250 || (%needArmor && %closestDist > 150))
               return 0;
         }  

	 // calculate the default...
	 %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
      }

      //if we are playing DnD, check if objective was completed already, if not give some extra :) - Lagg... 3-29-2004
      if (Game.class $= "DnDGame")
      {
         if (%this.targetObjectId.objectiveCompleted)
            %weight -= 400;
         else
            %weight += 200;
      }

      if (%this.clientLevel1 > 0  && %level == 1 && %this.clientLevel1 == %client)
      {
         if (VectorDist(%client.player.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter()) < 300)
         {

            //check for LOS
            %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType |
              $TypeMasks::ForceFieldObjectType | $TypeMasks::VehicleObjectType;
            %hasLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %this.targetObjectId.getWorldBoxCenter(), %mask, 0);

            if (%hasLOS)
               %weight += 200;
         }
      }

      //if we are to close, reconsider the task
      if (VectorDist(%client.player.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter()) < 60)
         %weight -= 400;

      return %weight;
   }
}

function AIOMortarObject::assignClient(%this, %client)
{
   //error("AIOMortarObject::assignClient - " @ getTaggedString(%client.name));

   %client.objectiveTask = %client.addTask(AIMortarObject);
   %client.objectiveTask.initFromObjective(%this, %client);
   
   ////create the escort objective (require a targeting laser in this case...)
   //%client.escort = new AIObjective(AIOEscortPlayer)
                     //{
						      //dataBlock = "AIObjectiveMarker";
                        //weightLevel1 = $AIWeightEscortOffense[1];
                        //weightLevel2 = $AIWeightEscortOffense[2];
                        //description = "Escort " @ getTaggedString(%client.name);
                        //targetClientId = %client;
                        //offense = true;
                        //equipment = "Plasma PlasmaAmmo TargetingLaser";
								//buyEquipmentSet = "MediumMissileSet";
                     //};
   //MissionCleanup.add(%client.escort);

   ////if is siege game we have to do this right - Lagg... 11-3-2003
   //if (Game.class $= "SiegeGame")
   //{
      ////the objectives in $ObjectiveQ[1] are all offense objectives, $ObjectiveQ[2] has the defensive ones..
      //if (%client.team == game.offenseTeam)
         //$ObjectiveQ[1].add(%client.escort);
      //else
         //$ObjectiveQ[2].add(%client.escort);
   //}
   //else
      //$ObjectiveQ[%client.team].add(%client.escort);
}

function AIOMortarObject::unassignClient(%this, %client)
{
   //error("AIOMortarObject::unassignClient - " @ getTaggedString(%client.name));

   ////kill the escort objective
   //if (%client.escort)
   //{
      //clearObjectiveFromTable(%client.escort);//new function below to stop objective spam - Lagg... 1-27-2004
      //AIClearObjective(%client.escort);
      //%client.escort.delete();
      //%client.escort = "";
   //}   
 
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//------------------------------------------------------------------------

function AIMortarObject::initFromObjective(%task, %objective, %client)
{
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
   %task.equipment = %objective.equipment;
   %task.buyEquipmentSet = %objective.buyEquipmentSet;
   %task.desiredEquipment = %objective.desiredEquipment;
   %task.issuedByClient = %objective.issuedByClientId;
   //%task.mode = %task.targetObject.getDataBlock().getName();
   %task.mode = %objective.mode;

   %task.sendMsgTime = 0;
   %task.sendMsg = true;

   %task.useTeleport = false;
   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;
}

function AIMortarObject::assume(%task, %client) 
{
   %task.setWeightFreq(30);
   %task.setMonitorFreq(30);
   %task.state = moveToRange;
   %task.waitForTargetter = true;
   %task.celebrate = false;
   %task.waitTimerMS = 0;
   %task.targetAcquired = false;
   %task.sayAcquired = true;
   %client.needEquipment = AINeedEquipment(%task.equipment, %client);
   %client.divertposition = ""; //needed so we try to find markers again when re-taking this objective - roberto

   //even if we don't *need* equipemnt, see if we should buy some... 
   if (! %client.needEquipment && %task.buyEquipmentSet !$= "")
   {
      //see if we could benefit from inventory
      %needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
      %result = AIFindClosestInventory(%client, %needArmor);
      %closestInv = getWord(%result, 0);
      %closestDist = getWord(%result, 1);
      if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
      {
         //find where we are
	 %clientPos = %client.player.getWorldBoxCenter();
	 %objPos = %task.targetObject.getWorldBoxCenter();
	 %distToObject = %client.getPathDistance(%objPos);
			
	 if (%distToObject < 0 || %closestDist < 100)//%distToObject)
	    %client.needEquipment = true;
      }
   }

   //mark the current time for the buy inventory state machine
   %task.buyInvTime = getSimTime();

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0 && isObject(%task.targetObject) )
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//error("-Mortar object assume- %task.usetele = " @ %task.useTele @ " %task.tele = " @ %task.tele);

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
			//error("-Mortar object assume- Sparky's teleporter is closer than MPB's.");
		}
	}
    //--------- End teleporters check ---------
}

function AIMortarObject::retire(%task, %client)
{
   %client.setTargetObject(-1);
   
   //remove the associated lazeObjective
   if (%task.targetterObjective)
   {
      clearObjectiveFromTable(%task.targetterObjective);//new function below to stop objective spam - Lagg... 1-27-2004
      AIClearObjective(%task.targetterObjective);
      %task.targetterObjective.delete();
      %task.targetterObjective = "";
   }
}

function AIMortarObject::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;

   //let the monitor decide when to stop mortaring
   %task.setWeight(%task.baseWeight);
}

function AIMortarObject::monitor(%task, %client)
{
   //first, buy the equipment
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
         %equipmentList = %task.desiredEquipment;

      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(30);
	 %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
         //if this task is the objective task, choose a new objective
	 if (%task == %client.objectiveTask)
	 {
	    AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	 }
	 return;
      }
   }

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();
   
   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
	 if (%client.isAIControlled())
	 {
	    if (%task.chat !$= "")
	    {
	       %chatMsg = getWord(%task.chat, 0);
	       %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
	          AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, -1);
	    }
	    else if (%task.targetObject > 0)
	    {
	       %type = %task.targetObject.getDataBlock().getName();
	       if (%type $= "GeneratorLarge")
	          AIMessageThreadTemplate("AttackBase", "ChatSelfAttackGenerator", %client, -1);
	       else if (%type $= "SensorLargePulse")
	          AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
	       else if (%type $= "SensorMediumPulse")
	          AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
	       else if (%type $= "TurretBaseLarge")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackTurrets", %client, -1);
	       else if (%type $= "StationVehicle")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackVehicle", %client, -1);
	    }
         }
      }
   }

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            //don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk); }
else { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeExpress); }

            return;
         }
      }
      else
         %task.useTeleport = false;
   }
	else if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
   //--------------------------------------------------- monitor telporter - end -


   //make sure we still have something to destroy
   if (isObject(%task.targetObject) && %task.targetObject.getDamageState() !$= "Destroyed")
   {
      %clientPos = %client.player.getWorldBoxCenter();
      %targetPos = %task.targetObject.getWorldBoxCenter();
      %distance = %client.getPathDistance(%targetPos);
      if (%distance < 0)
         %distance = 32767;
      
      //make sure we still have equipment
      %client.needEquipment = AINeedEquipment(%task.equipment, %client);
      if (%client.needEquipment)
      {
		//if this task is the objective task, choose a new objective
		if (%task == %client.objectiveTask)
		{
			AIUnassignClient(%client);
			Game.AIChooseGameObjective(%client);

			return;
		}
      }
      //next move to within 300 
      else if (%distance > 300)
      {

      //Divert attacker path - Roberto
      //In some maps (like Titan) we want bots that are spawned at the base to NOT take the shortest route to Missile/Mortar/MortarSpam locations (these have high priority)
      //It's a good idea because we don't bots with these objectives all going through a tunnel, nothing will be accomplished in that way.

         %DivertPathExists = isObject(nameToId("T" @ %client.team @ "DivertLocations"));
         %distanceVec = 30000;

         if (%DivertPathExists)
         {
         	%distanceVec = VectorDist(%client.player.position, %targetPos);
         }

         if (%distanceVec > 325 && %distanceVec < 30000) //must be further than 325m from targetobject to go to a divert location
         {
            //Pick a location to move to if we don't have one
            if (%client.divertPosition $= "")
            {
            %DivertLocations = nameToId("T" @ %client.team @ "DivertLocations");
            %locCount = %DivertLocations.getCount();

            //get the marker that is closest to the target object
            for (%i = 0; %i < %locCount; %i++)
            {
               %currMarker = %DivertLocations.getObject(%i);
               %markerToObjDist = VectorDist(%currMarker.position, %targetPos);

               //if that marker from DivertLocations is closer to the target object than we are this is our marker
               if (%markerToObjDist < %distanceVec)
               {
                  %client.divertPosition = %currMarker.position;
                  break;
               }
            }

            //If we still haven't found a close marker just go near the object instead
            if (%client.divertPosition $= "")
            {
               %client.divertPosition = %targetPos;
            }
         }
		 else //we have a divert position, is it still closer to our desired location than we are?
		 {
			%divertCanContinue = ( VectorDist(%client.divertPosition, %targetPos) < %distanceVec );

			//We are way off track, abort divert and move right to desired location
			if (!%divertCanContinue) 
			{
				%client.divertPosition = %targetPos;
			}
		 }

         //By now we should have a marker as destination or the target object itself, Mooooooo!
         %client.setTargetObject(-1);
		//don't move like a maniac when near lava - roberto
		if (%client.inDanger) { %client.stepMove(%client.divertPosition, 0.25, $AIModeWalk); }
		else { %client.stepMove(%client.divertPosition, 0.25, $AIModeExpress); }
         }
         //NO DIVERTLOCATIONS SIMGROUP or we are close to target object, just move near the object
         else
         {
            %client.setTargetObject(-1);
            %client.stepMove(%task.targetObject.getWorldBoxCenter(), 15);

            %client.divertposition = ""; //stop diverting our path
         }
      }
      
      //now start ask for someone to laze the target, and start a 20 sec timer
      else if (%task.waitForTargetter)
      {
         //see if we've started the timer
         if (%task.waitTimerMS == 0)
         {
	    //range the object
	    %client.stepRangeObject(%task.targetObject, "BasicTargeter", 150, 300);

            //now ask for a targeter...
	    %targetType = %task.targetObject.getDataBlock().getName();
	    if (%targetType $= "TurretBaseLarge")
	       AIMessageThread("ChatCmdTargetTurret", %client, -1);
	    else if (%targetType $= "SensorLargePulse")
	       AIMessageThread("ChatCmdTargetSensors", %client, -1);
	    else if (%targetType $= "SensorMediumPulse")
	       AIMessageThread("ChatCmdTargetSensors", %client, -1);
	    else
	       AIMessageThread("ChatTargetNeed", %client, -1);

            %task.waitTimerMS = getSimTime();

            //create the targetter objective
	    if (! %task.targetterObjective)
	    {
	       %task.targetterObjective = new AIObjective(AIOLazeObject)
	               {
	                       dataBlock = "AIObjectiveMarker";
	                       weightLevel1 = $AIWeightLazeObject[1];
	                       weightLevel2 = $AIWeightLazeObject[2];
	                       description = "Laze the " @ %task.targetObject.getName();
	                       targetObjectId = %task.targetObject;
			       issuedByClientId = %client;
	                       offense = true;
	                       equipment = "TargetingLaser";
	               };
	       MissionCleanup.add(%task.targetterObjective);

	       //if this is siege game we have to do stuff this right - Lagg... 11-3-2003
               if (Game.class $= "SiegeGame")
               {
                  //the objectives in $ObjectiveQ[1] are all offense objectives, $ObjectiveQ[2] defense
                  if (%client.team == game.offenseTeam)
                     $ObjectiveQ[1].add(%task.targetterObjective);
                  else
                     $ObjectiveQ[2].add(%task.targetterObjective);
               }
               else
                  $ObjectiveQ[%client.team].add(%task.targetterObjective);
            }

	    %task.targetterObjective.lastLazedTime = 0;

	    ////remove the escort (want a targetter instead)
	    //if (%client.escort)
	    //{
               //clearObjectiveFromTable(%client.escort);//new function below to stop objective spam - Lagg... 1-27-2004
	       //AIClearObjective(%client.escort);
	       //%client.escort.delete();
	       //%client.escort = "";
	    //}
         }									
         else
         {
            %elapsedTime = getSimTime() - %task.waitTimerMS;
	    if (%task.targetterObjective.group > 0)
	       %targetter = %task.targetterObjective.group.clientLevel[1];
	    else
	       %targetter = %task.targetterObjective.clientLevel[1];

	    //see if we can find a target near our objective
	    %task.targetAcquired = false;
	    %numTargets = ServerTargetSet.getCount();
	    for (%i = 0; %i < %numTargets; %i++)
	    {
	       %targ = ServerTargetSet.getObject(%i);
	       %targDist = VectorDist(%targ.getWorldBoxCenter(), %task.targetObject.getWorldBoxCenter());
	       if (%targDist < 300)
	       {
	          %task.targetAcquired = true;
		  break;
	       }
	    }

	    if (%task.targetAcquired)
            {
               %task.waitForTargetter = false;
	       %task.waitTimerMS = 0;
	       %task.celebrate = true;
	       %task.sayAcquired = false;
	       AIMessageThread("ChatTargetAcquired", %client, -1);
            }

	    //else see if we've run out of time
	    else if ((! %targetter || ! %targetter.isAIControlled()) && %elapsedTime > 10000)
	    {
               %task.waitForTargetter = false;
	       %task.waitTimerMS = 0;
	       %task.celebrate = true;
	    }  
         }
      }
      
      //now we should finally be attacking with or without a targetter
      //eventually, the target will be destroyed, or we'll run out of ammo...
      else
      {
         //dissolve the human control link
	 if (%task == %client.objectiveTask)
	    aiReleaseHumanControl(%client.controlByHuman, %client);

	 //see if we didn't acquired a spotter along the way
	 if (%task.targetterObjective.group > 0)
	    %targetter = %task.targetterObjective.group.clientLevel[1];
	 else
	    %targetter = %task.targetterObjective.clientLevel[1];

	 if (! %task.targetAcquired && AIClientIsAlive(%targetter) && %targetter.isAIControlled())
	 {
	    %client.setTargetObject(-1);
            %task.waitForTargetter = true;
	 }
	 else
	 {
	    //see if we can find a target near our objective
	    if (! %task.targetAcquired)
	    {
	       %numTargets = ServerTargetSet.getCount();
	       for (%i = 0; %i < %numTargets; %i++)
	       {
	          %targ = ServerTargetSet.getObject(%i);
		  %targDist = VectorDist(%targ.getWorldBoxCenter(), %task.targetObject.getWorldBoxCenter());
		  if (%targDist < 300)
		  {
		     %task.targetAcquired = true;
		     break;
		  }
	       }
	       //see if we found a target (must be by a human)
	       if (%task.targetAcquired && %task.sayAcquired)
	       {
	          %task.sayAcquired = false;
		  AIMessageThread("ChatTargetAcquired", %client, -1);
	       }
	    }
				
	    //set the target object, and keep attacking it
	    if (%client.getStepStatus() $= "Finished")
	       %client.setTargetObject(%task.targetObject, 300, "Mortar");
	    else
	       %client.setTargetObject(-1);
         }
      }
   }

   //the target must have been destroyed  :)
   else
   {
      //dissolve the human control link
      if (%task == %client.objectiveTask)
         aiReleaseHumanControl(%client.controlByHuman, %client);

      %client.setTargetObject(-1);
      %client.clearStep();
      %client.stop();

      if (%task.celebrate)
      {
         if (%task.waitTimerMS == 0)
	 {
	    //client animation "woohoo"!  :)
	    //choose the animation range
	    %minCel = 3;
	    %maxCel = 8;

	    //pick a random sound
	    if (getRandom() > 0.25)
	       %sound = "gbl.awesome";
	    else if (getRandom() > 0.5)
	       %sound = "gbl.thanks";
	    else if (getRandom() > 0.75)
	       %sound = "gbl.nice";
	    else
	       %sound = "gbl.rock";
	    %randTime = mFloor(getRandom() * 500) + 1;
	    schedule(%randTime, %client, "AIPlayAnimSound", %client, %task.targetObject.getWorldBoxCenter(), %sound, %minCel, %maxCel, 0);

	    //team message
	    AIMessageThread("ChatEnemyTurretsDestroyed", %client, -1);

	    //set the timer
            %task.waitTimerMS = getSimTime();
	 }

	 //else see if the celebration period is over
	 else if (getSimTime() - %task.waitTimerMS > 3000)
	    %task.celebrate = false;
      }
      else
      {
         //if this task is the objective task, choose a new objective
	 if (%task == %client.objectiveTask)
	 {
	    AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	 }
      }
   }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO Place Camera +++
// have to fix this so a repair object objective is created for the newly deployed thingy??
//rotate the objective marker, bot will place on the "Y" same as all other deploy objectives
//                                                                            - Lagg... 1-14-2003

function AIOPlaceCamera::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }

	//make sure the deploy objective is valid
	//if (%this.isInvalid)
		//return 0;

	//first, make sure we haven't deployed too many...
 	if (%this.equipment $= "CameraGrenade")
        %maxAllowed = $TeamDeployableMax[DeployedCamera];
        else
                return 0;
   
	if ($TeamDeployedCount[%client.team, %this.equipment] >= %maxAllowed)
		return 0;

	//now make sure there are no other items in the way...
  	  InitContainerRadiusSearch(%this.location, 1, $TypeMasks::VehicleObjectType |
  	                                               $TypeMasks::MoveableObjectType |
  	                                               $TypeMasks::StaticShapeObjectType |
  	                                               $TypeMasks::TSStaticShapeObjectType |
  	                                               $TypeMasks::ForceFieldObjectType |
  	                                               $TypeMasks::ItemObjectType |
  	                                               $TypeMasks::PlayerObjectType | $TypeMasks::TurretObjectType);                   
	%objSearch = containerSearchNext();

	//make sure we're not invalidating the deploy location with the client's own player object
	if (%objSearch == %client.player)
		%objSearch = containerSearchNext();

	//did we find an object which would block deploying the equipment?
	if (isObject(%objSearch))
	{
		return 0;
	}

   if (Game.class $= "CTFGame")
   {
      if (%this.GenDefend == true)
      {
         //for GenDefend mode, only deploy equipment if our gen was destroyed last time and we still didn't deploy everything we needed - roberto
         if ($AIGenDisabledCount[%client.team] <= 0)
         {
         	return 0; //stop if we already deployed everything for gen defense
         }
      }
      else if (!$AITeamLosing[%client.team])//this is not a GenDefend objective so we can deploy, BUT only deploy if we are losing, will make it more challenging for the human team
      {
         return 0; //the AI is not losing, don't deploy non-GenDefend cameras
      }
   }

	//check equipment requirement
	%needEquipment = AINeedEquipment(%this.equipment, %client);
	
        //if don't need equipment, see if we've past the "point of no return", and should continue regardless
 	if (! %needEquipment)
 	{
 		%needArmor = AIMustUseRegularInvStation(%this.equipment, %client);
 		%result = AIFindClosestInventory(%client, %needArmor);
 		%closestInv = getWord(%result, 0);
 		%closestDist = getWord(%result, 1);
 
 		//if we're too far from the inv to go back, or we're too close to the deploy location, force continue
 		if (%closestDist > 50 && VectorDist(%client.player.getWorldBoxCenter(), %task.location) < 150)
 		{
 			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
 			if (%weight < $AIWeightContinueDeploying)
 				%weight = $AIWeightContinueDeploying;
 			return %weight;
 		}
 	}

	//if this bot is linked to a human who has issued this command, up the weight
	if (%this.issuedByClientId == %client.controlByHuman)
	{
		//make sure we have the potential to reach the minWeight
		if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
		{
			if ($AIWeightHumanIssuedCommand < %minWeight)
				return 0;
			else
				%weight = $AIWeightHumanIssuedCommand;
		}
		else
		{
			// calculate the default...
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
			if (%weight < $AIWeightHumanIssuedCommand)
				%weight = $AIWeightHumanIssuedCommand;
		}
	}
	else
	{
		//make sure we have the potential to reach the minWeight
		if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			return 0;

		// calculate the default...
		%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	}

   //add or decrease weight if the objective has inBase or inRoom fields - roberto
   //------------------------------------------------------------------------------
   if (%this.inBase)
   {
      if (%client.inBase)
      {
         if (%weight <= 4000) { %weight += 100; }
      }
      else
      {
         if (Game.class $= "SiegeGame")
         {
            %weight -= 100;
         }
      }
   }

   if (%this.inRoom)
   {
      if (%client.inRoom)
      {
         if (%weight <= 4000) { %weight += 200; }
      }
      else
      {
         %weight -= 100;
      }
   }
   //------------------------------------------------------------------------------

	return %weight;
}

function AIOPlaceCamera::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIPlaceCamera);
   %task = %client.objectiveTask;
   %task.initFromObjective(%this, %client);
}

function AIOPlaceCamera::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//----------------------------------------------------------------------------------------------- AI Place Camera ---

function AIPlaceCamera::initFromObjective(%task, %objective, %client)
{
	//initialize the task vars from the objective
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.location = %objective.location;
   %task.equipment = %objective.equipment;
	%task.buyEquipmentSet = %objective.buyEquipmentSet;
	%task.desiredEquipment = %objective.desiredEquipment;
	%task.issuedByClient = %objective.issuedByClientId;
	%task.chat = %objective.chat;
    %task.GenDefend = %objective.GenDefend; //should we deploy this after our gen was destroyed last time? - roberto

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;

	//initialize other task vars
	%task.sendMsg = true;
	%task.sendMsgTime = 0;

	//use the Y-axis of the rotation as the desired direction of deployement,
	//and calculate a walk to point 3 m behind the deploy point. 
	%task.deployDirection = MatrixMulVector("0 0 0 " @ getWords(%objective.getTransform(), 3, 6), "0 1 0");
	%task.deployDirection = VectorNormalize(%task.deployDirection);
}

function AIPlaceCamera::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(15);
	
	%client.needEquipment = AINeedEquipment(%task.equipment, %client);
   
   //mark the current time for the buy inventory state machine
	%task.buyInvTime = getSimTime();

	%task.passes = 0;
	%task.deployAttempts = 0;
	%task.checkObstructed = false;
	%task.waitMove = 0;

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0)
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
		}
	}
    //--------- End teleporters check ---------
}

function AIPlaceCamera::retire(%task, %client)
{
}

function AIPlaceCamera::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;

   %task.setWeight(%task.baseWeight);
}

function AIPlaceCamera::monitor(%task, %client)
{
   //first, buy the equipment
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
         %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
      {
         ////force a nervous reaction every 15 sec - Lagg...
         //if (getSimTime() - %task.buyInvTime > 15000)
         //{
           // %client.setDangerLocation(%client.player.getWorldBoxCenter(), 20);
            //%task.buyInvTime = getSimTime();
            //%client.buyInvTime = %task.buyInvTime;
         //}
         return;
      }
      else if (%result $= "Finished")
      {	
        %task.setMonitorFreq(30);
        %client.needEquipment = false;
        //if we made it past the inventory buying, reset the inv time
	   %task.buyInvTime = getSimTime();
      }
      else if (%result $= "Failed")
      {
         //if this task is the objective task, choose a new objective
	 if (%task == %client.objectiveTask)
	 {
	    AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	 }
         return;
      }
   }

   //make sure we still have equipment
   %client.needEquipment = AINeedEquipment(%task.equipment, %client);
   if (%client.needEquipment)
   {
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         AIUnassignClient(%client);
	 Game.AIChooseGameObjective(%client);
	 return;
      }
   }

   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
         if (%client.isAIControlled())
         {
            if (%task.chat !$= "")
	    {
               %chatMsg = getWord(%task.chat, 0);
               %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
	          AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, -1);
            }
         }
      }
   }

	if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}


   //see if we're supposed to be engaging anyone...
   if (AIClientIsAlive(%client.shouldEngage))
   {
      %hasLOS = %client.hasLOSToClient(%client.shouldEngage);
      %losTime = %client.getClientLOSTime(%client.shouldEngage);
      if (%hasLOS || %losTime < 1000)
         %client.setEngageTarget(%client.shouldEngage);
      else
         %client.setEngageTarget(-1);
   }
   else
   {
      %client.setEngageTarget(-1);
   }


   //calculate the deployFromLocation
   %factor = -4;// * (3 - (%task.passes * 0.5));
   %task.deployFromLocation = VectorAdd(%task.location,VectorScale(%task.deployDirection, %factor));

   //see if we're within range of the deploy location
   %clLoc = %client.player.position;
   %distance = VectorDist(%clLoc, %task.deployFromLocation);
   %dist2D = VectorDist(%client.player.position, getWords(%task.deployFromLocation, 0, 1) SPC getWord(%client.player.position, 2));

   //set the aim when we get near the target...  this will be overwritten when we're actually trying to deploy
   if (%distance < 10 && %dist2D < 10)
      %client.aimAt(%task.location, 1000);

   if ((%client.pathDistRemaining(20) > %distance + 0.25) || %dist2D > 0.5)
   {
      %task.deployAttempts = 0;
      %task.checkObstructed = false;
      %task.waitMove = 0;
	//don't move like a maniac when near lava - roberto
	if (%client.inDanger) { %client.stepMove(%task.deployFromLocation, 0.25, $AIModeWalk); }
	else { %client.stepMove(%task.deployFromLocation, 0.25, $AIModeExpress); }
      %task.setMonitorFreq(15);
      return;
   }
   
   if (%task.deployAttempts < 1 && %task.passes < 1 && !AIClientIsAlive(%client.getEngageTarget()))
   {
		//dissolve the human control link
		if (%task == %client.objectiveTask)
			aiReleaseHumanControl(%client.controlByHuman, %client);

	   %task.setMonitorFreq(3);
      %client.stop();
		if (%task.deployAttempts == 0)
			%deployPoint = %task.location;
		else
	      %deployPoint = findTurretDeployPoint(%client, %task.location, %task.deployAttempts);
      if(%deployPoint !$= "")
      {
         // we have possible point
         %task.deployAttempts++;
         %client.aimAt(%deployPoint, 2000);

			//try to place the camera
			//%client.deployPack = true;
         %client.lastThrownObject = -1;
         %client.player.throwStrength = 2;
         %client.player.use(CameraGrenade);

		//we deployed or tried to deploy equipment because objective's GenDefend is true
		//keep decreasing the value. At 0 we won't deploy GenDefend objectives. - Roberto
		if (%task.GenDefend == true)
		{
			$AIGenDisabledCount[%client.team] = $AIGenDisabledCount[%client.team] - 1;

			if ($AIGenDisabledCount[%client.team] < 0)
			{
				$AIGenDisabledCount[%client.team] = 0; 
			}
			echo("-AI TACTICS- Camera deployed. AIGenDisabledCount for team " @ %client.team @ " is now " @ $AIGenDisabledCount[%client.team]);
		}
         
         // check if camera deployed
         if (isObject(%client.lastDeployedObject))
			{
				//see if there's a "repairObject" objective for the newly deployed thingy...
				if (%task == %client.objectiveTask)
				{
					%deployedObject = %client.lastDeployedObject;

					//search the current objective group and search for a "repair Object" task...
					%objective = %client.objective;

					//delete any previously associated "AIORepairObject" objective
					if (isObject(%objective.repairObjective))
					{
						AIClearObjective(%objective.repairObjective);
						%objective.repairObjective.delete();
						%objective.repairObjective = "";
					}

					//add the repair objective
	            %objective.repairObjective = new AIObjective(AIORepairObject)
		                              {
											      dataBlock = "AIObjectiveMarker";
		                                 weightLevel1 = %objective.weightLevel1 - 100;
		                                 weightLevel2 = 0;
		                                 description = "Repair the Deployed Camera";
													targetObjectId = %deployedObject;
													issuedByClientId = %client;
		                                 offense = false;
													defense = true;
		                                 equipment = "RepairPack";
                                                 buyEquipmentSet = "LightRepairSet";
		                              };
					%objective.repairObjective.deployed = true;
					%objective.repairObjective.setTransform(%objective.getTransform());
					%objective.repairObjective.group = %objective.group;
			      MissionCleanup.add(%objective.repairObjective);
	            $ObjectiveQ[%client.team].add(%objective.repairObjective);

					//finally, unassign the client so he'll go do something else...
			      AIUnassignClient(%client);
					Game.AIChooseGameObjective(%client);
				}

				//finished
				return;
			}
      }
	}
	else if (!%task.checkObstructed)
	{
		%task.checkObstructed = true;

	   //see if anything is in our way
	   InitContainerRadiusSearch(%task.location, 4, $TypeMasks::MoveableObjectType | $TypeMasks::VehicleObjectType |
             $TypeMasks::PlayerObjectType);
	   %objSrch = containerSearchNext();
		if (%objSrch == %client.player)
		   %objSrch = containerSearchNext();
	   if (%objSrch)
			AIMessageThread("ChatMove", %client, -1);
	}
	else if (%task.waitMove < 5 && %task.passes < 1)
	{
		%task.waitMove++;

		//try another pass at deploying 
		if (%task.waitMove == 5)
		{
			%task.waitMove = 0;
			%task.passes++;
			%task.deployAttempts = 0;

			//see if we're *right* underneath the deploy point
			%deployDist2D = VectorDist(getWords(%client.player.position, 0, 1) @ "0", getWords(%task.location, 0, 1) @ "0");
			if (%deployDist2D < 0.25)
			{
				%client.pressjump();
				//%client.deployPack = true;
				%client.player.throwStrength = 2;
				%client.player.use(CameraGrenade);

				//we deployed or tried to deploy equipment because objective's GenDefend is true
				//keep decreasing the value. At 0 we won't deploy GenDefend objectives. - Roberto
				if (%task.GenDefend == true)
				{
					$AIGenDisabledCount[%client.team] = $AIGenDisabledCount[%client.team] - 1;

					if ($AIGenDisabledCount[%client.team] < 0)
					{
						$AIGenDisabledCount[%client.team] = 0; 
					}
					echo("-AI TACTICS- Camera deployed. AIGenDisabledCount for team " @ %client.team @ " is now " @ $AIGenDisabledCount[%client.team]);
				}

	         // check if pack deployed
	         //if(%client.player.getMountedImage($BackpackSlot) == 0)
				   // {
					//don't add a "repairObject" objective for ceiling turrets
					if (%task == %client.objectiveTask)
					{
						AIUnassignClient(%client);
						Game.AIChooseGameObjective(%client);
					}
				//}
			}
		}
	}
	else
	{
		//find a new assignment - and remove this one from the Queue
      if (%task == %client.objectiveTask)
      {
			%client.objective.isInvalid = false;
	      AIUnassignClient(%client);
			Game.AIChooseGameObjective(%client);
       }
	}
}


//--------------------------------------------------------------------------------------- AIO Repair Object ---
//if you have a problem with bot getting stuck navigating to repair any object,
//place objective marker in spot you want bot to stand for repair at about bots waist level.
//if there is no problem with bot getting stuck repairing this object set the AIORepairObject
//markers transform to the same exact transform as the Object (x, y, z) updated - 10-18-2003

//fixed spam error when trying to repair a deployed object in case it gets destroyed while
//in loop. also fixed bots finding repair pack if base is down :) ----------- Lagg... 3-26-2003

function AIORepairObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team != %client.team)
   {
      return 0;      
   }
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   //- - - stop doing this objective if conditions in isDisabled, isEnabled, isSameTeam, are true - roberto
   if ( aiExtraWeightCheck(%this, %client) > -1 ) { return 0; }

   //---MISSION SPECIFIC 1---//
   //1.Copy this entire function to your mission extras script.
   //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
   //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
   //---MISSION SPECIFIC 1---//

   %distToObj = VectorDist(%this.targetObjectId.position, %client.player.position);

   //in ctf don't repair stuff if we're over 9000 miles away - roberto
   if (Game.class $= "CTFGame")
   {
   		if (%distToObj > 175)
   		{
   			return 0;
   		}
   }
      
   //no need to repair if the object isn't in need
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamagePercent() <= 0)
      return 0;
   else
	{
		//if this bot is linked to a human who has issued this command, up the weight
		if (%this.issuedByClientId == %client.controlByHuman)
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			{
				if ($AIWeightHumanIssuedCommand < %minWeight)
					return 0;
				else
					%weight = $AIWeightHumanIssuedCommand;
			}
			else
			{
				// calculate the default...
				%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
				if (%weight < $AIWeightHumanIssuedCommand)
					%weight = $AIWeightHumanIssuedCommand;
			}
		}
		else
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
				return 0;

			// calculate the default...
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
		}

      //add or decrease weight if the objective has inBase or inRoom fields - roberto
      //------------------------------------------------------------------------------
      if (%this.inBase)
      {
         if (%client.inBase)
         {
         	if (Game.class $= "SiegeGame") //inbase in Siege means in the Defense team's base
         	{
            	if (%weight <= 4000) { %weight += 100; }
         	}
         	else
         	{
         		if (Game.class $= "CTFGame" || Game.class $= "DnDGame") //inbase in CTF/DnD means in the enemy base
         		{
					%weight -= 200;
         		}
         	}
         }
         else
         {
            if (Game.class $= "SiegeGame")
            {
               %weight -= 100;
            }
         }
      }

      if (%this.inRoom)
      {
         if (%client.inRoom)
         {
            if (%weight <= 4000) { %weight += 200; }
         }
         else
         {
            %weight -= 100;
         }
      }
      //------------------------------------------------------------------------------

      	//---Some extra checks if we see the object or we're close to it---
		%mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType |
		$TypeMasks::ForceFieldObjectType | $TypeMasks::VehicleObjectType;
		%hasLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %this.targetObjectId.getWorldBoxCenter(), %mask, 0);
		%hasRepPack = (%client.player.getInventory("RepairPack") > 0);

		if (%hasRepPack)
		{
			if ( (%hasLOS && %distToObj <= 90) || (%distToObj <= 40) )
			{
				if ( %this.targetObjectId.getClassName() $= "Turret" ) //if it's a turret, don't increase repair weight if we have humans in our team
				{
		            %enemyTeam = 1;
		            if (%client.team == 1) { %enemyTeam = 2; }

					if ( $AIHumansInTeam[%client.team] >= 1 && $AIHumansInTeam[%enemyTeam] <= 0 )
					{
						%weight -= 100; //decrease our weight a little if we have humans in our team

						//even more if we're not losing
						if ( !$AITeamLosing[%client.team] )
						{
							%weight -= 150;
						}
					}
					else
					{
						%weight += 125; 
					}
				}
				else
				{
					%weight += 125;
				}
			}
		}

		return %weight;
	}
}

function AIORepairObject::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AIRepairObject);
   %client.objectiveTask.initFromObjective(%this, %client);
}

function AIORepairObject::unassignClient(%this, %client)
{
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
}

//----------------------------------------------------------------------------------------------- AI Repair Object ---

function AIRepairObject::initFromObjective(%task, %objective, %client)
{
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
	//need to force this objective to only require a repair pack
   //%task.equipment = %objective.equipment;
   %task.equipment = "RepairPack";
	%task.buyEquipmentSet = %objective.buyEquipmentSet;
	%task.desiredEquipment = %objective.desiredEquipment;
	%task.issuedByClient = %objective.issuedByClientId;
        //%task.location = %objective.location;
        %task.location = %objective.position;
   %task.mode = %objective.mode;

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;
        
	%task.deployed = %objective.deployed;
	if (%task.deployed)
	{
                if (!isObject(%task.targetObject) && isObject(%objective.repairObjective))
		{
                    //if this task is the objective task, choose a new objective
                    if (%task == %client.objectiveTask)
		    {
                        AIUnassignClient(%client);
	                Game.AIChooseGameObjective(%client);			
		    }
                    return;                    
	        }
		%task.location = %objective.position;
		%task.deployDirection = MatrixMulVector("0 0 0 " @ getWords(%objective.getTransform(), 3, 6), "0 1 0");
		%task.deployDirection = VectorNormalize(%task.deployDirection);
	}
}

function AIRepairObject::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(15);
   %client.needEquipment = AINeedEquipment(%task.equipment, %client);
      
   //clear the target object, and range it
   %client.setTargetObject(-1);
	if (! %client.needEquipment)
	{
	    if (%task.deployed)
	    {
                if (!isObject(%task.targetObject) && isObject(%objective.repairObjective))
		{
                    //if this task is the objective task, choose a new objective
                    if (%task == %client.objectiveTask)
		    {
                        AIUnassignClient(%client);
	                Game.AIChooseGameObjective(%client);			
		    }
		    return;                    
	        }
		   %task.repairLocation = VectorAdd(%task.location,VectorScale(%task.deployDirection, -4.0));//was -4
			%client.stepMove(%task.repairLocation, 0.25);
	    }

            //if the objective marker was moved than we have a problemed target
            //and we move to the objective marker to fix bot getting stuck trying
            //to repair object - Lagg... 10-10-2003            
            else if (%task.targetObject.getWorldBoxCenter() == %task.location)
                %client.stepRangeObject(%task.targetObject, "DefaultRepairBeam", 2, 4);                    
            else
	        %client.stepMove(%task.location);
	}

	//mark the current time for the buy inventory state machine
	%task.buyInvTime = getSimTime();
	%task.needToRangeTime = 0;
	%task.pickupRepairPack = -1;
	%task.usingInv = false;

	//set a tag to help the repairPack.cs script fudge acquiring a target
	%client.repairObject = %task.targetObject;

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0 && isObject(%task.targetObject) )
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
		}
	}
    //--------- End teleporters check ---------
}

function AIRepairObject::retire(%task, %client)
{
   %client.setTargetObject(-1);
	%client.repairObject = -1;
}

function AIRepairObject::weight(%task, %client)
{
        //update the task weight
	if (%task == %client.objectiveTask)
		%task.baseWeight = %client.objectiveWeight;

   //let the monitor decide when to stop repairing
   %task.setWeight(%task.baseWeight);
}

function AIRepairObject::monitor(%task, %client)
{
   //echo("AIRepairObject --- monitor");
   //first, buy the equipment
   if (%client.needEquipment)
   {
           %task.setMonitorFreq(5);

		//first, see if we still need a repair pack
		if (%client.player.getInventory(RepairPack) > 0)
		{
			%client.needEquipment = false;
		   %task.setMonitorFreq(15);

			//if this is to repair a deployed object, walk to the deploy point...
			if (%task.deployed)
			{
                           if (!isObject(%task.targetObject) && isObject(%objective.repairObjective))
		           {
                               %client.setTargetObject(-1);
                               //if this task is the objective task, choose a new objective
                               if (%task == %client.objectiveTask)
		               {
                                  //AIUnassignClient(%client);
	                          //Game.AIChooseGameObjective(%client);			
		               }
		               AIClearObjective(%objective.repairObjective);
		               %objective.repairObjective.delete();
		               %objective.repairObjective = "";
                               clearObjectiveFromTable(%objective.repairObjective);//new function below to stop objective spam - Lagg... 1-27-2004
                               echo("AIRepairObject - DOES THIS GET CALLED AT ALL it is an error you know init - is not an object so delete repairobjective");
                               return;                    
	                   }
			   %task.repairLocation = VectorAdd(%task.location,VectorScale(%task.deployDirection, -4.0));//was -4
				//don't move like a maniac when near lava - roberto
				if (%client.inDanger) { %client.stepMove(%task.repairLocation, 0.25, $AIModeWalk); }
				else { %client.stepMove(%task.repairLocation, 0.25, $AIModeExpress); }
			}

         //if the objective marker was moved than we have a problemed target
         //and we move to the objective marker to fix bot getting stuck trying
         //to repair object - Lagg... 10-10-2003            
         
         if ( isObject(%task.targetObject) )
         {
            if (%task.targetObject.getDataBlock().getName() $= "MobileBaseVehicle")
            {
               %client.stepRangeObject(%task.targetObject, "DefaultRepairBeam", 10, 12);          
            }
            else if (VectorDist(%task.targetObject.getWorldBoxCenter(), %task.location) < 1.0)
            {
               %client.stepRangeObject(%task.targetObject, "DefaultRepairBeam", 2, 4);
            }
            else
            {
				//don't move like a maniac when near lava - roberto
				if (%client.inDanger) { %client.stepMove(%task.location, 0.25, $AIModeWalk); }
				else { %client.stepMove(%task.location, 0.25, $AIModeExpress); }
            }
         }
		}
		else
		{
			// check to see if there's a repair pack nearby
			%closestRepairPack = -1;
			%closestRepairDist = 32767;

			//search the AIItemSet for a repair pack (someone might have dropped one...)
			%itemCount = $AIItemSet.getCount();
			for (%i = 0; %i < %itemCount; %i++)
			{
				%item = $AIItemSet.getObject(%i);
				if (%item.getDataBlock().getName() $= "RepairPack" && !%item.isHidden())
				{
                                        %dist = %client.getPathDistance(%item.getWorldBoxCenter());
					if (%dist > 0 && %dist < %closestRepairDist)
					{
                                                %closestRepairPack = %item;
						%closestRepairDist = %dist;
					}
				}
			}

			//choose whether we're picking up the closest pack, or buying from an inv station...
                        //with all these errors by Dynamix it is a wonder the bots do anything - Lagg... 3-25-2003
			if ((isObject(%closestRepairPack) && %closestRepairPack != %task.pickupRepairPack) || (%task.buyInvTime != %client.buyInvTime))
                        {
                                %task.pickupRepairPack = %closestRepairPack;

				//initialize the inv buying
				%task.buyInvTime = getSimTime();
				AIBuyInventory(%client, "RepairPack", %task.buyEquipmentSet, %task.buyInvTime);

				//now decide which is closer
				if (isObject(%closestRepairPack))
				{
                                        //added a little here to check for power - Lagg... - 3-25-2003
					if (isObject(%client.invToUse) && (%client.invToUse.isEnabled()) && (%client.invToUse.isPowered()))
					{
                                                %dist = %client.getPathDistance(%client.invToUse.getWorldBoxCenter());
						if (%dist < %closestRepairDist)
							%task.usingInv = true;
						else
							%task.usingInv = false;
					}
					else
                                           %task.usingInv = false;
				}
				else
					%task.usingInv = true;
			}

			//now see if we found a closer repair pack
			if (!%task.usingInv && isObject(%task.pickupRepairPack))
			{
				//don't move like a maniac when near lava - roberto
				if (%client.inDanger) { %client.stepMove(%task.pickupRepairPack.position, 0.25, $AIModeWalk); }
				else { %client.stepMove(%task.pickupRepairPack.position, 0.25, $AIModeExpress); }

				%distToPack = %client.getPathDistance(%task.pickupRepairPack.position);
				if (%distToPack < 10 && %client.player.getMountedImage($BackpackSlot) > 0)
					%client.player.throwPack();

				//and we're finished until we actually have a repair pack... should now! - Lagg...
				return;
			}
			else
			{
		                %result = AIBuyInventory(%client, "RepairPack", %task.buyEquipmentSet, %task.buyInvTime);
				if (%result $= "InProgress")
                                   return;
				else if (%result $= "Finished")
				{
					%client.needEquipment = false;
				        %task.setMonitorFreq(15);

                                        //if we have to repair the station that we are using get away from it
                                        %client.setDangerLocation(%client.invToUse.getWorldBoxCenter(), 20);

					//if this is to repair a deployed object, walk to the deploy point...
					if (%task.deployed)
					{ 	
					   %task.repairLocation = VectorAdd(%task.location,VectorScale(%task.deployDirection, -6.0));//was -4
						//don't move like a maniac when near lava - roberto
						if (%client.inDanger) { %client.stepMove(%task.repairLocation, 0.25, $AIModeWalk); }
						else { %client.stepMove(%task.repairLocation, 0.25, $AIModeExpress); }
					}
                                        //if the objective marker was moved than we have a problemed target
                                        //and we move to the objective marker to fix bot getting stuck trying
                                        //to repair object - Lagg... 10-10-2003

                                        else if (%task.targetObject.getDataBlock().getName() $= "MobileBaseVehicle")
                                           %client.stepRangeObject(%task.targetObject, "DefaultRepairBeam", 10, 12);         
                                        //else if (%task.targetObject.getWorldBoxCenter() == %task.location)
                                        else if (VectorDist(%task.targetObject.getWorldBoxCenter(), %task.location) < 1.0)
                                           %client.stepRangeObject(%task.targetObject, "DefaultRepairBeam", 2, 4);
                                        else
										   //don't move like a maniac when near lava - roberto
										   if (%client.inDanger) { %client.stepMove(%task.location, 0.25, $AIModeWalk); }
										   else { %client.stepMove(%task.location, 0.25, $AIModeExpress); }
				}
				else if (%result $= "Failed")
				{
			           //if this task is the objective task, choose a new objective
			           if (%task == %client.objectiveTask)
			           {
			              AIUnassignClient(%client);
			              Game.AIChooseGameObjective(%client);
			           }
				   return;
				}
			     }
		          }
   }
	//if we made it past the inventory buying, reset the inv time
	%task.buyInvTime = getSimTime();
   
	//chat
	if (%task.sendMsg)
	{
		if (%task.sendMsgTime == 0)
			%task.sendMsgTime = getSimTime();
		else if (getSimTime() - %task.sendMsgTime > 7000)
		{
			%task.sendMsg = false;
		   if (%client.isAIControlled())
			{
				if (%task.chat !$= "")
				{
					%chatMsg = getWord(%task.chat, 0);
					%chatTemplate = getWord(%task.chat, 1);
					if (%chatTemplate !$= "")
						AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
					else
						AIMessageThread(%task.chat, %client, -1);
				}
				else if (%task.targetObject > 0)
				{
					%type = %task.targetObject.getDataBlock().getName();
					if (%type $= "GeneratorLarge")
						AIMessageThreadTemplate("RepairBase", "ChatSelfRepairGenerator", %client, -1);
					else if (%type $= "StationVehicle")
						AIMessageThreadTemplate("RepairBase", "ChatSelfRepairVehicle", %client, -1);
					else if (%type $= "SensorLargePulse")
						AIMessageThreadTemplate("RepairBase", "ChatSelfRepairSensors", %client, -1);
					else if (%type $= "SensorMediumPulse")
						AIMessageThreadTemplate("RepairBase", "ChatSelfRepairSensors", %client, -1);
					else if (%type $= "TurretBaseLarge")
						AIMessageThreadTemplate("RepairBase", "ChatSelfRepairTurrets", %client, -1);
				}
			}
		}
	}

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            //don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk); }
else { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeExpress); }

            return;
         }
      }
      else
         %task.useTeleport = false;
   }
	else if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
   //--------------------------------------------------- monitor telporter - end -

   //if deployable got killed before we get to it - Lagg...
   if (!isObject(%task.targetObject))
   {
      %client.setTargetObject(-1);

      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         AIUnassignClient(%client);
	 Game.AIChooseGameObjective(%client);
         return;
      }
   }

   if (%task.targetObject.getDamagePercent() > 0)
   {
      //make sure we still have equipment
      %client.needEquipment = AINeedEquipment(%task.equipment, %client);
      if (%client.needEquipment)
      {
         //if this task is the objective task, choose a new objective
         if (%task == %client.objectiveTask)
	 {
            AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	    return;
	 }
      }
      
		if (%task.deployed)
		{
			//see if we're within range of the deploy location
		   %clLoc = %client.player.position;
		   %distance = VectorDist(%clLoc, %task.repairLocation);
			%dist2D = VectorDist(%client.player.position, getWords(%task.repairLocation, 0, 1) SPC getWord(%client.player.position, 2));

			//set the aim when we get near the target...  this will be overwritten when we're actually trying to repair deployed
 			if (%distance < 10 && %dist2D < 10)
 		      %client.aimAt(%task.location, 1000);

			//see if we're at the deploy location
		   if ((%client.pathDistRemaining(20) > %distance + 0.25) || %dist2D > 0.3)
			{
	         %client.setTargetObject(-1);
	         //don't move like a maniac when near lava - roberto
			 if (%client.inDanger) { %client.stepMove(%task.repairLocation, 0.25, $AIModeWalk); }
			 else { %client.stepMove(%task.repairLocation, 0.25, $AIModeExpress); }
			}
			else
			{
				%client.stop();
                                %client.setTargetObject(%task.targetObject, 12, "Repair");//increased range - Lagg...
			}
		}
		else
		{
                        //modified here in case bot gets stuck trying to get to repair spot - Lagg... - 4-21-2003
			%currentTime = getSimTime();
			if (%currentTime > %task.needToRangeTime)
			{
				//force a rangeObject every 45 seconds...
                                %task.needToRangeTime = %currentTime + 45000;
				%client.setTargetObject(-1);
                                %client.setDangerLocation(%task.targetObject.position, 15);
                                //%client.player.cycleWeapon();
                                //if ( !%client.player.getImageTrigger( $BackpackSlot ) )
                                   //%client.player.setImageTrigger( $BackpackSlot, true );
                                
                                //if the objective marker was moved than we have a problemed target
                                //and we move to the objective marker to fix bot getting stuck trying
                                //to repair object - Lagg... 10-10-2003            
                                
                                if (%task.targetObject.getDataBlock().getName() $= "MobileBaseVehicle")
                                   %client.stepRangeObject(%task.targetObject, "DefaultRepairBeam", 10, 12);             
                                //else if (%task.targetObject.getWorldBoxCenter() == %task.location)
                                else if (VectorDist(%task.targetObject.getWorldBoxCenter(), %task.location) < 1.0)
                                   %client.stepRangeObject(%task.targetObject, "DefaultRepairBeam", 2, 4);
                                else
						        //don't move like a maniac when near lava - roberto
								if (%client.inDanger) { %client.stepMove(%task.location, 0.25, $AIModeWalk); }
								else { %client.stepMove(%task.location, 0.25, $AIModeExpress); }
	 }

	 //if we've ranged the object, start repairing, else unset the object
	 else if (%client.getStepStatus() $= "Finished")
         {
            //if we are standing in the trigger of the target object, move a little
            if (%task.targetObject.getDataBlock().getName() $= "MobileBaseVehicle")
            {
               if (VectorDist(%task.targetObject.station.position, %client.player.position) < 2)
                  %client.setDangerLocation(%task.targetObject.station.position, 30);
            }
            else if (%task.targetObject.getDataBlock().getName() $= "StationInventory")
            {
               if (VectorDist(%task.targetObject.position, %client.player.position) < 2)
                  %client.setDangerLocation(%task.targetObject.position, 30);
            }
                        
	    //dissolve the human control link
	    if (%task == %client.objectiveTask)
	       aiReleaseHumanControl(%client.controlByHuman, %client);

	    %client.setTargetObject(%task.targetObject, 12, "Repair");
            //if (%task.targetObject.getDataBlock().getName() $= "MobileBaseVehicle")
            //echo("REPAIRING A MPB = " @ getTaggedString(%client.name));
         }
         else
            %client.setTargetObject(-1);
      }
   }
   else
   {
      %client.setTargetObject(-1);
      
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         AIUnassignClient(%client);
         Game.AIChooseGameObjective(%client);
      }
   }
}

//--------------------------------------------------------------------------------------------- AIO Touch Object ---
//Heavily modified Lagg... 5-17-2005

function AIOTouchObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
   {
      return 0;
   }

   if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
   {
      return 0;
   }

   if ( !isObject(%this.targetObjectId) )
   {
   	  return 0;
   }

   %weightReduction = 0; //will increase if the player's grabbing the enemy flag and doesn't have a sensorjammer pack or any pack

   //some extra checks for CTF - roberto
   if (Game.class $= "CTFGame")
   {
      %flagGrabEnFlagHome = (%this.mode $= "FlagGrab" && %this.targetObjectId.isHome);
      %droppedFlagNoCarrier = (%this.mode $= "FlagDropped" && !(%this.targetObjectId.isHome || %this.targetObjectId.carrier !$= ""));

      //If we're grabbing the enemy flag that's home or a dropped flag 
      if ( %flagGrabEnFlagHome || %droppedFlagNoCarrier)
      {
         //if isDisabled, isEnabled, isSameTeam, are true
         if (aiExtraWeightCheck(%this, %client) > -1)
         {
            //stop doing the objective if the enemy flag's at the enemy flagstand
            if (%flagGrabEnFlagHome) 
            {
               return 0;
            }
            //if it's a dropped enemy flag only get it if it's not too close to the enemy flagstand
            else if (%droppedFlagNoCarrier && %client.team != %this.targetObjectId.team)
            {
               %dropFlagToEnStandDist = VectorDist(%this.targetObjectId.getWorldBoxCenter(), %this.targetObjectId.originalPosition);

               if (%dropFlagToEnStandDist < 75) //too close to enemy flagstand
               {
                  return 0;
               }
            }
         }


         %distance = %client.getPathDistance(%this.targetObjectId.getWorldBoxCenter());
         %droppedEnemyFlag = false; 

         if (%this.mode $= "FlagDropped")
         {
            //if it's the enemy flag
            if ( %this.targetObjectId.team != %client.team )
            {
               //decrease the weight if we don't have a sensorjammer pack or no pack
               if (%client.player.getInventory("SensorJammerPack") < 1 || %client.player.getMountedImage($BackpackSlot) == 0)
               {
                  %weightReduction = 300;
               }

               //find our home flag location
               %homeTeam = %client.team;
               %homeFlag = $AITeamFlag[%homeTeam];
               
               //if we're far from our homeflag position, then allow extra checks by making %droppedEnemyFlag True
               if (%client.getPathDistance(%homeFlag.originalPosition) > 300)
               {
                  %droppedEnemyFlag = true;
               }
            }
          }
          else
          {
            //decrease the weight if we don't have a sensorjammer pack or no pack
            if (%client.player.getInventory("SensorJammerPack") < 1 || %client.player.getMountedImage($BackpackSlot) == 0)
            {
               %weightReduction = 300;
            }
          }

         //if near generator in ctf, cloakers and heavies have low priority of grabbing flag because we want them destroying gens instead
         if (%client.isCloakerOrHeavy && %this.genNear)
         {
            if (%this.mode $= "FlagGrab" || (%this.mode $= "FlagDropped" && %droppedEnemyFlag) )
            {
               return 0;
            }
         }

         //do extra checks if we aren't really close 
           if (%distance > 75)
           {
              //if we have low health and we're pretty far, low priority
              if ( %client.player.getDamagePercent() >= %client.grabFlagDamagePercent )
              {
                  if (%this.mode $= "FlagGrab" || (%this.mode $= "FlagDropped" && %droppedEnemyFlag) )
                  {
                     return 0;
                  }
              }
           }
       }
   }

   switch$ (%this.mode)
   {
      case "TouchFlipFlop":
         if(%this.targetObjectId.team == %client.team || %this.targetObjectId.isHidden())
         {
	        return 0;
         }

         %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

         //don't even try to touch a flip flop surrounded by a forcefield - Lagg...
         if ($CurrentMissionType $= "Siege")
         {
            InitContainerRadiusSearch(%this.targetObjectId.position, 14, $TypeMasks::ForceFieldObjectType);
            %objSearch = containerSearchNext();
            if (%objSearch > 0 && %objSearch.isPowered() && %objSearch.team != %client.team)
            {
               return 0;
            }

            //---MISSION SPECIFIC 1---//
            //1.Copy this entire function to your mission extras script.
            //2.Put mission specific checks here to see if we should run this objective or not, or to decrease/increase weight. - roberto
            //Look in the SiegeScriptingNotes to see what Lagg used for a map for this objective and how you could modify it for yours.
            //---MISSION SPECIFIC 1---//
         }
            //add or decrease weight if the objective has inBase or inRoom fields - roberto
            //------------------------------------------------------------------------------
            if (%this.inBase)
            {
               if (%client.inBase)
               {
                  if (%weight <= 4000) { %weight += 100; }
               }
            }

            if (%this.inRoom)
            {
               if (%client.inRoom)
               {
                  if (%weight <= 4100) { %weight += 350; }
               }
            }
            //------------------------------------------------------------------------------
            
        return %weight;
      case "FlagGrab":
			if (! %this.targetObjectId.isHome)
         	{
				  return 0;
         	}
			else
         	{
         		%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

         		if ( (%weight - %weightReduction) >= 3500)
         		{
         			%weight -= %weightReduction;
         		}
         		else if ( (%weight - 200) >= 3450 && %weightReduction > 0)
         		{
         			%weight -= 200;
         		}
         		else if (%weightReduction > 0)
         		{
         			%weight -= 150;
         		}

         		//more weight for sensor jamming dudes
         		if (%client.player.getInventory("SensorJammerPack") > 0 && %weight < 4000)
         		{
         			%weight += 50;
         		}

				return %weight;
         	}
      case "FlagDropped":
			if ((%this.targetObjectId.isHome) || (%this.targetObjectId.carrier !$= "")) 
            {
				return 0;
            }
			else
            {
                //we want to get the team trying to cap acting a little better than the Defending team - Lagg...
                if (%client.team != %this.targetObjectId.team)
                {
                    %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
                    %distance = %client.getPathDistance(%this.getWorldBoxCenter());
                    
                    //if we're holding the flag don't go too far to retrieve our dropped flag, reaching our base is more important - roberto
                    if (%client.player.holdingFlag && %distance > 100)
                    {
                        return 0;
                    }
                    
                    if (%distance < 0)
                    {
                        %distance = 32767;
                    }

                    //see if we're within 300 m
                    if (%distance < 300)
                    {
                        %weight += 100;
                    }

                    //see if we're within 200 m
                    if (%distance < 200)
                    {
                        %weight += 100;
                    }

                    //see if we're within 100 m
                    if (%distance < 100)
                    {
                        %weight += 100;
                    }

                    //see if we're within 45 m
                    if (%distance < 45)
                    {
                        %weight += 100;
                    }

	         		if ( (%weight - %weightReduction) > 3400)
	         		{
	         			%weight -= %weightReduction;
	         		}

                    return %weight;
                }
                else
                {
                    return AIODefault::weight(%this, %client, %level, %inventoryStr);
                }
            }
      case "FlagCapture":
						if (%this.targetObjectId.carrier != %client.player)
						{
							return 0;
						}

						//find our home flag location
						%homeTeam = %client.team;
						%homeFlag = $AITeamFlag[%homeTeam];
						%this.location = %homeFlag.originalPosition;

                        //never bump the pilot with the flag - Lagg...
                        if (%client.player.isMounted())
                        {
                           return 10000;
                        }
                        else
                        {
			   			return AIODefault::weight(%this, %client, %level, %inventoryStr);
			   		}
   }
   return 0;
}

function AIOTouchObject::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AITouchObject);
   %client.objectiveTask.initFromObjective(%this, %client);
   
   //create an AIOEscortPlayer objective to help out, if required
   if (%this.mode $= "FlagGrab")
   {
      %client.escort = new AIObjective(AIOAttackLocation) {
					position = %this.targetObjectId.position;
					dataBlock = "AIObjectiveMarker";
					lockCount = "0";
					homingCount = "0";
                                        description = "Attack Flag Location for Capper";
					targetClientId = "-1";
					targetObjectId = "-1";
					location = %this.targetObjectId.position;
					weightLevel1 = $AIWeightEscortCapper[1];
					offense = "1";
					defense = "0";
                                        desiredEquipment = "SniperRifle EnergyPack Mortar MortarAmmo";
					buyEquipmentSet = "LightEnergySniper";
					chat = "ChatCmdAttackFlag";
					issuedByHuman = "0";
					issuedByClientId = "-1";
					forceClientId = "-1";
				};
      MissionCleanup.add(%client.escort);
      $ObjectiveQ[%client.team].add(%client.escort);
   }   
   //after we grabbed the enemy flag
   else if (%this.mode $= "FlagCapture")
   {
      %client.escort = new AIObjective(AIOEscortPlayer)
                        {
							      dataBlock = "AIObjectiveMarker";
                           weightLevel1 = $AIWeightEscortCapper[1];
                           weightLevel2 = $AIWeightEscortCapper[2];
                           description = "Protect flag carrier " @ getTaggedString(%client.name);
                           targetClientId = %client;
                           offense = true;
									desiredEquipment = "Disc DiscAmmo";
									buyEquipmentSet = "LightEnergyDefault";
                           chat = "ChatCmdDefendCarrier";
                        };
      MissionCleanup.add(%client.escort);
      $ObjectiveQ[%client.team].add(%client.escort);
   }
}

function AIOTouchObject::unassignClient(%this, %client)
{

   //error("AIOTouchObject::unassignClient - Ahhh!");

   //kill the escort objective
   if (%client.escort)
   {
      clearObjectiveFromTable(%client.escort);
      AIClearObjective(%client.escort);
      %client.escort.delete();//seen spam error here if escort gets killed before call - Lagg... 1-7-2005
      %client.escort = "";
   }   
   %client.removeTask(%client.objectiveTask);
   %client.objectiveTask = "";
    //when this is smaller than 0, do a disc jump. We must add delays like this because the bot shoots before turning.
    %client.discJumpTimer = 100; //the reseted value, it will decrease when activated
    %client.discJumpTimerStarted = false; //we don't decrease the value now, reset
    %client.capperGrabbingFlag = false; //when this is true we ignore detecting turrets, mines, invs, or enemies and we just grab the flag - roberto
}



//----------------------------------------------------------------------------------------------- AI Touch Object ---
//ramdom set to go for flag/flipflop -                                      - Lagg... - 4-14-2003

function AITouchObject::initFromObjective(%task, %objective, %client)
{
		%task.objectiveId = %objective;
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
   %task.mode = %objective.mode;
   %task.equipment = %objective.equipment;
   %task.buyEquipmentSet = %objective.buyEquipmentSet;
   %task.desiredEquipment = %objective.desiredEquipment;
   %task.issuedByClient = %objective.issuedByClientId;
   %task.chat = %objective.chat;
   %task.useTeleport = false;
   %task.genNear = %objective.genNear; //if flag distance from a generator is smaller than 150m

   //additional weight if these aren't empty strings and zone triggers are added - roberto
   %task.inBase = %objective.inBase;
   %task.inRoom = %objective.inRoom;

   //don't add additional zone weight in Capture and hold because base ownership keeps changing 
   if (Game.class $= "CnHGame")
   {
      %task.inBase = "";
      %task.inRoom = "";
   }

   //list of objects that are required to be enabled, disabled, or in the same team in order to do this objective
   %task.isDisabled = %objective.isDisabled;
   %task.isEnabled = %objective.isEnabled;
   %task.isSameTeam = %objective.isSameTeam;

   //follow path - Roberto...
   %task.paths = %objective.paths;
   %task.followPath = false;

   //initialize other task vars
   %task.sendMsgTime = 0;

   //MPB Telepoter
   %task.useTeleport = false;

   //Sparky's Teleporters
   %task.useTele = false;
   %task.tele = -1;

   %task.sendMsgTime = 0;

   if (%task.mode $= "FlagGrab" || %task.mode $= "FlagCapture")
   {
      %task.sendMsg = true;
   }
   else
   {
      %task.sendMsg = false;
   }

   if (%task.mode $= "FlagCapture")
   {
      %task.location = %objective.location;
   }
   else if(%task.mode $= "TouchFlipFlop" || %task.mode $= "FlagGrab")
   {
      %task.location = %objective.location;
      //if  (%task.desiredEquipment $= "" && %client.player.getArmorSize() $= "Light" && %client.player.getInventory("Blaster") > 0)
      if  (%task.desiredEquipment $= "")
      {
         //pick a random set based on armor...
         %randNum = getRandom();
         if (%randNum < 0.2)
         {
            %task.desiredEquipment = "SensorJammerPack";
            %task.buyEquipmentSet = "HeavyEnergySet";
         }
         else if (%randNum < 0.4)
         {
            %task.desiredEquipment = "SensorJammerPack";
            %task.buyEquipmentSet = "LightSensorJammer";
         } 
         else if (%randNum < 0.6)
         {
            %task.desiredEquipment = "CloakingPack";
            %task.buyEquipmentSet = "LightCloakSet";
         }
         else if (%randNum < 0.8)
         {
            %task.desiredEquipment = "ShieldPack";
            %task.buyEquipmentSet = "HeavyShieldOff";
         }
         else
         {
            %task.desiredEquipment = "EnergyPack";
            %task.buyEquipmentSet = "LightEnergyDefault";
         }
      }
   }
   else
      %task.location = "";
}

function AITouchObject::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(15);
   %task.engageTarget = 0;

   //if we are to far from inventory are close to target forget inventory - Lagg...
   %client.needEquipment = false; 
   if (%task.buyEquipmentSet !$= "")
   {
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
         %equipmentList = %task.desiredEquipment;

      %needArmor = AIMustUseRegularInvStation(%equipmentList, %client);
      %result = AIFindClosestInventory(%client, %needArmor);
      %closestInv = getWord(%result, 0);
      %closestDist = getWord(%result, 1);
      if (AINeedEquipment(%equipmentList, %client) && %closestInv > 0)
      {
         //find where we are
	 %distToObject = %client.getPathDistance(%task.location);
	 if (%closestDist < 100 && %distToObject > 300)
	    %client.needEquipment = true;
      }
   }

   //mark the current time for the buy inventory state machine
   %task.buyInvTime = getSimTime();

   //Make the capper follow a path to the flag - roberto
   if (%task.mode $= "FlagGrab")
   {
      //--------------------------------------------------- follow paths --
      //set the destination paths for each team and game type
   
      //first see how many paths we have
      //check if attack path exists
      if (Game.class $= "SiegeGame")
         %team = Game.offenseTeam == %client.team ? 1 : 2;
      else
         %team = %client.team;

      //first see how many paths we have
      %mx = getWordCount(%task.paths);

	  %objPos = %task.targetObject.getWorldBoxCenter();
      %farEnough = (%client.getPathDistance(%objPos) > 200);

	  //modified so we remember last path position, or we start at the last marker if needed - roberto
      //I know all of this could had easily be done with a for loop looking through all the markers and just getting the closest one to continue a path
      //but maybe when you have many paths it's not so fast to loop through 10 markers and it's faster to just instantly set the destination to a remembered marker

      if (%mx > 0 && %farEnough && %task.targetObject.isHome) //only follow path if the enemy flag is home, else just grab it
      {
         if (%client.lastIndexPathToFlag > -1 && %client.lastPathToFlag > -1)
         {
         	%task.locationIndex = %client.lastIndexPathToFlag; //last marker in path
         	%tg = %client.lastPathToFlag; 	//which path to use
         	%task.group = nameToId("T" @ %team @ "PathToFlag" @ %tg);
            %task.count = %task.group.getCount();  
		    //echo("PATH TO FLAG " @ %client.nameBase @ " following from where we left of, marker" @ %task.locationIndex @ " from PathToFlag " @ %tg);
         }
         else
         {
			%random = mFloor(getRandom(1, %mx));
			%random--;
			%tg = getWord(%task.paths, %random); //which path to use
			%client.lastPathToFlag = %tg;

			%task.group = nameToId("T" @ %team @ "PathToFlag" @ %tg);
            %task.count = %task.group.getCount();  
			
			//get the last marker in the path, if we are close we shouldn't go to the beginning of the path, start from the last marker
			%lastMarker = %task.group.getObject(%task.count -1); 
            %distToLastMarker = VectorDist(%client.player.position, %lastMarker.position);

            if (%distToLastMarker < 300) //if we are close, start from the last marker
            {
				%task.locationIndex = %task.count -1; 
				%client.lastIndexPathToFlag = %task.count -1;
				//echo("CAPTURE FLAG " @ %client.nameBase @ " on first try, close to enemy flag, start following last marker " @ %task.locationIndex @ " from PathToFlag " @ %tg);
            }//
            else //we are not even close to the last marker in the path, try two times to see if we're close to last marker, this gonna be long but better than looping through 6 markers per path and checking distance.
            {
            	//try with another path again, will only work for two paths, no point trying with tons of paths
            	if (%tg < 2)
            	{
            		%tg++;
            	}
            	else
            	{
            		%tg--;
            	}

            	%nextPathExists = isObject(nameToId("T" @ %team @ "PathToFlag" @ %tg));
            	%nextPathFoundMarker = false;

            	//if there's one more path to try, see if the distance from the last marker is good or not
            	if (%nextPathExists)
            	{
       			    %client.lastPathToFlag = %tg;
					%task.group = nameToId("T" @ %team @ "PathToFlag" @ %tg);
					%task.count = %task.group.getCount();  

					//get the last marker in the path, if we are close we shouldn't go to the beginning of the path, start from the last marker
					%lastMarker = %task.group.getObject(%task.count -1); 
					%distToLastMarker = VectorDist(%client.player.position, %lastMarker.position);

		            if (%distToLastMarker < 300) //if we are close, start from the last marker
		            {
		            	%nextPathFoundMarker = true;
						%task.locationIndex = %task.count -1; 
						%client.lastIndexPathToFlag = %task.count -1;
						//echo("CAPTURE FLAG " @ %client.nameBase @ " on second try, close to enemy flag, start following last marker " @ %task.locationIndex @ " from PathToFlag " @ %tg);
		            }
            	}

            	if (!%nextPathFoundMarker) //only a single path exists, and we aren't close to the last marker in it, or we have no memory of following it, start from the beginning
            	{
					%random = mFloor(getRandom(1, %mx));
					%random--;
					%tg = getWord(%task.paths, %random); //which path to use
					%task.group = nameToId("T" @ %team @ "PathToFlag" @ %tg);
					%task.count = %task.group.getCount();  

					%task.locationIndex = 0; 
					%client.lastPathToFlag = %tg;
					%client.lastIndexPathToFlag = 0;

					//echo("CAPTURE FLAG " @ %client.nameBase @ " following path from the beginning.");
            	}
            }
         }

         //make sure there are markers in the PathToFlag group
         if (%task.count > 0)
         {
            %task.followPath = true;
         }
          else
          {
              %task.followPath = false;
          }
      }
       else
       {
           %task.followPath = false;
       }
   }

    //--------- Check both teleporters -------- + edit - Roberto
   	%trgObj = %task.objectiveId;

   	//--- Start MPB Check ---
	//first check for deployed MPB
	%result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
	%closestMPB = getWord(%result, 0);
	%closestMPBDist = getWord(%result, 1);

	//next find the teleporter
	%result = SweepForTeleporters(%client);//new function in aiVehicle.cs
	%closestTel = getWord(%result, 0);
	%closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0 && isObject(%task.targetObject) )
   {
      %disToTarg = %client.getPathDistance(%trgObj.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %trgObj.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
      {
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      }
      else
      {
         %task.useTeleport = false;             
      }
   }
	//--- End MPB Check ---

	//--- Start Sparky's Check ---
	%result = aiTeleporterCheck(%client, %trgObj);
	%task.useTele = getWord(%result, 0);//true or false
	%task.tele = getWord(%result, 1);//this is the linked tele to use

	//if we found both a MPB teleporter and a sparky's teleporter, use what's closer to us
	if (%task.useTele && %task.useTeleport)
	{
		if ( %client.getPathDistance(%task.tele.getTransform()) < %closestTelDist )
		{
			%task.useTeleport = false;  //don't use the MPB teleport then
		}
	}
    //--------- End teleporters check ---------
}

function AITouchObject::retire(%task, %client)
{
}

function AITouchObject::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;

   %player = %client.player;

   //see if we can find someone to shoot at...
   if (%client.getEngageTarget() <= 0)
   {
      %losTimeout = $AIClientMinLOSTime + ($AIClientLOSTimeout * %client.getSkillLevel());
      %myLocation = %client.player.getWorldBoxCenter();
      %result = AIFindClosestEnemy(%client, 40, %losTimeout);
      %task.engageTarget = getWord(%result, 0);
   }

   %task.setWeight(%task.baseWeight); 
}

function AITouchObject::monitor(%task, %client)
{
   //echo("AITouchObject::monitor - " @ getTaggedString(%client.name) @ ", " @ %client.objectiveWeight);
	%canfight = false; 
	%player = %client.player;

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            //don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk); }
else { %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeExpress); }

            //echo("moving towards a TELEPORTER");
            return;
         }
      }
      else
         %task.useTeleport = false;
   }
   //--------------------------------------------------- monitor telporter - end -

   //first, buy the equipment
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
      {
         %equipmentList = %task.equipment;
      }
      else
      {
	%equipmentList = %task.desiredEquipment;
	%result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
		}
      if (%result $= "InProgress")
      {
         return;
      }
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(15);
	 %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
         %task.setMonitorFreq(15);
	 %client.needEquipment = false;
      }
   }

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();

   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
	 if (%client.isAIControlled())
	 {
	    if (%task.chat !$= "")
	    {
	       %chatMsg = getWord(%task.chat, 0);
	       %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
	          AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, -1);
            }
         }
      }
   }

   //get clients skill
   %skill = %client.getSkillLevel();

   //echo("Skill = " @ %skill);

   //keep updating the position, in case the flag is flying through the air...
   if (%task.location !$= "")
   {
      %touchPos = %task.location;
   }
   else
   {
      %touchPos = %task.targetObject.getWorldBoxCenter();
   }
      
   //see if we need to engage a new target
   %engageTarget = %client.getEngageTarget();
   if (!AIClientIsAlive(%engageTarget) && %task.engageTarget > 0)
   {
      %client.setEngageTarget(%task.engageTarget);
   }
   //else see if we should abandon the engagement
   else if (AIClientIsAlive(%engageTarget)) 
   {
      %myPos = %client.player.getWorldBoxCenter();
      %testPos = %engageTarget.player.getWorldBoxCenter();
      %distance = %client.getPathDistance(%testPos);
      if (%distance < 0 || %distance > 50)
      {
        %client.setEngageTarget(-1);
      }
   }

   //see if we have completed our objective
   if (%task == %client.objectiveTask)
   {
      %completed = false;
      switch$ (%task.mode)
      {
         case "TouchFlipFlop":
	    if (%task.targetObject.team == %client.team)
	       %completed = true;
	 case "FlagGrab":
	    if (!%task.targetObject.isHome)
               %completed = true;
	 case "FlagDropped":
	    if ((%task.targetObject.isHome) || (%task.targetObject.carrier !$= ""))
	       %completed = true;
	 case "FlagCapture":
	    if (%task.targetObject.carrier != %client.player)
	       %completed = true;
      }
      if (%completed)
      {
         %client.lastIndexPathToFlag = -1;
	     %client.lastPathToFlag = -1;
         AIUnassignClient(%client);
	 	Game.AIChooseGameObjective(%client);
	 return;
      }
   }

	if (%task.useTele) //if we are using Sparky's teleporters
	{
		%pos = %client.player.position;
		%dest = %task.tele.position;

		//useTele is turned off by Teleporters.cs after entering, but also check if tele is disabled
		if (!%task.tele.isPowered() || %task.tele.isDisabled() ) 
		{
			%task.useTele = false;
			return;
		}
		else
		{
			//error("AIAttackObject::monitor - WE ARE USING SPARKY'S TELEPORTER - " @ getTaggedString(%client.name));
			//don't move like a maniac when near lava - roberto
if (%client.inDanger) { %client.stepMove(%dest, 0.25, $AIModeWalk); }
else { %client.stepMove(%dest, 0.25, $AIModeExpress); }

			return;
		}
	}
   else if (%task.mode $= "FlagCapture" && %skill > 0.4)
   {
      %player = %client.player;
      %myArmor = %player.getArmorSize();
      %task.followPath = false;

      //cut away any sand bags
      if (hasLargePack(%player))
         %player.throwPack();

      if (!%player.isMounted())
      {
         //see if we can steal a ride
         %abV = -1;
         if (%myArmor !$= "Heavy")
         {
            //max dis to search for abandon vehicles, 250
            %clDis = 250;
            for (%i = 0; %i < $aiVehicleSet.getCount(); %i ++)
            {         
               %vehicle = $aiVehicleSet.getObject(%i);
               if (isObject(%vehicle))
               {            
                  %type = %vehicle.getDataBlock().getName();
                  if (%vehicle.getMountNodeObject(0) <= 0 && %type !$= "MobileBaseVehicle")
                  {
                     if (%myArmor $= "Light" || %type !$= "ScoutVehicle")
                     {
                        %dis = VectorDist(%player.position, %vehicle.position);
                        %dis1 = VectorDist(%vehicle.position, %touchPos);
                
                        if (%dis < %clDis && %dis1 > 600)
                        {                  
                           %abV = %vehicle;
                           %clDis = %dis;
                        }
                     }
                  }
               }
            }
         }

         if (%abV > 0)
         {
            //since we have the flag, hurry up and just touch the vehicle anywhere to mount it.
            %client.pilotVehicle = true;//needed to let ai mount pilot seat

            if (%client.inWater)
            {               
               %vPos2D = getWord(%abV.getPosition(), 0) SPC getWord(%abV.getPosition(), 1) SPC "0";
               %myPos2D = getWord(%client.player.position, 0) SPC getWord(%client.player.position, 1) SPC "0";
               if (vectorDist(%myPos2D, %vPos2D) < 2.5)
               {
                  %client.pressJump();
                  %client.stepMove(%abV.getWorldBoxCenter(), 0.25, $AIModeGainHeight);
               }
               else
                  %client.stepMove(%abV.position, 0.25, $AIModeExpress);
            }
            else 
               %client.stepMove(getWords(%abv.getTransform(), 0, 2), 0.25, $AIModeMountVehicle);
         }
         //we don't have a close vehicle to steal
         else
         {  
            %homeFlag = $AITeamFlag[%client.team];

            //if we're within range of the flag's home position, and the flag isn't home, start idling...
            if (VectorDist(%player.position, %touchPos) < 50 && !%homeFlag.isHome) 
            {
               %canfight = true; //we will fight enemies

               if (%client.getStepName() !$= "AIStepIdlePatrol")
               {
	              %client.stepIdle(%touchPos);
               }
            }
            else //we are too far away from the flag's home position, we must reach the original position of our flag
            {
	           
	            if ($AIFastCappers && Game.class $= "CTFGame" && !%client.inBase) //make bot look for markers to go at homeflag with high speed - roberto 
	            {
					%ourpos = %client.player.position;
					%homeFlagPos = %homeFlag.originalPosition;  //position of our home flag
					%playerToHomeFlagdist = VectorDist(%ourpos, %homeFlagPos);
					%foundMarker = false;
					%player = %client.player;
					
               //%playerAlt = ( getWord(%ourpos, 2) - getTerrainHeight(%ourpos) );

					 if (%playerToHomeFlagdist >= 300) //if we are far enough from the home flag 
					 {
						%lineofsight_obs = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType | $TypeMasks::ForceFieldObjectType; 
						%hasFlagLOS = !containerRayCast(%ourpos, %homeFlagPos, %lineofsight_obs, 0); 
						%enemOrigFlagPos = $AITeamFlag[%task.targetObject.team].originalPosition;

						%abovePos = getWord(%ourpos, 0) SPC getWord(%ourpos, 1) SPC getWord(%ourpos, 2) + 50;
                  %belowPos = getWord(%ourpos, 0) SPC getWord(%ourpos, 1) SPC getWord(%ourpos, 2) - 25;

						%hasUpLos = !containerRayCast(%ourpos, %abovePos, %lineofsight_obs, 0); 
                  %playerAltOk = containerRayCast(%ourpos, %belowPos, %lineofsight_obs, 0); 

					 	if (!%hasFlagLOS && %hasUpLos) //if we can't see the flag, try finding a boost marker if our view is not blocked above
					 	{
						    %task.CapperBoostSimgroup = nameToId("CapperBoostPoints");
						    %markerCount = %task.CapperBoostSimgroup.getCount();

						    //get the marker that is closest to our home flag 
						    for (%i = 0; %i < %markerCount; %i++)
						    {
						       %currMarker = %task.CapperBoostSimgroup.getObject(%i);
						       %markerToHomeFlagDist = (VectorDist(%currMarker.position, %homeFlagPos) + 1);
						       %playerToMarkerDist = VectorDist(%client.player.position, %currMarker.position);

						       //only if that marker is closer to the homeflag than us, and if the marker's far enough from us, and if we have energy for AIModeGainHeight
						       //if ( (%markerToHomeFlagDist < %playerToHomeFlagdist) && (%playerToMarkerDist > 15) && ( %client.player.getEnergyLevel() > 0.25 ) ) 
                         if ( (%markerToHomeFlagDist < %playerToHomeFlagdist) && (%playerToMarkerDist > 8) ) 
						       {
						          %canSeeMarker = !containerRayCast(%ourpos, %currMarker.position, %lineofsight_obs, 0); 

						          if (%canSeeMarker) 
						          {
                                        %client.stepEngage(-1);
                                        %client.lastDamageClient = -1;
                                        %client.lastDamageTurret = -1;
                                        %client.setEngageTarget(-1);
                                        %client.engageRemeq = -1;//added - Lagg...
                                        %client.pickUpItem = -1;//added - Lagg...

                                       %client.pressJump();

                                       if (%playerAltOk)
                                       {
                                          %client.stepMove(%currMarker.position, 0.25, $AIModeGainHeight);
                                       }
                                       else
                                       {
                                          %client.stepMove(%currMarker.position, 0.25, $AIModeExpress);
                                       }

           //                              //if we still near the enemy base use all our jet power to get away faster, else jet normally
           //                              if (VectorDist(%client.player.position, %enemOrigFlagPos) < 50)
           //                              {
											// %client.pressJump();
           //                              	%client.stepMove(%currMarker.position, 0.25, $AIModeGainHeight);
           //                              	//echo("Capper " @ getTaggedString(%client.name) @ " - GOING HOME USING MARKER- FULL JETS ACTIVE");
           //                              }
           //                              else
           //                              {
           //                              	%client.pressJump();
											// %client.stepMove(%currMarker.position, 0.25, $AIModeExpress); 
           //                              	//echo("Capper " @ getTaggedString(%client.name) @ " - GOING HOME USING MARKER - NORMAL JETS");
           //                              }

                                        %client.aimAt(%currMarker.position, 2500);
                                        applyKick(%client.player, "foward", %client.runToBaseSpeed, false);

                                        %foundMarker = true;
                                        %canfight = false;
                                        break;
						          }
						       }
						    }
						    //if we did not find a visible marker just normally walk to our flag
						    if (!%foundMarker)
							{
		                       %client.stepEngage(-1);
		                       %client.lastDamageClient = -1;
		                       %client.lastDamageTurret = -1;
		                       %client.setEngageTarget(-1);
		                       %client.engageRemeq = -1;//added - Lagg...
		                       %client.pickUpItem = -1;//added - Lagg...

								//don't move like a maniac when near lava - roberto
								if (%client.inDanger) { %client.stepMove($AITeamFlag[%client.team].position, 0.25, $AIModeWalk); }
								else { %client.stepMove($AITeamFlag[%client.team].position, 0.25, $AIModeExpress); }

						    	%canfight = false;
						    }
						}
						else  //we have LOS to flag, boost towards it
						{
                            %client.stepEngage(-1);
                            %client.lastDamageClient = -1;
                            %client.lastDamageTurret = -1;
                            %client.setEngageTarget(-1);
                            %client.engageRemeq = -1;//added - Lagg...
                            %client.pickUpItem = -1;//added - Lagg...
                            %client.aimAt(%touchPos, 2500);

                            //if we still near the enemy base use all our jet power to get away faster, else jet normally
                            if (VectorDist(%client.player.position, %enemOrigFlagPos) < 55)
                            {
								%client.pressJump();
                           		%client.stepMove($AITeamFlag[%client.team].position, 0.25, $AIModeGainHeight);
                            	//echo("Capper " @ getTaggedString(%client.name) @ " - GOING HOME - FULL JETS ACTIVE");
                            }
                            else
                            {
                           		%client.stepMove($AITeamFlag[%client.team].position, 0.25, $AIModeExpress);
                            	//echo("Capper " @ getTaggedString(%client.name) @ " - GOING HOME - NORMAL JETS");
                            }

                            applyKick(%client.player, "foward", %client.runToBaseSpeed, false);
                            %canfight = false;
						}
					 }
					 else //we are close to the flag, move normally without boosts, but don't fight yet since our flag is home, just hurry and move home
					 {
                          %client.stepEngage(-1);
                          %client.lastDamageClient = -1;
                          %client.lastDamageTurret = -1;
                          %client.setEngageTarget(-1);
                          %client.engageRemeq = -1;//added - Lagg...
                          %client.pickUpItem = -1;//added - Lagg...
                          %client.stepMove(%touchPos, 0.25, $AIModeExpress);
                          %canfight = false;	 	
					 }
	            }
                else if ($AIDiscJump && Game.class $= "CTFGame") //fast cappers is disabled, try disc jump in CTF 
                {
					AITryDiscJumpFlag(%client);
					%discJumpSuccess = AIDoDiscJump(%client); 
					//if DJ failed just move to the touchpos
					if (!%discJumpSuccess)
					{
						//don't move like a maniac when near lava - roberto
						if (%client.inDanger) { %client.stepMove(%touchPos, 0.25, $AIModeWalk); }
						else { %client.stepMove(%touchPos, 0.25, $AIModeExpress); }
					}
                }
                else //we aren't playing CTF or fastcappers/discjump is enabled, just move to touchpos
                {
					//don't move like a maniac when near lava - roberto
					if (%client.inDanger) { %client.stepMove(%touchPos, 0.25, $AIModeWalk); }
					else { %client.stepMove(%touchPos, 0.25, $AIModeExpress); }
                }
            }
         }
      }
      else if (%player.isMounted())// -------------------------------------- ADD FLIGHT CONTROLS HERE --- Lagg...
      {
         //get the vehicle and velocity
         %vehicle = %client.vehicleMounted;
         
         %type = %vehicle.getDataBlock().getName();
         %mySpd = VectorLen(%vehicle.getVelocity());
         %alt = getWord(%player.position, 2) - getTerrainHeight(%player.position);
         %myZ = getWord(%vehicle.getWorldBoxCenter(), 2);
         %collision = aiCollisionCheck(%vehicle, (%mySpd * 2));

         //just in case he smacks into an mpb
         if (%type $= "MobileBaseVehicle")
            AIDisembarkVehicle(%client);

         //echo("Speed = " @ %mySpd);

         //------------------------------------------------------- Avoidance ---
         if (%mySpd < 0.2)
         { 
            //InitContainerRadiusSearch(%vehicle.getWorldBoxCenter(), 9.0, $TypeMasks::InteriorObjectType |


             // $TypeMasks::VehicleObjectType | $TypeMasks::TSStaticShapeObjectType);
            //%avoid = containerSearchNext();

            //if (%avoid = %vehicle)
               //%avoid = containerSearchNext();

            //if we are moving slow and close to objects we are probably stuck
            //if (%avoid)
            //{
               %vx = getWord(%vehicle.getWorldBoxCenter(), 0);
               %vy = getWord(%vehicle.getWorldBoxCenter(), 1);
               %vz = getWord(%vehicle.getWorldBoxCenter(), 2);
               %vz += 2.5;
               %vR = getWords(%vehicle.getTransform(), 3, 6);

               //%client.setPilotDestination(%location, 1.0);//max speed
               %vehicle.setTransform(%vx SPC %vy SPC %vz SPC %vR);// --- set the Transform Here up a little

               error("AITouchObject Vehicle Avoidance - STUCK " @ getTaggedString(%client.name));

               //return;//i think we need this here
            //}
         }

         //----------------------------------------------------------------check to see if we are close to home...

         %pos2D = getWord(%vehicle.getWorldBoxCenter(), 0) SPC getWord(%vehicle.getWorldBoxCenter(), 1) SPC "0";
         %dest2D = getWord(%touchPos, 0) SPC getWord(%touchPos, 1) SPC "0";
         //if (VectorDist(%pos2D, %dest2D) < 100)
         if (VectorDist(%vehicle.getWorldBoxCenter(), %touchPos) < 100)
            AIDisembarkVehicle(%client);      

         //if in a land rover just drive it.
         if (%type $= "ScoutVehicle" || %type $= "AssaultVehicle")
            %client.setPilotDestination(%touchPos, 1);// --- max speed
         else
         {
            //---------------------------------------------------------------- we have to use some form of navigation here...

            %dest = getWord(%touchPos, 0) SPC getWord(%touchPos, 1) SPC getWord(%touchPos, 2) + 100;

            // - if can't see location marker gain height
            %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType;
            %hasLOS = !containerRayCast(%vehicle.getWorldBoxCenter, %dest, %mask, 0);//------- %hasLOS to %task.location ?
            //%hasLOS = aiCollisionCheck(%vehicle, 150);//------- %hasLOS to %task.location ?

            //if we get blocked fly straight up a little (only if we are going to hit something)
            //if (%myZ < (nameToId("MissionArea").flightCeiling - nameToId("MissionArea").flightCeilingRange) &&
            //((!%hasLOS || %collision) || %alt < 25))
            if ((%myZ < MissionArea.flightCeiling - MissionArea.flightCeilingRange) &&
              (%collision || %alt < 25))
            {
               //for shrikes and havocs
               if (%type $= "ScoutFlyer" || %type $= "HAPCFlyer")
               {
                  applyKick(%vehicle, "up", %skill * 5);
                  applyKick(%vehicle, "foward", %skill * 5, true);
                  //%client.setPilotPitchRange(-0.006, 0.0035, 0.005);
                  %client.setPilotPitchRange(-0.006, 0.0035, 0.01);
                  %client.setPilotDestination(%dest, 1);//Max Speed
               }
               //reversed for bombers
               else
               {
                  //%client.setPilotPitchRange(-0.01, 0.001, 0.05);
                  //applyKick(%vehicle, "down", 0.5, true);
                  //applyKick(%vehicle, "foward", %skill * 5, true);
                  //%client.setPilotDestination(%dest, 1);//Max Speed

                  %x = firstWord(%vehicle.getWorldBoxCenter());
                  %y = getWord(%vehicle.getWorldBoxCenter(), 1);
                  %z = getWord(%vehicle.getWorldBoxCenter(), 2);
                  %client.setPilotPitchRange(-0.005, 0.0045, 0.005);//real smooth setting here
                  %client.setPilotDestination(%x SPC %y SPC %z + 10, 1);// -- if we can't see %task.location move straight up
               } 

               //echo("blocked view or collision, - pitch up");
            }
            //else fly straight to home flag
            else 
            {
               %client.setPilotPitchRange(-0.005, 0.0045, 0.005);//real smooth setting here
               %client.setPilotDestination(%dest, 1);//Max Speed

               //echo("flying straight home");
            }
         }
      }
      //-------------------------------------------------------------------------------------------- end vehicle stuff
   }
   //if siege game we want a special way of capturing the flip flop - Lagg... 9-11-2003
   else if (%task.mode $= "TouchFlipFlop" && $CurrentMissionType $= "Siege")
   {

      //if there is a force field blocking the way - Lagg... 9-11-2003
      InitContainerRadiusSearch(%touchPos, 12, $TypeMasks::ForceFieldObjectType);
      %objSearch = containerSearchNext();
      if (%objSearch > 0 && %objSearch.isPowered())//should we check for all pass force fields? - Lagg ... (nah)
      {
         if (%task == %client.objectiveTask)
         {
            AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	    return;
         }
      }
      else
      {
		//don't move like a maniac when near lava - roberto
		if (%client.inDanger) { %client.stepMove(%touchPos, 0.25, $AIModeWalk); }
		else { %client.stepMove(%touchPos, 0.25, $AIModeExpress); }
      }
   }
	 else if (%task.followPath && %task.mode $= "FlagGrab") //follow the path to the flag - roberto
	{
		 //error("AITouchObject::monitor - Following PathToFlag - " @ getTaggedString(%client.name));

		 //set the path and follow it to the end
         if (%client.getPathDistance(%task.targetObject.position) < 50)
          {
            %task.followPath = false;
            %client.lastIndexPathToFlag = -1;
            %client.lastPathToFlag = -1;
            return;
          }
         
		 %task.spot = %task.group.getObject(%task.locationIndex).position;
		 //echo("We are now heading todards the marker " @ %task.locationindex);

		 %pos = %client.player.position;
		 %dest = %task.spot;
       %distToMarker = VectorDist(%dest, %pos);

		 //are we close to location index marker?
		 if (%distToMarker < 35) //be less accurate since Scouts are faster
		 {
		 	//echo("We reached a marker. Distance is: " @ VectorDist(%dest, %pos));
		    //if we have another location index
		    if ((%task.count - 1) > %task.locationIndex)//has to be -1 on the groups count
		    {
		       %task.locationIndex++;
		       %client.lastIndexPathToFlag = %task.locationIndex;
		       //error("AITouchObject::monitor - lastIndexPathToFlag is " @ %client.lastIndexPathToFlag);
		    }
		    //else we are at end of trail
		    else
		    {
		    	//echo("We reached the path end");
				//if this task is the objective task, choose a new objective
				//if (%task == %client.objectiveTask)
				//{
					%task.followPath = false;
					%client.lastIndexPathToFlag = -1;
					%client.lastPathToFlag = -1;
					//echo("We stopped following the path.");
					return;
				//}
		    }
		 }
		 else //move to the marker
		 {
			//don't move like a maniac when near lava - roberto
			if (%client.inDanger) { %client.stepMove(%task.spot, 0.25, $AIModeWalk); }
			else { %client.stepMove(%task.spot, 0.25, $AIModeExpress); }
		 } 
	}
   else  // no path to follow, just move to the flag
   {
		//don't move like a maniac when near lava - roberto
		if (%client.inDanger) { %client.stepMove(%touchPos, 0.25, $AIModeWalk); }
		else { %client.stepMove(%touchPos, 0.25, $AIModeExpress); }
      //echo("CAPPER: " @ %client.nameBase @ " Simply moving to flag, no specific instructions.");
      //echo("CAPPER: " @ %client.nameBase @ "  moving to an object, NO BOOST.");
   }

   //if (VectorDist(%client.player.position, %touchPos) < 10)
   //{
      //dissolve the human control link
    //  if (%task == %client.objectiveTask)
    //  {
    //     if (aiHumanHasControl(%task.issuedByClient, %client))
	//	 {
	//	    aiReleaseHumanControl(%client.controlByHuman, %client);

		    //should re-evaluate the current objective weight
	//	    %inventoryStr = AIFindClosestInventories(%client);
	//	    %client.objectiveWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
	//	 }
    //  }
      
   //see if we're supposed to be engaging anyone...
   if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
   {
   	  %flagIdleCanFight = true; //by defauly fight back anyone

   	  if (%client.player.holdingFlag) //if we have the flag only attack enemies that are close to our homeflag original position so we don't go away from oru base
   	  {
		%homeFlagOrigPos = $AITeamFlag[%client.team].originalPosition;  //original position of our home flag
		%teamFlagDistToEnemy = VectorDist(%homeFlagOrigPos, %client.shouldEngage.position);

   	  	 if (%teamFlagDistToEnemy < 75)
   	  	 {
   	  	 	%flagIdleCanFight = true;
   	  	 }
   	  	 else
   	  	 {
   	  	 	%flagIdleCanFight = false;
   	  	 }
   	  }

   	  if (%flagIdleCanFight)
   	  {
   	  	%client.setEngageTarget(%client.shouldEngage); //only be able to fight back if we don't want to go fast to our base

   	  	if (%client.player.holdingFlag)
   	  	{
   	  		//echo("CAPPER " @ %client.nameBase @ " FLAG CAPTURE IDLING AND FIGHTING CLOSE ENEMY"); 
   	  	}
   	  	//echo("CAPPER: " @ %client.nameBase @ " We can fight with a client.");
   	  	//echo("AI Capper or Grabber, we will fight back.");

   	  }
   	  else
   	  {
   	  	if (%client.player.holdingFlag)
   	  	{
   	  		//echo("CAPPER " @ %client.nameBase @ " ENEMY IS TOO FAR FROM OUR FLAG TO FIGHT THEM"); 
   	  	}
   	  }
   }

   //clear the last damaged tag
   %client.lastDamageClient = -1;

	//%mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType;
	//%targetLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %task.targetObject.getWorldBoxCenter(), %mask, 0);

   %distToTargetObj = VectorDist(%client.player.position, %touchPos);

   //--------stop the capper from fighting turrets/players/etc. while they are close to grabbing, just get the fucking flag you idiot bot!------
   if (%task.mode $= "FlagGrab" || %task.mode $= "FlagCapture")
   {
      //%targetObjPos = %task.targetObject.getWorldBoxCenter();
      %targetObjPos = %touchPos;
      %isHealthy = false; //if the bot doesn't have enough health there's no point in grabbing the flag and dying 
      %hasLosToObj = false;
      %distGood = false;
      

      if (%task.mode $= "FlagCapture") //if we have the flag we don't need to be close to our home flag or see it to not fight, just move home and ignore everyone
      {
         if (!%canfight) //we are far from our home flag or we are  home but our flag isn't here, only now we can ignore stuff
         {
            %hasLosToObj = true;
            %distGood = true;
            %isHealthy = true;
         }
      }
      else
      {
         %distGood = (%distToTargetObj < 65);
         if (%distGood)
         {
			%hasLosToObj = true; //let's have the bot always see the object, so he doesn't fight when looking at markers, as a test

	        switch$ (%player.getArmorSize()) 
	        {
	            case "Light":
	                %isHealthy = (%player.getDamagePercent() <= 0.70);
	            case "Medium":
	                %isHealthy = (%player.getDamagePercent() <= 0.75);
	            case "Heavy":
	                %isHealthy = false; //heavies should not ignore danger even before capping so they can destroy some stuff
	        }
	    }
      }

      if (%hasLosToObj && %isHealthy && %distGood)
      {
         %client.capperGrabbingFlag = true;
         %client.lastDamageClient = -1;
         %client.lastDamageTurret = -1;
         %client.setEngageTarget(-1);
         %client.stepEngage(-1);
         %client.engageRemeq = -1;//added - Lagg...
         %client.pickUpItem = -1;//added - Lagg...

         //clear the flag with the spinfusor!
         if (%task.mode $= "FlagGrab" && !%client.clearedFlag)
         {
         	if (%distToTargetObj > 7.5 && %distToTargetObj < 27.5)
         	{
         		if (%client.player.getInventory("Disc") > 0 && %client.player.getInventory("DiscAmmo") > 0)
         		{
					%obstTypes = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType | $TypeMasks::ForceFieldObjectType;
					%targetObjPosElevated = getWord(%targetObjPos, 0) SPC getWord(%targetObjPos, 1) SPC getWord(%targetObjPos, 2) + 0.1; //0.1m above the flag 
					%muzzlePoint = %client.player.getMuzzlePoint($WeaponSlot);
					%hasLosToObj = !containerRayCast(%muzzlePoint, %targetObjPosElevated, %obstTypes, 0);
					%goodZpos = ( (getWord(%muzzlePoint, 2) - getWord(%targetObjPos, 2)) > 0.75);

					if (%hasLosToObj && %goodZpos)
					{
						%client.aimAt(%targetObjPos, 1500);
						//for some reason sometimes the bot's pressFire() gets cleared and he doesn't shoot...
						schedule(10, 0, "aiDiscClearFlag", %client);
						%client.clearedFlag = true;
					}
				}
			}
         }
      } 
      else //don't ignore danger, we aren't that close to the flag or we don't have it
      {
         %client.capperGrabbingFlag = false; 
      }
   }
   else //we aren't in any modes that would make us want to ignore danger
   {
      %client.capperGrabbingFlag = false; 
   }
   //-----------------------------------------------------------------------------------------------------------

   if (%distToTargetObj < 10) //we are close to our target object flag/flip flop etc
   {
		%task.followPath = false; 
		%player = %client.player;

		//dissolve the human control link
		if (%task == %client.objectiveTask)
		{
			if (aiHumanHasControl(%task.issuedByClient, %client))
			{
				aiReleaseHumanControl(%client.controlByHuman, %client);

				//should re-evaluate the current objective weight
				%inventoryStr = AIFindClosestInventories(%client);
				%client.objectiveWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
			}
		}		
	}
}


//--------------------------------------------------------------------------------------------------------------------------
//AI Objective functions

function ClientHasAffinity(%objective, %client)
{
   if (%objective.offense && %client.offense)
      return true;
   else if (%objective.defense && !%client.offense)
      return true;
   else
      return false;
}

//this function needs a bit of work, one would say - LaggAlot 8/22/09
function ClientHasRequiredEquipment(%objective, %client)
{
   return true;
}

function AIODefault::weight(%objective, %client, %level, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //set the base weight
   switch (%level)
   {
      case 1:
         %weight = %objective.weightLevel1;
      case 2:
         %weight = %objective.weightLevel2;
      case 3:
         %weight = %objective.weightLevel3;
      default:
         %weight = %objective.weightLevel4;
   }
   
   //check Affinity
   if (ClientHasAffinity(%objective, %client))
      %weight += 40;

   //if the objective doesn't require any equipment, it automatically get's the +100...
   if (%objective.equipment $= "" && %objective.desiredEquipment $= "")
      %weight += 100;
   else
   {
      //check equipment requirement
      %needEquipment = AINeedEquipment(%objective.equipment, %client);
	      
      //check Required equipment
      if (%objective.equipment !$= "" && !%needEquipment)
         %weight += 100;

      //figure out the percentage of desired equipment the bot has
      else if (%objective.desiredEquipment !$= "")
      {
         %count = getWordCount(%objective.desiredEquipment);
	 %itemCount = 0;
	 for (%i = 0; %i < %count; %i++)
	 {
	    %item = getWord(%objective.desiredEquipment, %i);
	    if (!AINeedEquipment(%item, %client))
	    %itemCount++;
	 }

	 //add to the weight
	 %weight += mFloor((%itemCount / %count) * 75);
      }
   }
      
   //find the distance to target
   if (%objective.targetClientId !$= "" || %objective.targetObjectId !$= "")
   {
      if (AIClientIsAlive(%objective.targetClientId))
      {   
         %targetPos = %objective.targetClientId.player.getWorldBoxCenter();
      }
      else if (VectorDist(%objective.location, "0 0 0") > 1)
         %targetPos = %objective.location;
      else
      {   
         if(%objective.targetObjectId > 0)
            %targetPos = %objective.targetObjectId.getWorldBoxCenter();
      }
   }

   //make sure the destination is accessible
   %distance = %client.getPathDistance(%targetPos);
   if (%distance < 0)
      return 0;

   %closestInvIsRemote = (getWordCount(%inventoryStr) == 4);
   %closestInv = getWord(%inventoryStr, 0);
   %closestDist = getWord(%inventoryStr, 1);
   %closestRemoteInv = getWord(%inventoryStr, 2);
   %closestRemoteDist = getWord(%inventoryStr, 3);
	
   //if we need equipment, the distance is from the client, to an inv, then to the target
   if (%needEquipment)
   {
      //echo("AIODefault::weight - %needArmor = " @ %needArmor SPC getTaggedString(%client.name));
      //if we need a regular inventory station, and one doesn't exist, exit

      %needArmor = AIMustUseRegularInvStation(%objective.equipment, %client);// - added here to fix below - Lagg... 3-10-2004

      if (!isObject(%closestInv) && %needArmor)// - we got an error here - %needArmor = "" (lets just review/redo the whole function) * fixed for now - Lagg... ***
      {
         //error("AIODefault::weight - %needArmor = True no invos" @ getTaggedString(%client.name));
         return 0;
      }

      //find the closest inv based on whether we require armor (from a regular inv station)
      if (!%closestInvIsRemote)
      {
         %needArmor = false;
	 %weightDist = %closestDist;
	 %weightInv = %closestInv;
      }
      else
      {
         //%needArmor = AIMustUseRegularInvStation(%objective.equipment, %client);// - removed here moved up ^ - Lagg... 3-10-2004
	 if (%needArmor)
	 {
	    %weightDist = %closestDist;
	    %weightInv = %closestInv;
	 }
	 else
	 {
	    %weightDist = %closestRemoteDist;
	    %weightInv = %closestRemoteInv;
	 }
      }

      //tested this next check to be true, now need to ad a search for items in each aiObjective :) - Lagg... 1-20-2004
      //if we don't need armor, and there's no inventory station, see if the equipment we need

      // ------------------------------------------------------------- have to modify to see what is closer item or inventory *

      //is something we can pick up off the ground (likely this would be a repair pack...)
      //updated AIODeployEquipment to consider picking up droped packs :) - Lagg... 3-10-2004

      //echo("AIODefault::weight - %weightDist = " @ %weightDist SPC getTaggedString(%client.name));
      if (%weightDist >= 32767)
      {
         
         %itemType = getWord(%objective.equipment, 0);
  	 %found = false;
  	 %itemCount = $AIItemSet.getCount();
  	 for (%i = 0; %i < %itemCount; %i++)
  	 {
  	    %item = $AIItemSet.getObject(%i);
  	    if (%item.getDataBlock().getName() $= %itemType && !%item.isHidden())
  	    {
	       %weightDist = %client.getPathDistance(%item.getWorldBoxCenter());
	       if (%weightDist > 0)
	       {
	          %weightInv = %item;  //set the var so the distance function will work...
	  	  %found = true;
	  	  break;
	       }
  	    }
  	 }

  	 if (! %found)
  	    return 0;
      }
 
      //now find the distance used for weighting the objective
      %tempDist = AIGetPathDistance(%targetPos, %weightInv.getWorldBoxCenter());
      if (%tempDist < 0)
         %tempDist = 32767;

      %distance = %weightDist + %tempDist;
   }

   // -------------------------------------------------- lets give him some personallity on this one :) - Lagg... 2-13-2004
   //if (%objective.targetObjectId > 0 && !%needEquipment)
   //{
      //if (VectorDist(%client.player.getWorldBoxCenter(), %targetPos) < 250)
      //{

         ////check for LOS
         //%mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType |
           //$TypeMasks::ForceFieldObjectType | $TypeMasks::VehicleObjectType;
         //%hasLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %objective.targetObjectId.getWorldBoxCenter(), %mask, 0);

         //if (%hasLOS)
         //{
            ////echo("AIODefault::weight - We Got LOS - " @ getTaggedString(%client.name));
            //%weight += 400;
         //}
      //}
   //}

   //are we the closest to inventory - Lagg... 4-9-2004
   %closestInv = AIFindClosestInventory(%client, false);
   %closestDist = getWord(%closestInv, 1);
   if (%closestDist < 1)
      %weight += 30;
   
   //see if we're within 200 m
   if (%distance < 200)
      %weight += 30;
      
   //see if we're within 90 m
   if (%distance < 90)
      %weight += 30;

   //see if we're within 45 m
   if (%distance < 45)
      %weight += 30;
      
   //see if we're within 20 m
   if (%distance < 20)
      %weight += 30;

   //if we get bonus for being close we should get penalty if far - Lagg... 2-15-2004
   //see if we're passed 450 m
   if (%distance >= 200)
      %weight -= 30;
      
   //see if we're pass 900 m
   if (%distance > 500)
      %weight -= 30;

   //see if we're pass 1350 m
   if (%distance > 1150)
      %weight -= 30;
      
   //see if we're pass 1600 m
   if (%distance > 1800)
      %weight -= 30;
   
   //final return, since we've made it through all the rest
   return %weight;
}

function AIODefault::QuickWeight(%objective, %client, %level, %minWeight)
{
   //can't do a quick weight when re-evaluating a client's current objective
   if (%client.objective == %objective)
      return true;

   //do a quick check to disqualify this objective if it can't meet the minimum weight
   switch (%level)
   {
      case 1:
         %testWeight = %objective.weightLevel1;
      case 2:
	 %testWeight = %objective.weightLevel2;
      case 3:
	 %testWeight = %objective.weightLevel3;
      default:
	 %testWeight = %objective.weightLevel4;
   }

   //if (%objective.getName() $= "AIOTouchObject")
   //{
      //error("AIODefault::QuickWeight - TestWeight = " @ (%testWeight + 260));
      //echo("AIODefault::QuickWeight - MinWeight = " @ %minWeight);
   //}

   if (%testWeight + 260 < %minWeight)
      return false;
   else
      return true;
}

//--------------------------------------------------------------
//AI Objective Q functions...

$ObjectiveClientsSet = 0;
function AIObjectiveFindClients(%objective)
{
   //create and clear the set
   if (! $ObjectiveClientsSet)
   {
      $ObjectiveClientsSet = new SimSet();
      MissionCleanup.add($ObjectiveClientSet);
   }
   $ObjectiveClientsSet.clear();
   
   %clientCount = 0;
   %count = ClientGroup.getCount();
   for(%i = 0; %i < %count; %i++)
   {
		%cl = ClientGroup.getObject(%i);
      if (%cl.objective == %objective)
         $ObjectiveClientsSet.add(%cl);
   }
   return $ObjectiveClientsSet.getCount();
}

function AIObjectiveGetClosestClient(%location, %team)
{
   if (%location $= "")
      return -1;
      
   if (%team $= "")
      %team = 0;
      
   %closestClient = -1;
   %closestDistance = 32767;
   %count = ClientGroup.getCount();
   for(%i = 0; %i < %count; %i++)
   {
		%cl = ClientGroup.getObject(%i);
      if (%cl.isAIControlled() && (%cl.team == %team || %team == 0) && AIClientIsAlive(%cl))
      {
         %testPos = %cl.player.getWorldBoxCenter();
         %distance = VectorDist(%location, %testPos);
         if (%distance < %closestDistance)
         {
            %closestDistance = %distance;
            %closestClient = %cl;
         }
      }
	}
   
   return %closestClient;
}

function AICountObjectives(%team)
{
	%objCount = 0;
	%count = $ObjectiveQ[%team].getCount();
	for (%i = 0; %i < %count; %i++)
	{
		%grp = $ObjectiveQ[%team].getObject(%i);
		if (%grp.getClassName() !$= "AIObjective")
			%objCount += %grp.getCount();
		else
			%objCount++;
	}
	error("DEBUG" SPC %team SPC "has" SPC %objCount SPC "objectives.");
}

function AIAddTableObjective(%objective, %weight, %level, %bump, %position)
{
	$objTable[%position, objective] = %objective;
	$objTable[%position, weight] = %weight;
	$objTable[%position, level] = %level;
	$objTable[%position, bump] = %bump;
	$objTableCount = %position + 1;

      //error("TAbleCount = " @ $objTableCount);
}

//-----------------------------------------------------------------------------------------------------------------------
//This function had me stumped for a long time. The use of the $Objective[Q] and also the $objectTable[] to choose
//objectives was the problem, when objectives are deleted from the $Objective[Q] they were still left in the $objTable[],
//the clients choose from the $objTable and so the objective that was deleted no longer exsisted while the client was
//trying to choose it.                                                                                   - Lagg... 1-27-2004
//DO NOT FOR ANY REASON cHANGE,REPLACE OR MODIFY THIS FUNCTION IN ANY WAY.
//ALL THE AIOBECTIVE SCRIPTS DEPEND ON IT!                                                               - Lagg... 5-29-2004

function AIChooseObjective(%client, %useThisObjectiveQ)
{
   //pick which objectiveQ to use, or use the default
   if (%useThisObjectiveQ <= 0 && %client.team < 0)
      return;

   if (%useThisObjectiveQ <= 0)
      %useThisObjectiveQ = $ObjectiveQ[%client.team];

   if (!isObject(%useThisObjectiveQ) || %useThisObjectiveQ.getCount() <= 0)
      return;

   //since most objectives check for inventory, find the closest inventory stations first
   %inventoryStr = AIFindClosestInventories(%client);

   //find the most appropriate objective
   if (!%client.objective)
   {
      //note, the table is never empty, during the course of this function
      AIAddTableObjective(0, 0, 0, 0, 0);
   }
   else
   {
      //should re-evaluate the current objective weight - but never decrease the weight!!!
      %testWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
      if (%testWeight <= 0 || %testWeight > %client.objectiveWeight)
         %client.objectiveWeight = %testWeight;

      if (%client.objectiveWeight > 0)
         AIAddTableObjective(%client.objective, %client.objectiveWeight, %client.objectiveLevel, 0, 0);
      else
         AIAddTableObjective(0, 0, 0, 0, 0);
   }

   %objCount = %useThisObjectiveQ.getCount();
   for (%i = 0; %i < %objCount; %i++)
   {
      %objective = %useThisObjectiveQ.getObject(%i);

      //don't re-evaluate the client's own
      if (%objective == %client.objective)
         continue;

      //try this objective at each of the 4 weight levels to see if it is weighted higher
      for (%level = 1; %level <= 4; %level++)
      {
         %minWeight = 0;
	 %bumpWeight = 0;
         %bumpClient = "";

         //we can bump clients off the objective for the first three levels
         if (%level <= 3)
         {
            //if the objective is part of a group, check the whole group
	    if (%objective.group > 0)
            {
               %bumpClient = %objective.group.clientLevel[%level];
               %bumpWeight = %bumpClient.objectiveWeight;
            }
            else
            {
               %bumpClient = %objective.clientLevel[%level];
               %bumpWeight = %bumpClient.objectiveWeight;
            }
         }

         //find the minimum weight the objective must have to be considered
         %minWeight = (%bumpWeight > $objTable[0, weight] ? %bumpWeight : $objTable[0, weight]);

         //evaluate the weight
	 %weight = %objective.weight(%client, %level, %minWeight, %inventoryStr);

	 //make sure we got a valid weight
	 if (%weight <= 0)
	    break;

         //if it's the highest so far, it now replaces anything else in the table
         if (%weight > $objTable[0, weight])
         {
            //never bump someone unless you out- weight them
            if (%weight > %bumpWeight)
            {
               AIAddTableObjective(%objective, %weight, %level, %bumpClient, 0);

               //no need to keep checking the other levels
	       break;
            }
         }

         //else if it's equal to the highest objective we've seen so far, and higher from our current objective
         else if (%weight == $objTable[0, weight] && %weight > %client.objectiveWeight)
         {
            //never bump someone unless you outweigh them
            if (%weight > %bumpWeight)
            {
	       //if this wouldn't require us to bump someone, or the table is empty
               if (%bumpWeight <= 0 || $objTable[0, weight] <= 0)
               {
						//if the table currently contains objectives which would require bumping someone, clear it
						if ($objTable[0, bump] > 0)
							%position = 0;
						else
							%position = $objTableCount;

						//add it to the table
						AIAddTableObjective(%objective, %weight, %level, %bumpClient, %position);

						//no need to keep checking the other levels
						break;
					}

					//otherwise, the table is not empty, and this would require us to bump someone
					//only add it if everything else in the table would also require us to bump someone
					else if ($objTable[0, bump] > 0)
					{
						AIAddTableObjective(%objective, %weight, %level, %bumpClient, $objTableCount);

						//no need to keep checking the other levels
						break;
					}
				}
			}

			//else it must have been less than our highest objective so far- again, no need to keep checking other levels...
			else
				break;
		}
	}

	//if we have a table of possible objectives which are higher than our current- choose one at random
	if ($objTableCount > 0 && $objTable[0, objective] != %client.objective)
	{
		//choose the new one
		%index = mFloor(getRandom() * ($objTableCount - 0.01));

		//clear the old objective
      if (%client.objective)
      {
         if (%client.objectiveLevel <= 3)
			{
				if (%client.objective.group > 0)
				{
					if (%client.objective.group.clientLevel[%client.objectiveLevel] == %client)
						%client.objective.group.clientLevel[%client.objectiveLevel] = "";
				}
				else
				{
					if (%client.objective.clientLevel[%client.objectiveLevel] == %client)
		            %client.objective.clientLevel[%client.objectiveLevel] = "";
				}
			}
         %client.objective.unassignClient(%client);
      }
		
		//assign the new
      %chooseObjective = $objTable[%index, objective];
      %client.objective = %chooseObjective;
      %client.objectiveWeight = $objTable[%index, weight];
      %client.objectiveLevel = $objTable[%index, level];
		if (%client.objectiveLevel <= 3)
		{
			if (%chooseObjective.group > 0)
		      %chooseObjective.group.clientLevel[%client.objectiveLevel] = %client;
			else
		      %chooseObjective.clientLevel[%client.objectiveLevel] = %client;
		}
      //error("%chooseObjective.assignClient = " @ %chooseObjective.getName());

      //make sure we still have a valid objective - Lagg... 1-27-2004
      if (isObject(%chooseObjective))
        %chooseObjective.assignClient(%client);
      else
      {
        //error("AIChooseObjective - Failed please choose again :) " @ getTaggedString(%client.name));
        AIChooseObjective(%client, %useThisObjectiveQ);//if failed choose again - Lagg... 1-27-2004
      }

      //see if this objective needs to be acknowledged
      if (%chooseObjective.shouldAcknowledge && %chooseObjective.issuedByHuman && isObject(%chooseObjective.issuedByClientId))
      {
         //cancel any pending acknowledgements - a bot probably just got bumped off this objective
         cancel(%chooseObjective.ackSchedule);
         %chooseObjective.ackSchedule = schedule(5500, %chooseObjective, "AIAcknowledgeObjective", %client, %chooseObjective);
      }

      //if we had to bump someone off this objective, 
		%bumpClient = $objTable[%index, bump];
      if (%bumpClient > 0)
      {
         //unassign the bumped client and choose a new objective
			AIUnassignClient(%bumpClient);
         Game.AIChooseGameObjective(%bumpClient);
      }
	}

   //debuging - refresh aidebugq() if required
   //if ($AIDebugTeam >= 0)
   //   aiDebugQ($AIDebugTeam);
}

//new function to clean up the mess they made with the aiChooseObjective code above
//this gets called when ever AIObjectives need to be deleted from the $Objective[Q] - Lagg... 1-27-2004
function clearObjectiveFromTable(%objective)
{
   for(%i = 0; %i < $objTableCount; %i ++)
   {
      if ($objTable[%i, objective] = %objective)
      {
         $objTable[%i, objective] = 0;
	 $objTable[%i, weight] = 0;
	 $objTable[%i, level] = 0;
	 $objTable[%i, bump] = 0;

         break;
      }
   }
}

function AIAcknowledgeObjective(%client, %objective)
{
   %objective.shouldAcknowledge = false;
   //make sure the client is still assigned to this objective
   if (%client.objective == %objective)
	   serverCmdAcceptTask(%client, %objective.issuedByClientId, -1, %objective.ackDescription);
}

function AIForceObjective(%client, %newObjective, %useWeight)
{
   //if we found a new objective, release the old, and assign the new
   if (%newObjective && %newObjective != %client.objective)
   {
      //see if someone was already assigned to this objective
		if (%newObjective.group > 0)
	      %prevClient = newObjective.group.clientLevel[1];
		else
	      %prevClient = newObjective.clientLevel[1];
      if (%prevClient > 0)
         AIUnassignClient(%prevClient);

		//see if we should override the weight
		if (%useWeight < %newObjective.weightLevel1)
			%useWeight = %newObjective.weightLevel1;
      
      //release the client, and force the assignment
      AIUnassignClient(%client);
      %client.objective = %newObjective;
      %client.objectiveWeight = %useWeight;
      %client.objectiveLevel = 1;
		if (%newObjective.group > 0)
	      %newObjective.group.clientLevel[1] = %client;
		else
	      %newObjective.clientLevel[1] = %client;
		%newObjective.forceClientId = %client;
      %newObjective.assignClient(%client);

      //don't acknowledge anything that's been forced...
      %newObjective.shouldAcknowledge = false;
      
      //now reassign the prev client
      if (%prevClient)
         Game.AIChooseGameObjective(%prevClient);
   }
   
   //debuging - refresh aidebugq() if required
   //if ($AIDebugTeam >= 0)
    // aiDebugQ($AIDebugTeam);
}

function AIUnassignClient(%client)
{
	//first, dissolve any link with a human
	aiReleaseHumanControl(%client.controlByHuman, %client);

   if (%client.objective)
   {
      if (%client.objectiveLevel <= 3)
		{
			if (%client.objective.group > 0)
			{
				//make sure the clientLevel was actually this client
				if (%client.objective.group.clientLevel[%client.objectiveLevel] == %client)
					%client.objective.group.clientLevel[%client.objectiveLevel] = "";
			}
			else
			{
				if (%client.objective.clientLevel[%client.objectiveLevel] == %client)
		         %client.objective.clientLevel[%client.objectiveLevel] = "";
			}
		}
      %client.objective.unassignClient(%client);
      %client.objective = "";
		%client.objectiveWeight = 0;
   }
   
   //debuging - refresh aidebugq() if required
   //if ($AIDebugTeam >= 0)
      //aiDebugQ($AIDebugTeam);
}
  
function AIClearObjective(%objective)
{
   %count = ClientGroup.getCount();
   for(%i = 0; %i < %count; %i++)
   {
		%cl = ClientGroup.getObject(%i);
      if (%cl.objective == %objective)
         AIUnassignClient(%cl);
   }
   
   //debuging - refresh aidebugq() if required
   //if ($AIDebugTeam >= 0)
      //aiDebugQ($AIDebugTeam);
}

//New function to disk jump away from enemy base whenever possible - Roberto
//It's pretty shit since I did not use any fancy math calculations and it's my first script, feel free to improve it. 
//FastCappers is much better so I wouldn't recommend using this
function AITryDiscJumpFlag(%client)
{
    if (!%client.discJumpTimerStarted) //only try to disc jump if we didn't already had an alarm set for it
    { 
        %player = %client.player;
        %ourHomeFlag = $AITeamFlag[%client.team]; //our home flag object
        %homeFlagPos = %ourHomeFlag.originalPosition;  //position of our home flag
        %ourCurrPosition = %player.position; 
        %altitudeGood = (getWord(%player.position, 2) - getTerrainHeight(%player.position) <= 2);
        
        %hasAmmoHasDisc = (%player.getInventory("Disc") > 0 && %player.getInventory("DiscAmmo") >= 1);
        %isHealthy = false;
        %farEnough = (VectorDist(%ourCurrPosition, %homeFlagPos) > 100); //no need to DJ if we're close to our home flag
        %obsTypes = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType | $TypeMasks::ForceFieldObjectType;
        %flagLosOk = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %homeFlagPos, %obsTypes, 0);
        
        //depending on our armor, pack, and health, return if we are able to disc jump
        switch$ (%player.getArmorSize())
        {
            case "Light":
                %isHealthy = (%player.getDamagePercent() <= 0.20 || (%player.getEnergyPercent() >= 0.6 && %player.getInventory("ShieldPack") >= 1));
            case "Medium":
                %isHealthy = (%player.getDamagePercent() <= 0.28 || (%player.getEnergyPercent() >= 0.6 && %player.getInventory("ShieldPack") >= 1));
            case "Heavy":
                %isHealthy = (%player.getDamagePercent() <= 0.40 || (%player.getEnergyPercent() >= 0.5 && %player.getInventory("ShieldPack") >= 1));
        }
        
        //if we are healthy or we have the shield pack, or good armor, and we did not enter a base (must use zone triggers), we are not flying, we can discjump
        if (%hasAmmoHasDisc && %isHealthy && %flagLosOk && %altitudeGood && %farEnough)
        {
            //echo("Flag DiscJump: We are ready to disc jump, aiming behind us now.");
            
            //get our XYZ position
            %ourx = getWord(%ourCurrPosition, 0);
            %oury = getWord(%ourCurrPosition, 1);
            %ourz = getWord(%ourCurrPosition, 2);

            //get the flag XYZ position
            %flagx = getWord(%homeFlagPos, 0);
            %flagy = getWord(%homeFlagPos, 1);
            %flagz = getWord(%homeFlagPos, 2);

            %horShootDist = 0.25; //shoot X metters backwards from our position 
            %verShootDist = -800; //shoot X metters upwards/downwards from our position 

            //this will determine if we add to our X or Y or not
            %addFactorX = 1; 
            %addFactorY = 1;

            //if we are to the right, the aim position will be even more to the right
            %add_x = (%ourx - %flagx) >= 0; 
            %add_y = (%oury - %flagy) >= 0;
            if (%add_x)
            {
                %addFactorX = 1;
            }
            else
            {
                %addFactorX = -1;
            }

            if (%add_y)
            {
                %addFactorY = 1;
            }
            else
            {
                %addFactorY = -1;
            }

            //if our X or Y position is bigger than flag x position, we add more X or Y to our position so we shoot back, this will propel us towards the flag
            %discjumpXlook = %ourx + (%horShootDist * %addFactorX);
            %discjumpYlook = %oury + (%horShootDist * %addFactorY);
            %discjumpZlook = %ourz + %verShootDist;

            %discFinalPosition = %discjumpXlook SPC %discjumpYlook SPC %discjumpZlook;
            
            //use the shield pack first if we have it
            if (%client.player.getInventory("ShieldPack") >= 1)
            {
                %client.player.use("ShieldPack");
            }

            //now turn backwards then shoot
            %client.stepEngage(-1);
            %client.lastDamageClient = -1;
            %client.lastDamageTurret = -1;
            %client.setEngageTarget(-1);
            %client.engageRemeq = -1;
            %client.pickUpItem = -1;    
            %client.aimAt(%discFinalPosition, 2500);
            %client.player.use("Disc");
            %client.discJumpTimerStarted = true; //the AI is set to press fire in X frames, because we don't want to shoot before aiming (bot aims slower than shooting)
        }
    }
    else     //disc jump timer is started, decrease it
    {
        %client.discJumpTimer -= 60; //decrease timer by this number every time the function is called 
    }
}

//will do the actual disc jump
function AIDoDiscJump(%client)
{
   if (%client.discJumpTimer <= 0)
   {
        %player = %client.player;
       //only jump if we have good altitude
        %goodAlt = (getWord(%player.position, 2) - getTerrainHeight(%player.position) <= 1.25);
        %obsTypes = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::TSStaticShapeObjectType | $TypeMasks::ForceFieldObjectType;
        %flagLosOk = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), $AITeamFlag[%client.team].originalPosition, %obsTypes, 0);
       
       if (%goodAlt && %flagLosOk)
       {
            %client.stepEngage(-1);
            %client.lastDamageClient = -1;
            %client.lastDamageTurret = -1;
            %client.setEngageTarget(-1);
            %client.engageRemeq = -1;
            %client.pickUpItem = -1;
            %player.use("Disc");
            %client.pressFire(5); //
            %client.discJumpTimer = 100;
            %client.discJumpTimerStarted = false; //stop decreasing the disc jump timer
            //echo("DiscJump: Woooo, successful discjump!");
            return 1; //return success
       }
       else
       {
           %client.discJumpTimer = 100;
           %client.discJumpTimerStarted = false; //stop decreasing the disc jump timer
           return 0;
       }
   }
    else
    {
        return 0; //we did not disc jump
    }
}

//shoot enemy flagstand with spinfusor
function aiDiscClearFlag(%client)
{
   if (isObject(%client) && AIClientIsAlive(%client))
   {

        %targetFlagTeam = 1;

        if (%client.team == 1)
        {
        	%targetFlagTeam = 2;
        }
        else
        {
        	%targetFlagTeam = 1;
        }

	   if ( %client.player.getInventory("Disc") > 0 && %client.player.getInventory("DiscAmmo") > 0 && isObject($AITeamFlag[%targetFlagTeam]) )
	   {
	        %client.stepEngage(-1);
	        %client.lastDamageClient = -1;
	        %client.lastDamageTurret = -1;
	        %client.setEngageTarget(-1);
	        %client.engageRemeq = -1;
	        %client.pickUpItem = -1;

	        %client.player.use("Disc");
			//%enFlagPos = $AITeamFlag[%targetFlagTeam].originalPosition; 
		    //%client.player.use("Disc");
			//%client.aimAt(%enFlagPos, 1250);
		    %client.pressFire(1); //why doesn't the bot always shoot?
        	//echo(%client.nameBase @ " cleared the enemy flagstand with the spinfusor!");
	   }
   }
}

//extra checks mainly for siege to prevent bots from doing objectives - roberto
function aiExtraWeightCheck(%objective, %client)
{
   %finalWeight = -1; //the weight check for the objective will proceed as usual if -1

   //check for 8 objects in total
   for (%i = 0; %i < 8; %i++)
   {
      %emptyCount = 3;
      //(%targetObjectId.getGroup().powerCount <= 0)

      if (%objective.isDisabled !$= "" && %objective.isDisabled !$= " ")
      {
         %currObjName = getWord(%objective.isDisabled, %i);

         if (%currObjName !$= " ")
         {
            %currObjId = nameToId(%currObjName);

            if ( isObject(%currObjId) )
            {
               if ( %currObjId.getClassName() !$= "SimGroup" )
               {
                  if ( %currObjId.isEnabled() ) //false if anything here is enabled, can't do objective
                  {
                     %finalWeight = 0;
                     break;
                  }
                  // else
                  // {
                  //    echo(%client.nameBase @ " : will do objective because all required OBJECTS are DISABLED.");
                  // }
               }
               else
               {
                  if ( %currObjId.powerCount > 0 )
                  {
                     %finalWeight = 0;
                     break;
                  }
                  // else
                  // {
                  //    echo(%client.nameBase @ " : will do objective because all required SIMGROUPS are NOT POWERED.");
                  // }
               }
            }
         }
         else
         {
            %emptyCount--;
         }
      }
      else
      {
         %emptyCount--;
      }
      //------------------

      if (%objective.isEnabled !$= "" && %objective.isEnabled !$= " ")
      {
         %currObjName = getWord(%objective.isEnabled, %i);

         if (%currObjName !$= " ")
         {
            %currObjId = nameToId(%currObjName);

            if ( isObject(%currObjId) )
            {
               if ( %currObjId.getClassName() !$= "SimGroup" )
               {
                  if ( !%currObjId.isEnabled() ) //false if anything here is disabled, can't do objective
                  {
                     %finalWeight = 0;
                     break;
                  }
               }
               else
               {
                  if ( %currObjId.powerCount <= 0 )
                  {
                     %finalWeight = 0;
                     break;
                  }
               }
            }
         }
         else
         {
            %emptyCount--;
         }
      }
      else
      {
         %emptyCount--;
      }
      //------------------

      if (%objective.isSameTeam !$= "" && %objective.isSameTeam !$= " ")
      {
         %currObjName = getWord(%objective.isSameTeam, %i);

         if (%currObjName !$= " ")
         {
            if ( nameToId(%currObjName).team != %client.team) //can't do objective if anything here is not in the same team as the bot
            {
               //echo("[-isSameTeam-] - One of the objects is not in the same team. Setting weight to 0.");
               %finalWeight = 0;
               break;
            }
         }
         else
         {
            %emptyCount--;
         }
      }
      else
      {
         %emptyCount--;
      }

      //stop loop if there's nothing written in any field
      if (%emptyCount <= 0)
      {
         break;
      }
   }

   return %finalWeight;
}
